Package: scf
Title: Analyzing the Survey of Consumer Finances
Version: 1.0.10
Authors@R: 
    c(person("Joseph","Cohen", role = c("aut","cre"),
             email = "joseph.cohen@qc.cuny.edu"))
Description: Analyze public-use micro data from the Survey of Consumer Finances.
  Provides tools to download prepared data files, construct replicate-weighted
  multiply imputed survey designs, compute descriptive statistics and model
  estimates, and produce plots and tables. Methods follow design-based inference
  for complex surveys and pooling across multiple imputations. See the package
  website and the code book for background.
License: MIT + file LICENSE
URL: https://github.com/jncohen/scf
BugReports: https://github.com/jncohen/scf/issues
Depends: R (>= 3.6)
Imports: ggplot2, haven, httr, rlang, stats, survey, utils, quantreg
Suggests: dplyr, hexbin, kableExtra, knitr, mitools, rmarkdown,
        testthat (>= 3.0.0)
VignetteBuilder: knitr
Config/testthat/edition: 3
Encoding: UTF-8
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-05-20 17:18:48 UTC; jcohen
Author: Joseph Cohen [aut, cre]
Maintainer: Joseph Cohen <joseph.cohen@qc.cuny.edu>
Repository: CRAN
Date/Publication: 2026-05-21 07:20:02 UTC
Built: R 4.4.3; ; 2026-05-24 23:56:05 UTC; unix

Package: scf
Title: Analyzing the Survey of Consumer Finances
Version: 1.0.10
Authors@R: 
    c(person("Joseph","Cohen", role = c("aut","cre"),
             email = "joseph.cohen@qc.cuny.edu"))
Description: Analyze public-use micro data from the Survey of Consumer Finances.
  Provides tools to download prepared data files, construct replicate-weighted
  multiply imputed survey designs, compute descriptive statistics and model
  estimates, and produce plots and tables. Methods follow design-based inference
  for complex surveys and pooling across multiple imputations. See the package
  website and the code book for background.
License: MIT + file LICENSE
URL: https://github.com/jncohen/scf
BugReports: https://github.com/jncohen/scf/issues
Depends: R (>= 3.6)
Imports: ggplot2, haven, httr, rlang, stats, survey, utils, quantreg
Suggests: dplyr, hexbin, kableExtra, knitr, mitools, rmarkdown,
        testthat (>= 3.0.0)
VignetteBuilder: knitr
Config/testthat/edition: 3
Encoding: UTF-8
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-05-20 17:18:48 UTC; jcohen
Author: Joseph Cohen [aut, cre]
Maintainer: Joseph Cohen <joseph.cohen@qc.cuny.edu>
Repository: CRAN
Date/Publication: 2026-05-21 07:20:02 UTC
Built: R 4.6.0; ; 2026-05-22 00:24:19 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(AIC,scf_model_result)
S3method(SE,scf_MIresult)
S3method(coef,scf_MIresult)
S3method(coef,scf_model_result)
S3method(formula,scf_model_result)
S3method(predict,scf_model_result)
S3method(print,scf_corr)
S3method(print,scf_freq)
S3method(print,scf_glm)
S3method(print,scf_logit)
S3method(print,scf_mean)
S3method(print,scf_median)
S3method(print,scf_mi_survey)
S3method(print,scf_ols)
S3method(print,scf_percentile)
S3method(print,scf_prop_test)
S3method(print,scf_quantreg)
S3method(print,scf_ttest)
S3method(print,scf_xtab)
S3method(residuals,scf_model_result)
S3method(scf_deflate,default)
S3method(scf_deflate,scf_mean)
S3method(scf_deflate,scf_median)
S3method(scf_deflate,scf_percentile)
S3method(scf_deflate,scf_ttest)
S3method(summary,scf_corr)
S3method(summary,scf_freq)
S3method(summary,scf_glm)
S3method(summary,scf_logit)
S3method(summary,scf_mean)
S3method(summary,scf_median)
S3method(summary,scf_ols)
S3method(summary,scf_percentile)
S3method(summary,scf_quantreg)
S3method(summary,scf_ttest)
S3method(summary,scf_xtab)
S3method(vcov,scf_MIresult)
S3method(vcov,scf_model_result)
export(SE)
export(scf_MIcombine)
export(scf_activate_theme)
export(scf_corr)
export(scf_deflate)
export(scf_design)
export(scf_download)
export(scf_freq)
export(scf_glm)
export(scf_implicates)
export(scf_load)
export(scf_logit)
export(scf_mean)
export(scf_median)
export(scf_ols)
export(scf_pctile_cut)
export(scf_pctile_sum)
export(scf_percentile)
export(scf_plot_bbar)
export(scf_plot_cbar)
export(scf_plot_dbar)
export(scf_plot_dist)
export(scf_plot_hex)
export(scf_plot_hist)
export(scf_plot_smooth)
export(scf_prop_test)
export(scf_quantreg)
export(scf_regtable)
export(scf_subset)
export(scf_theme)
export(scf_ttest)
export(scf_update)
export(scf_update_by_implicate)
export(scf_xtab)
import(ggplot2)
importFrom(rlang,.data)
importFrom(stats,AIC)
importFrom(stats,IQR)
importFrom(stats,aggregate)
importFrom(stats,as.formula)
importFrom(stats,ave)
importFrom(stats,binomial)
importFrom(stats,coef)
importFrom(stats,confint)
importFrom(stats,deviance)
importFrom(stats,family)
importFrom(stats,formula)
importFrom(stats,pnorm)
importFrom(stats,predict)
importFrom(stats,pt)
importFrom(stats,qnorm)
importFrom(stats,qt)
importFrom(stats,residuals)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,var)
importFrom(stats,vcov)
importFrom(stats,weighted.mean)
importFrom(stats,weights)
importFrom(stats,xtabs)
importFrom(survey,SE)
importFrom(survey,cv)
importFrom(survey,withReplicates)
importFrom(utils,head)
importFrom(utils,tail)
importFrom(utils,unzip)
importFrom(utils,write.csv)

SCF: An R Package for Analyzing the Survey of Consumer Finances
================

[![License:
MIT](https://img.shields.io/badge/license-MIT-blue.svg)](https://opensource.org/licenses/MIT)
[![R
version](https://img.shields.io/badge/R-%3E%3D%203.6-blue.svg)](https://CRAN.R-project.org/)
[![Lifecycle:
stable](https://img.shields.io/badge/lifecycle-stable-blue.svg)](https://lifecycle.r-lib.org/articles/stages.html)
[![CRAN
status](https://www.r-pkg.org/badges/version/scf)](https://CRAN.R-project.org/package=scf)

## Overview

The `scf` R package provides a structured, reproducible, and
pedagogically-conscious toolkit for analyzing the U.S. Federal Reserve’s
**Survey of Consumer Finances (SCF)**, one of the highest-quality data
sources for information on U.S. households’ balance sheets and income
statements.

It wraps replicate-weighted, multiply-imputed SCF data into a custom
data object (`scf_mi_survey`) with which users can implement custom
easy-to-use functions for generating proper population estimates for
descriptive statistics, hypothesis testing, regression
modeling, and high-quality visualizations.

## Table of Contents

- [Features](#features)
- [Installation](#installation)
- [Getting Started](#getting-started)
- [Documentation](#documentation)

## Features

### Data Preparation

- `scf_download()`: Downloads and preprocesses SCF microdata, including
  all five implicates and 999 replicate weights.
- `scf_load()`: Loads `.rds` files into structured `scf_mi_survey`
  objects ready for analysis.
- `scf_update()`: Adds or transforms variables uniformly across all implicates.
- `scf_update_by_implicate()`: Applies a user-defined transformation to
  each implicate’s data frame separately. Use when a computation depends
  on the within-implicate distribution (e.g., implicate-specific ranks
  or percentile thresholds).
- `scf_subset()`: Subsets the data consistently across all implicates.

### Descriptive Statistics

- `scf_freq()`: Weighted frequency tables for categorical variables.
- `scf_xtab()`: Cross-tabulations by row, column, or cell percentages.
- `scf_mean()`, `scf_median()`, `scf_percentile()`: Computes groupwise
  or overall statistics using Rubin’s Rules or a commensurate methodology.
- `scf_pctile_sum()`: Creates percentile-based grouping variables for a
  continuous variable and optionally computes a summary statistic within
  each group. Supports an implicate-specific survey-weighted method
  (default) and the Federal Reserve’s published stacking convention.
- `scf_corr()`: Weighted Pearson correlations.

### Statistical Inference

- `scf_ttest()`: One-sample and two-sample t-tests for continuous
  variables.
- `scf_prop_test()`: One-sample and two-sample proportion tests for
  binary variables.
- `scf_MIcombine()`: Combines estimates across imputations using Rubin’s
  Rules (internal to most functions).

### Regression Modeling

- `scf_ols()`: Linear regression with pooled estimates and implicate
  diagnostics.
- `scf_glm()`: Generalized linear models (e.g., logistic, Poisson).
- `scf_logit()`: Wrapper for logistic regression with optional odds
  ratio output.
- `scf_quantreg()`: Weighted quantile regression with pooled coefficients
  across implicates. Supports multiple quantiles in a single call.

All model functions return objects of class `scf_model_result`, with
methods for `coef()`, `vcov()`, `predict()`, `AIC()`, `residuals()`, and
`summary()`.

### Visualization

- `scf_plot_dist()`: Kernel density plots for visualizing and comparing distributions by group.
- `scf_plot_dbar()`: Bar plots of categorical variable distributions.
- `scf_plot_bbar()`: Stacked bar plots for two categorical variables.
- `scf_plot_cbar()`: Bar plots for continuous variable summaries by
  group.
- `scf_plot_smooth()`: Smoothed line plots for continuous distributions.
- `scf_plot_hist()`: Weighted histograms of continuous variables.
- `scf_plot_hex()`: Weighted hexbin plots for bivariate continuous data.

### Diagnostics and Output

- `scf_deflate()`: Converts nominal dollar estimates from `scf_mean()`,
  `scf_median()`, `scf_percentile()`, and `scf_ttest()` to real dollars
  using CPI-U-RS deflation factors from the Federal Reserve's SCF
  Bulletin SAS macro.
- `scf_regtable()`: Produces formatted regression tables from one or
  more `scf_model_result` objects, with options for console, HTML, and
  CSV output.
- `print()`, `summary()`: Custom methods for clean, interpretable output
  in analysis and teaching.

## Installation

Install the latest version of the package through CRAN:

``` r
install.packages("scf")
```

The package requires **R ≥ 3.6** and the following packages:

- `survey` (for replicate-weighted designs)
- `ggplot2` (for plotting)
- `httr`, `haven` (for downloading and reading SCF data)
- `mitools`, `stats`, `utils`, `methods`, and others (loaded
  automatically)

Use `install.packages()` to install any missing dependencies manually if
needed.

## Getting Started

### Download and Load Data

``` r, eval = F
# Download SCF data for 2022:
scf_download(2022)

# Load the data into a survey design object:
scf2022 <- scf_load(2022)
```

```r, include = F
# This document will use mock data for CRAN compliance
# use the above method to download and load data in your analysis instead of:
scf2022 <- readRDS(system.file("extdata", "scf2022_mock_raw.rds", package = "scf"))
# NOTE: Mock data for demonstration only. 
# Use `scf_download()` and `scf_load()` for full SCF datasets.
```


### Explore and Summarize

#### Univariate Distributions

``` r
# Frequency of education categories
scf_freq(scf2022, ~edcl)

# Median household net worth
scf_median(scf2022, ~networth)

# 90th percentile of income
scf_percentile(scf2022, ~income, q = 0.9)

# Histogram of net worth distribution
scf_plot_hist(scf2022, ~networth)

# Smoothed density plot of income
scf_plot_smooth(scf2022, ~income)
```

#### Bivariate Relationships

``` r
# Cross-tabulation of education and homeownership
scf_xtab(scf2022, ~edcl, ~own)

# Stacked bar chart: homeownership by education
scf_plot_bbar(scf2022, ~edcl, ~own)

# Weighted bar chart: mean net worth by education
scf_plot_cbar(scf2022, ~networth, ~edcl, stat = "mean")

# Grouped median income by race
scf_median(scf2022, ~income, by = ~racecl)

# Correlation between income and net worth
scf_corr(scf2022, ~income, ~networth)

# Hexbin plot: income vs. net worth
scf_plot_hex(scf2022, ~income, ~networth)
```

### Statistical Testing

``` r
# One-sample proportion test: Is more than 10% of households rich?
scf_prop_test(scf2022, ~I(networth > 1e6), p = 0.10, alternative = "greater")

# Two-sample proportion test: Are women less likely to be rich?
scf_prop_test(scf2022, ~I(networth > 1e6), ~factor(hhsex, labels = c("Male", "Female")), alternative = "less")

# One-sample t-test: Is mean income different from $75,000?
scf_ttest(scf2022, ~income, mu = 75000)

# Two-sample t-test: Are older households wealthier?
scf_ttest(scf2022, ~networth, ~I(age > 50), alternative = "greater")
```

### Regression Modeling

``` r
# Linear regression: Predict net worth from income and education
scf_ols(scf2022, networth ~ income + factor(edcl))

# Generalized linear model: Predict borrowing with logistic regression
scf_glm(scf2022, hborrff ~ income + age + factor(edcl), family = binomial())

# Logit wrapper: Predict probability of owning stocks
scf_logit(scf2022, ~I(owns_stocks == 1) ~ age + income + factor(edcl))
```

### Plotting and Visualization

``` r

# Bar chart of a single categorical variable
scf_plot_dbar(scf2022, ~edcl)

# Stacked bar chart comparing education by race
scf_plot_bbar(scf2022, ~edcl, ~racecl, scale = "percent", percent_by = "row")

# Smoothed line plot of net worth distribution
scf_plot_smooth(scf2022, ~networth, xlim = c(0, 2e6), method = "loess")

# Histogram of income distribution
scf_plot_hist(scf2022, ~income, bins = 40, xlim = c(0, 300000))

# Bar chart of mean net worth by education level
scf_plot_cbar(scf2022, ~networth, ~edcl, stat = "mean")

# Hexbin plot: net worth vs. income
scf_plot_hex(scf2022, ~income, ~networth, bins = 60)

```

### Wrangling and Transformation

``` r
# Create new variables across all implicates
scf2022 <- scf_update(scf2022,
  rich = networth > 1e6,
  senior = age >= 65,
  log_income = log(income + 1)
)

# Apply implicate-specific transformations (e.g., implicate-specific ranks)
scf2022 <- scf_update_by_implicate(scf2022, function(df) {
  threshold <- quantile(df$networth, probs = 0.90, na.rm = TRUE)
  df$top10nw <- df$networth >= threshold
  df
})

# Subset to working-age households with positive net worth
scf_sub <- scf_subset(scf2022, age >= 25 & age < 65 & networth > 0)

# Extract implicate-level estimates from a frequency table
freq <- scf_freq(scf_sub, ~own)
scf_implicates(freq, long = TRUE)
```

### Percentile Grouping

``` r
# Mean net worth by decile (implicate method, statistically preferred)
scf_pctile_sum(scf2022, ~networth)

# Top 10% vs. bottom 90%, stack method (fast; replicates Fed convention)
scf_pctile_sum(scf2022, ~networth,
               probs  = c(0, 0.9, 1),
               labels = c("bottom90", "top10"),
               method = "stack")

# Add a percentile grouping variable to the object for use in other functions
scf2022 <- scf_pctile_sum(scf2022, ~networth,
                           probs = c(0, 0.5, 0.9, 1),
                           labels = c("bottom50", "next40", "top10"),
                           stat = "none")
scf_median(scf2022, ~income, by = ~networth_pctile)
```

### Deflating to Real Dollars

``` r
# Convert nominal mean income estimates across years to 2022 dollars
m2016 <- scf_mean(scf2016, ~income)
m2022 <- scf_mean(scf2022, ~income)

m2016_real <- scf_deflate(m2016, from_year = 2016, to_year = 2022)
m2022_real <- scf_deflate(m2022, from_year = 2022)  # to_year defaults to 2022
```

### Quantile Regression

``` r
# Median regression: net worth on income and education
scf_quantreg(scf2022, networth ~ income + factor(edcl), tau = 0.5)

# Multiple quantiles in one call
scf_quantreg(scf2022, networth ~ income + factor(edcl),
             tau = c(0.25, 0.5, 0.75, 0.9))
```

### Regression Tables

``` r
# Compare OLS and logit models in a single formatted table
m_ols   <- scf_ols(scf2022, networth ~ income + factor(edcl))
m_logit <- scf_logit(scf2022, ~I(networth > 1e6) ~ income + factor(edcl))

scf_regtable(m_ols, m_logit)                          # console output
scf_regtable(m_ols, m_logit, output = "html")         # HTML fragment
scf_regtable(m_ols, m_logit, output = "csv",
             file = "results/table1.csv")             # CSV file
```

## Documentation:

For detailed examples, function documentation, and usage guides, consult
the package vignettes and reference manual.

- [SCF Homepage](https://github.com/jncohen/scf)
- [**CRAN Package Page**](https://CRAN.R-project.org/package=scf)

## Note on Mock Data

This package includes a small mock dataset (`scf2022_mock_raw.rds`) for testing purposes.  
It includes only 75 rows and select variables. It is structurally valid,  
but **not suitable for analytical use or inference**. Mock data objects carry a 
"mock" = TRUE attribute and may trigger warnings in functions to discourage 
interpretive use.


## Citation

If you use `scf` in published work, please cite it as:

> Joseph N. Cohen (2026). *scf: Analyzing the Survey of Consumer Finances.* R package. ver. 1.0.7. <https://github.com/jncohen/scf>

Use `citation("scf")` in R for formatted references.

## Author

Joseph N. Cohen  
Department of Sociology & Program in Data Analytics  
Queens College, City University of New York  
<joseph.cohen@qc.cuny.edu>

#' Estimate Generalized Linear Model from SCF Microdata
#'
#' @description
#' Estimates generalized linear models (GLMs) with SCF public-use microdata.
#' Use this function when modeling outcomes that follow non-Gaussian
#' distributions (e.g., binary or count data). Rubin's Rules are used to combine
#' implicate-level coefficient and variance estimates. 
#'
#' GLMs are performed across SCF implicates using `svyglm()` and returns
#' pooled coefficients, standard errors, z-values, p-values, and fit diagnostics
#' including AIC and pseudo-R-Squared when applicable.
#'
#' @section Implementation:
#' This function fits a GLM to each implicate in a `scf_mi_survey` object
#' using `survey::svyglm()`. The user specifies a model formula and a valid GLM
#' family (e.g., `binomial()`, `poisson()`, `gaussian()`). Coefficients and
#' variance-covariance matrices are extracted from each implicate and pooled
#' using Rubin's Rules.
#'
#' @section Details:
#' Generalized linear models (GLMs) extend linear regression to accommodate
#' non-Gaussian outcome distributions. The choice of `family` determines the
#' link function and error distribution. For example:
#' - `binomial()` fits logistic regression for binary outcomes
#' - `poisson()` models count data
#' - `gaussian()` recovers standard OLS behavior
#'
#' Model estimation is performed independently on each implicate using
#' `svyglm()` with replicate weights. Rubin's Rules are used to pool coefficient
#' estimates and variance matrices. For the pooling procedure, see
#' [scf_MIcombine()].
#'
#' @param object A `scf_mi_survey` object, typically created using [scf_load()] and [scf_design()].
#' @param formula A valid model formula, e.g., `rich ~ age + factor(edcl)`.
#' @param family A GLM family object such as [binomial()], [poisson()], or [gaussian()]. Defaults to `binomial()`.
#'
#' @return An object of class `"scf_glm"` and `"scf_model_result"` with:
#' \describe{
#'   \item{results}{A data frame of pooled coefficients, standard errors, z-values, p-values, and significance stars.}
#'   \item{fit}{A list of fit diagnostics including mean and SD of AIC; for binomial models, pseudo-R2 and its SD.}
#'   \item{models}{A list of implicate-level `svyglm` model objects.}
#'   \item{call}{The matched function call.}
#' }
#'
#' @examples
#' \donttest{
#' # Do not implement these lines in real analysis:
#' # Use functions `scf_download()` and `scf_load()`
#' td <- tempfile("glm_")
#' dir.create(td)
#' 
#' src <- system.file("extdata", "scf2022_mock_raw.rds", package = "scf")
#' file.copy(src, file.path(td, "scf2022.rds"), overwrite = TRUE)
#' scf2022 <- scf_load(2022, data_directory = td)
#'
#' # Example for real analysis: Run logistic regression
#' model <- suppressWarnings(scf_glm(scf2022, own ~ hhsex, family = binomial()))
#' summary(model)
#' 
#' # Do not implement these lines in real analysis: Cleanup for package check
#' unlink(td, recursive = TRUE, force = TRUE)
#' }
#'
#' @section Internal Suppression:
#'
#' For CRAN compliance and to prevent diagnostic overload during package checks,
#' this function internally wraps each implicate-level model call in `suppressWarnings()`.
#' This suppresses the known benign warning:
#'
#'   `"non-integer #successes in a binomial glm!"`
#'
#' which arises from using replicate weights with `family = binomial()`. This suppression
#' does not affect model validity or inference. Users wishing to inspect warnings can
#' run `survey::svyglm()` directly on individual implicates via `model$models[[i]]`.
#'
#' For further background, see:
#' https://stackoverflow.com/questions/12953045/warning-non-integer-successes-in-a-binomial-glm-survey-packages
#'
#' @seealso [scf_ols()], [scf_logit()], [scf_regtable()]
#'
#' @export
scf_glm <- function(object, formula, family = binomial()) {
  if (!inherits(object, "scf_mi_survey"))
    stop("Input must be of class 'scf_mi_survey'")
  if (!inherits(formula, "formula"))
    stop("Model must be specified as a formula")
  if (!inherits(family, "family"))
    stop("Family must be a valid GLM family object (e.g., binomial())")

  if (isTRUE(attr(object, "mock"))) {
    warning("Mock data detected. Do not interpret results as valid SCF estimates.", call. = FALSE)
  }

  designs <- object$mi_design

  models <- lapply(designs, function(imp) {
    tryCatch(
      suppressWarnings(survey::svyglm(formula, design = imp, family = family)),
      error = function(e) NULL
    )
  })
  models <- Filter(Negate(is.null), models)
  if (length(models) < 2)
    stop("Too few successful implicate-level models to pool.")

  coefs_list <- lapply(models, coef)
  vars_list  <- lapply(models, vcov)
  common_terms <- Reduce(intersect, lapply(coefs_list, names))
  coefs_list <- lapply(coefs_list, function(x) x[common_terms])
  vars_list  <- lapply(vars_list, function(v) v[common_terms, common_terms, drop = FALSE])

  # Drop the survey design from each model object after extracting what is
  # needed. svyglm stores the full svrepdesign (data + 999 replicate weights)
  # in $survey.design; keeping it multiplies object size by ~5x for no benefit.
  models <- lapply(models, function(m) { m$survey.design <- NULL; m })

  pooled <- scf_MIcombine(coefs_list, vars_list)

  est  <- coef(pooled)
  se   <- SE(pooled)
  zval <- est / se
  pval <- 2 * pnorm(-abs(zval))

  coef_table <- data.frame(
    term = names(est),
    estimate = est,
    std.error = se,
    z.value = zval,
    p.value = pval,
    stars = cut(pval,
                breaks = c(-Inf, 0.001, 0.01, 0.05, 0.10, Inf),
                labels = c("***", "**", "*", ".", ""),
                right = FALSE),
    stringsAsFactors = FALSE
  )

  aics <- sapply(models, function(m) {
    suppressWarnings(
      tryCatch(AIC(m), error = function(e) NA_real_)
    )
  })

  pseudo_r2 <- if (identical(family$family, "binomial")) {
    sapply(models, function(m) 1 - m$deviance / m$null.deviance)
  } else {
    rep(NA_real_, length(models))
  }

  fit_stats <- list(
    pseudo_r2 = if (all(!is.na(pseudo_r2))) mean(pseudo_r2) else NA_real_,
    pseudo_r2.sd = if (all(!is.na(pseudo_r2))) sd(pseudo_r2) else NA_real_,
    AIC = if (all(!is.na(aics))) mean(aics) else NA_real_,
    AIC.sd = if (all(!is.na(aics))) sd(aics) else NA_real_
  )

  out <- list(
    results = coef_table,
    fit = fit_stats,
    vcov = pooled$variance,
    models = models,
    call = match.call(),
    formula = formula
  )
  class(out) <- c("scf_glm", "scf_model_result")
  return(out)
}
#' @export
print.scf_glm <- function(x, digits = 4, ...) {
  cat("Generalized Linear Model (Multiply-Imputed SCF)\n")
  cat("--------------------------------------------------\n")

  df <- x$results
  df$estimate   <- round(df$estimate, digits)
  df$std.error  <- round(df$std.error, digits)
  df$z.value    <- round(df$z.value, digits)
  df$p.value    <- format.pval(df$p.value, digits = digits)

  print(df[, c("term", "estimate", "std.error", "z.value", "p.value", "stars")], row.names = FALSE)

  cat("\nModel Fit Diagnostics:\n")
  if (!is.null(x$fit$pseudo_r2) && !is.na(x$fit$pseudo_r2)) {
    cat("  Pseudo R-squared: ", formatC(x$fit$pseudo_r2, digits = 3, format = "f"),
        " (SD: ", formatC(x$fit$pseudo_r2.sd, digits = 3, format = "f"), ")\n", sep = "")
  }
  if (!is.null(x$fit$AIC)) {
    cat("  Mean AIC:         ", formatC(x$fit$AIC, digits = 0, format = "f"),
        " (SD: ", formatC(x$fit$AIC.sd, digits = 0, format = "f"), ")\n", sep = "")
  }

  cat("\nNote: Model fit pooled across implicates via Rubin's Rules.\n")
  cat("      Inspect individual models via `object$models[[i]]`.\n")
  invisible(x)
}

#' @export
summary.scf_glm <- function(object, digits = 4, ...) {
  cat("SCF Generalized Linear Model Summary\n")
  cat("------------------------------------\n")

  df <- object$results
  df$estimate   <- round(df$estimate, digits)
  df$std.error  <- round(df$std.error, digits)
  df$z.value    <- round(df$z.value, digits)
  df$p.value    <- format.pval(df$p.value, digits = digits)

  cat("Pooled Coefficient Estimates:\n")
  print(df[, c("term", "estimate", "std.error", "z.value", "p.value", "stars")], row.names = FALSE)

  cat("\nModel Diagnostics:\n")
  if (!is.null(object$fit$pseudo_r2) && !is.na(object$fit$pseudo_r2)) {
    cat("  Pseudo R-squared: ", formatC(object$fit$pseudo_r2, digits = 3, format = "f"),
        " (SD: ", formatC(object$fit$pseudo_r2.sd, digits = 3, format = "f"), ")\n", sep = "")
  }
  if (!is.null(object$fit$AIC)) {
    cat("  Mean AIC:         ", formatC(object$fit$AIC, digits = 0, format = "f"),
        " (SD: ", formatC(object$fit$AIC.sd, digits = 0, format = "f"), ")\n", sep = "")
  }

  cat("\nCall:\n")
  print(object$call)

  invisible(object)
}

#' Summarize SCF Variables by Percentile Group
#'
#' @description
#' Creates a percentile-based grouping variable for a continuous SCF variable
#' and optionally computes a summary statistic within each group. Two methods
#' are available. The \code{"implicate"} method (the default) computes
#' survey-weighted quantile thresholds separately within each implicate, the
#' statistically preferable approach under multiple imputation, as it correctly
#' accounts for between-implicate variation in imputed values. The
#' \code{"stack"} method replicates the Federal Reserve's published convention,
#' in which all five implicates are pooled with weights divided by five,
#' observations are sorted by the percentile variable and household identifier,
#' percentile groups are assigned from the cumulative weighted population share,
#' and a flat weighted statistic is computed directly on the stacked data.
#'
#' @details
#' The two methods will generally produce similar but not identical results.
#' Use \code{method = "stack"} when replication of the Federal Reserve's
#' published percentile-group point estimates is required. Note that 
#' \code{method = "stack"} with \code{stat != "none"} computes a flat weighted 
#' statistic on the pooled stacked data and does not use replicate-weight 
#' survey machinery or Rubin's combining rules. No standard error is 
#' returned in this case.
#'
#' When \code{stat = "none"}, the function returns the input
#' \code{scf_mi_survey} object with the grouping variable added, ready to pass
#' to \code{\link{scf_mean}}, \code{\link{scf_median}}, or other estimation
#' functions via the \code{by} argument. For standard grouping variables
#' already published by the Fed, such as net worth percentile (\code{nwcat})
#' and income percentile (\code{inccat}), those variables are included directly
#' in the data returned by \code{\link{scf_load}} and can be passed to
#' \code{by} without calling \code{scf_pctile_sum}.
#' 
#' For \code{method = "stack"}, percentile groups are assigned using the
#' Federal Reserve SAS macro logic: after stacking implicates and dividing
#' weights by five, observations are sorted by the variable used to define the
#' percentile groups and by household identifier. The cumulative weighted
#' population share is then used to assign groups. This is not equivalent to
#' first estimating a percentile threshold and then applying a simple
#' \code{>} or \code{>=} comparison, especially when observations lie at or
#' near a percentile boundary.
#'
#'
#' @param scf A \code{scf_mi_survey} object created with \code{\link{scf_load}}.
#' @param var A one-sided formula naming the continuous variable to cut
#'   (e.g., \code{~networth}).
#' @param probs Numeric vector with values in between 0 and 1 defining group boundaries,
#'   including 0 and 1 as endpoints. Defaults to deciles
#'   (\code{seq(0, 1, by = 0.1)}).
#' @param labels Optional character vector of group labels, length equal to
#'   \code{length(probs) - 1}. If \code{NULL} (default), labels are generated
#'   automatically in the form \code{"p0-p10"}, \code{"p10-p20"}, etc.
#' @param varname Optional name for the new grouping variable. Defaults to
#'   \code{"{var}_pctile"} (e.g., \code{"networth_pctile"}).
#' @param method Character. One of \code{"implicate"} (default) or
#'   \code{"stack"}. See Details.
#' @param stat Character. One of \code{"mean"} (default), \code{"median"},
#'   or \code{"none"}. When \code{"none"}, returns the updated
#'   \code{scf_mi_survey} object with the grouping variable added and no
#'   summary statistic computed. When \code{"mean"} or \code{"median"},
#'   computes the statistic within each percentile group and returns a data
#'   frame of results.
#' @param stat_var Optional one-sided formula naming the variable to summarize
#'   within each group. Defaults to \code{var} if not supplied.
#'
#' @return When \code{stat = "none"}, returns the input \code{scf_mi_survey}
#'   object with a new factor variable added to each implicate's data frame.
#'   The variable is named according to \code{varname} (default:
#'   \code{"{var}_pctile"}), and can be passed directly to
#'   \code{\link{scf_mean}}, \code{\link{scf_median}}, or other estimation
#'   functions via the \code{by} argument. When \code{stat = "mean"} or
#'   \code{"median"} with \code{method = "implicate"}, returns the output of
#'   \code{\link{scf_mean}} or \code{\link{scf_median}} respectively. When
#'   \code{stat != "none"} with \code{method = "stack"}, returns a data frame
#'   with columns \code{group}, \code{variable}, and \code{estimate}. No
#'   standard error is returned for the stack method.
#'
#' @examples
#' # Do not implement these lines in real analysis:
#' # Use functions `scf_download()` and `scf_load()`
#' td <- tempfile("pctile_sum_")
#' dir.create(td)
#' src <- system.file("extdata", "scf2022_mock_raw.rds", package = "scf")
#' file.copy(src, file.path(td, "scf2022.rds"), overwrite = TRUE)
#' scf2022 <- scf_load(2022, data_directory = td)
#'
#' # Mean net worth, top vs bottom 90 percent, stack method (fast)
#' scf_pctile_sum(scf2022, ~networth,
#'                probs  = c(0, 0.9, 1),
#'                labels = c("bottom90", "top10"),
#'                method = "stack")
#'
#' \dontrun{
#' # Implicate method (default): requires full SCF data; unreliable on mock data
#' scf_pctile_sum(scf2022, ~networth)
#'
#' # Return grouping variable only, no summary statistic (implicate method)
#' scf2022 <- scf_pctile_sum(scf2022, ~networth,
#'                            probs  = c(0, 0.9, 1),
#'                            labels = c("bottom90", "top10"),
#'                            stat   = "none")
#' }
#'
#' # Do not implement these lines in real analysis: Cleanup for package check
#' unlink(td, recursive = TRUE, force = TRUE)
#'
#' @seealso \code{\link{scf_mean}}, \code{\link{scf_median}},
#'   \code{\link{scf_percentile}}, \code{\link{scf_update_by_implicate}}
#'
#' @export
scf_pctile_sum <- function(scf, var,
                           probs    = seq(0, 1, by = 0.1),
                           labels   = NULL,
                           varname  = NULL,
                           method   = c("implicate", "stack"),
                           stat     = c("mean", "median", "none"),
                           stat_var = NULL) {
  
  method <- match.arg(method)
  stat   <- match.arg(stat)
  
  # --- Input validation ---
  if (!inherits(scf, "scf_mi_survey")) {
    stop("`scf` must be an object of class 'scf_mi_survey'.", call. = FALSE)
  }
  if (!inherits(var, "formula")) {
    stop("`var` must be a one-sided formula (e.g., ~networth).", call. = FALSE)
  }
  if (!is.numeric(probs) || any(probs < 0) || any(probs > 1)) {
    stop("`probs` must be a numeric vector with values in [0, 1].", call. = FALSE)
  }
  if (!is.null(stat_var) && !inherits(stat_var, "formula")) {
    stop("`stat_var` must be a one-sided formula (e.g., ~networth).", call. = FALSE)
  }
  
  probs <- sort(unique(probs))
  
  if (probs[1] != 0 || probs[length(probs)] != 1) {
    stop("`probs` must include 0 and 1 as endpoints.", call. = FALSE)
  }
  
  n_groups <- length(probs) - 1
  
  if (n_groups < 2) {
    stop("`probs` must define at least two groups (length >= 3).", call. = FALSE)
  }
  
  if (!is.null(labels)) {
    if (!is.character(labels) || length(labels) != n_groups) {
      stop(sprintf("`labels` must be a character vector of length %d.", n_groups),
           call. = FALSE)
    }
  }
  
  if (isTRUE(attr(scf, "mock"))) {
    warning("Mock data detected. Do not interpret results as valid SCF estimates.",
            call. = FALSE)
  }
  
  # --- Derived values ---
  varname_in <- all.vars(var)[1]
  
  if (is.null(varname)) {
    varname <- paste0(varname_in, "_pctile")
  }
  
  if (is.null(stat_var)) {
    stat_var <- var
  }
  
  stat_varname <- all.vars(stat_var)[1]
  interior_probs <- probs[probs > 0 & probs < 1]
  
  if (is.null(labels)) {
    labels <- paste0("p", round(probs[-length(probs)] * 100),
                     "-p", round(probs[-1] * 100))
  }
  
  # --- Stack method (Federal Reserve convention) ---
  if (method == "stack") {
    M <- length(scf$mi_design)
    
    all_vars <- do.call(rbind, lapply(seq_along(scf$mi_design), function(i) {
      df <- scf$mi_design[[i]]$variables
      df$.implicate <- i
      df$.row_id <- seq_len(nrow(df))
      df
    }))
    
    all_vars$.wgt_stack <- all_vars$wgt / M
    
    # Identify household/person ID for SAS-style tie ordering.
    # The Federal Reserve SAS macro sorts by the percentile variable and PID.
    id_candidates <- c("y1", "Y1", "x1", "X1")
    id_var <- id_candidates[id_candidates %in% names(all_vars)][1]
    
    if (is.na(id_var)) {
      stop("Could not find household ID variable (`y1`, `Y1`, `x1`, or `X1`) ",
           "needed for stack-method percentile assignment.",
           call. = FALSE)
    }
    
    keep <- is.finite(all_vars[[varname_in]]) &
      is.finite(all_vars$.wgt_stack) &
      all_vars$.wgt_stack > 0
    
    if (stat != "none") {
      keep <- keep & is.finite(all_vars[[stat_varname]])
    }
    
    all_vars <- all_vars[keep, , drop = FALSE]
    
    # Federal Reserve SAS macro logic:
    # sort by percentile variable and household/person ID, then assign
    # percentile groups from cumulative weighted population share.
    ord <- order(all_vars[[varname_in]], all_vars[[id_var]])
    all_vars <- all_vars[ord, , drop = FALSE]
    
    cumshare <- cumsum(all_vars$.wgt_stack) / sum(all_vars$.wgt_stack)
    
    group_num <- rep(1L, length(cumshare))
    
    for (j in seq_len(n_groups)) {
      group_num[cumshare >= probs[j]] <- j
    }
    
    group_num[group_num < 1L] <- 1L
    group_num[group_num > n_groups] <- n_groups
    
    all_vars[[varname]] <- factor(labels[group_num], levels = labels)
    
    # If no summary statistic requested, add grouping variable to each
    # implicate and return the scf_mi_survey object. This uses the same
    # stack-level category assignment, then maps the assigned group back
    # to each original implicate row.
    if (stat == "none") {
      split_vars <- split(all_vars, all_vars$.implicate)
      
      updated_designs <- vector("list", M)
      
      for (i in seq_along(scf$mi_design)) {
        df_orig <- scf$mi_design[[i]]$variables
        df_new  <- split_vars[[as.character(i)]]
        
        if (is.null(df_new)) {
          stop("Stack-method grouping failed for implicate ", i, ".", call. = FALSE)
        }
        
        df_orig[[varname]] <- NA_character_
        df_orig[[varname]][df_new$.row_id] <- as.character(df_new[[varname]])
        df_orig[[varname]] <- factor(df_orig[[varname]], levels = labels)
        
        rep_cols <- grep("^wt1b", names(df_orig), value = TRUE)
        if (length(rep_cols) == 0) {
          stop("Could not find replicate weight columns in implicate.", call. = FALSE)
        }
        
        updated_designs[[i]] <- survey::svrepdesign(
          weights          = ~wgt,
          repweights       = as.matrix(df_orig[, rep_cols]),
          data             = df_orig,
          type             = "other",
          scale            = 1,
          rscales          = rep(1 / (length(rep_cols) - 1), length(rep_cols)),
          mse              = TRUE,
          combined.weights = TRUE
        )
      }
      
      scf$mi_design <- updated_designs
      return(scf)
    }
    
    # Flat weighted statistic on stacked data, Fed convention.
    out <- do.call(rbind, lapply(labels, function(g) {
      idx <- all_vars[[varname]] == g
      
      est <- if (stat == "mean") {
        weighted.mean(all_vars[[stat_varname]][idx],
                      all_vars$.wgt_stack[idx],
                      na.rm = TRUE)
      } else {
        .scf_wtd_median(x = all_vars[[stat_varname]][idx],
                        w = all_vars$.wgt_stack[idx])
      }
      
      data.frame(group    = g,
                 variable = stat_varname,
                 estimate = est,
                 stringsAsFactors = FALSE)
    }))
    
    rownames(out) <- NULL
    return(out)
  }
  
  # --- Implicate method (per-implicate survey-weighted quantiles) ---
  updated_designs <- vector("list", length(scf$mi_design))
  
  for (i in seq_along(scf$mi_design)) {
    design <- scf$mi_design[[i]]
    df     <- design$variables
    
    thresholds <- as.numeric(coef(
      suppressWarnings(suppressMessages(
        survey::svyquantile(var, design,
                            quantiles     = interior_probs,
                            se            = TRUE,
                            interval.type = "quantile")
      ))
    ))
    
    breaks <- c(-Inf, thresholds, Inf)
    
    df[[varname]] <- cut(df[[varname_in]],
                         breaks         = breaks,
                         labels         = labels,
                         include.lowest = TRUE,
                         right          = FALSE)
    
    rep_cols <- grep("^wt1b", names(df), value = TRUE)
    if (length(rep_cols) == 0) {
      stop("Could not find replicate weight columns in implicate.", call. = FALSE)
    }
    
    updated_designs[[i]] <- survey::svrepdesign(
      weights          = ~wgt,
      repweights       = as.matrix(df[, rep_cols]),
      data             = df,
      type             = "other",
      scale            = 1,
      rscales          = rep(1 / (length(rep_cols) - 1), length(rep_cols)),
      mse              = TRUE,
      combined.weights = TRUE
    )
  }
  
  scf$mi_design <- updated_designs
  
  # Return updated object if no stat requested
  if (stat == "none") return(scf)
  
  # Compute summary statistic using Rubin's rules
  by_formula <- stats::as.formula(paste0("~", varname))
  
  if (stat == "mean") {
    return(scf_mean(scf, stat_var, by = by_formula))
  } else {
    return(scf_median(scf, stat_var, by = by_formula))
  }
}


#' @rdname scf_pctile_sum
#' @export
scf_pctile_cut <- scf_pctile_sum


# Internal helper: weighted median for stacked-data summaries.
# This is used only when stat = "median"; percentile-group construction
# under method = "stack" is handled directly by cumulative weighted
# population-share assignment in scf_pctile_sum().
.scf_wtd_median <- function(x, w) {
  ok <- is.finite(x) & is.finite(w) & w > 0
  x <- x[ok]
  w <- w[ok]
  
  if (length(x) == 0L) return(NA_real_)
  
  ord <- order(x)
  x <- x[ord]
  w <- w[ord]
  
  cum_w <- cumsum(w) / sum(w)
  x[which(cum_w >= 0.5)[1L]]
}

#' Default Plot Theme for SCF Visualizations
#'
#' Defines the SCF package's default `ggplot2` theme, optimized for legibility,
#' clarity, and aesthetic coherence across print, desktop, and mobile platforms.
#'
#' @description
#' The theme is designed to:
#' - Render cleanly in **print** (single-column or wrapped layout)
#' - Scale well on **HD desktop** monitors without visual clutter
#' - Remain **legible on mobile** with clear fonts and sufficient contrast
#'
#' The default figure dimensions assumed for export are **5.5 inches by 5.5 inches**
#' at **300 dpi**, which balances compactness with accessibility across media.
#'
#' All theme settings are exposed via comments to enable easy brand customization.
#'
#' @param base_size Base font size. Defaults to 13.
#' @param base_family Font family. Defaults to "sans".
#' @param grid Logical. Show gridlines? Defaults to TRUE.
#' @param axis Logical. Include axis ticks and lines? Defaults to TRUE.
#' @param ... Additional arguments passed to [ggplot2::theme_minimal()].
#'
#' @return \code{scf_theme()} returns a \code{ggplot2} theme object.
#'   \code{scf_activate_theme()} returns \code{NULL} invisibly; called for
#'   its side effect of setting the session-wide \code{ggplot2} theme.
#'
#' @section Global activation:
#' \code{scf_activate_theme()} sets \code{scf_theme()} as the default
#' \code{ggplot2} theme for all plots in the current R session — equivalent
#' to \code{ggplot2::theme_set(scf_theme())}. The effect lasts only for the
#' session and does not persist across sessions. All \code{scf_plot_*()} functions
#' apply \code{scf_theme()} internally, so this is most useful when producing
#' custom plots alongside the package's built-in visualizations.
#'
#' @examples
#' library(ggplot2)
#' ggplot(mtcars, aes(factor(cyl))) +
#'   geom_bar(fill = "#0072B2") +
#'   scf_theme()
#'
#' # Activate globally for the session:
#' scf_activate_theme()
#'
#' @seealso [ggplot2::theme()], [scf_plot_dist()]
#' @name scf_theme
#' @export
scf_theme <- function(base_size = 13,
                      base_family = "sans",
                      grid = TRUE,
                      axis = TRUE,
                      ...) {

  ggplot2::theme_minimal(base_size = base_size, base_family = base_family, ...) +
    ggplot2::theme(

      ## Title styling
      plot.title = ggplot2::element_text(
        face = "bold",
        size = base_size + 2,
        hjust = 0
        # vjust = 1,            # vertical positioning
        # color = "#000000",    # custom title color
      ),

      ## Subtitle styling (if used)
      # plot.subtitle = ggplot2::element_text(
      #   face = "plain",
      #   size = base_size + 0,
      #   hjust = 0
      # ),

      ## Axis titles
      axis.title = ggplot2::element_text(
        size = base_size
        # face = "plain",      # could be "bold", "italic"
        # color = "black"
      ),

      ## Axis text (tick labels)
      axis.text = ggplot2::element_text(
        size = base_size - 1
        # angle = 0,           # optionally tilt text
        # color = "black"
      ),

      ## Axis ticks and lines
      axis.line = if (axis) ggplot2::element_line(color = "grey40") else ggplot2::element_blank(),
      axis.ticks = if (axis) ggplot2::element_line(color = "grey40") else ggplot2::element_blank(),

      ## Grid lines
      panel.grid.major = if (grid) ggplot2::element_line(color = "grey90") else ggplot2::element_blank(),
      panel.grid.minor = ggplot2::element_blank(),
      # Alternatives:
      # panel.grid.major.x = ggplot2::element_line(color = "grey90"),
      # panel.grid.major.y = ggplot2::element_line(color = "grey95"),

      ## Legend styling (if used)
      # legend.position = "right",   # or "bottom", "top", "none"
      # legend.title = ggplot2::element_text(size = base_size),
      # legend.text = ggplot2::element_text(size = base_size - 1),

      ## Margins (top, right, bottom, left)
      plot.margin = ggplot2::margin(10, 10, 10, 10)

      ## Background options (if dark mode or boxed wanted)
      # plot.background = ggplot2::element_rect(fill = "white", color = NA),
      # panel.background = ggplot2::element_rect(fill = "white", color = NA)
    )
}

#' Construct an SCF Multiply-Imputed Survey Object
#'
#' @description
#' Wraps a list of replicate-weighted survey designs into an `scf_mi_survey`
#' object. This is called internally by [scf_load()], but is also available
#' directly for users who construct their own implicate-level designs outside
#' the standard download-and-load workflow — for example, when integrating
#' external or custom-prepared SCF data files.
#'
#' Each element of `design` must be a [survey::svrepdesign()] object representing
#' one SCF implicate with replicate weights.
#'
#' @param design A list of five [survey::svrepdesign()] objects (one per implicate).
#' @param year Numeric SCF survey year (e.g., 2022).
#' @param n_households Numeric total U.S. households represented in `year`.
#'
#' @return An object of class "scf_mi_survey" with:
#' \describe{
#'   \item{mi_design}{List of replicate-weighted designs (one per implicate).}
#'   \item{year}{SCF survey year.}
#'   \item{n_households}{Estimated number of U.S. households.}
#' }
#'
#' @examples
#' # Do not implement these lines in real analysis:
#' # Use functions `scf_download()` and `scf_load()`
#' td <- tempfile("design_")
#' dir.create(td)
#'
#' src <- system.file("extdata", "scf2022_mock_raw.rds", package = "scf")
#' file.copy(src, file.path(td, "scf2022.rds"), overwrite = TRUE)
#' scf2022 <- scf_load(2022, data_directory = td)
#'
#' # Example for real analysis: Construct scf_mi_survey object
#' obj <- scf_design(
#'   design = scf2022$mi_design,
#'   year = 2022,
#'   n_households = attr(scf2022, "n_households")
#' )
#' class(obj)
#' length(obj$mi_design)
#'
#' # Do not implement these lines in real analysis: Cleanup for package check
#' unlink(td, recursive = TRUE, force = TRUE)
#'
#' @seealso [scf_load()], [scf_update()]
#' @export
scf_design <- function(design, year, n_households) {
  if (!is.list(design) || !all(sapply(design, inherits, "svyrep.design")))
    stop("`design` must be a list of `svyrep.design` objects (one per implicate).")
  structure(list(mi_design = design, year = year, n_households = n_households),
            class = "scf_mi_survey")
}

#' @export
print.scf_mi_survey <- function(x, ...) {
  cat("SCF Multiply-Imputed Survey Object\n")
  cat("----------------------------------\n")
  cat("Year:          ", x$year, "\n", sep = "")
  cat("Households (N):", format(x$n_households, big.mark = ","), "\n", sep = "")
  cat("Implicates:    ", length(x$mi_design), "\n", sep = "")
  cat("Replicate weights per implicate:",
      ncol(x$mi_design[[1]]$repweights), "\n")
  invisible(x)
}

# zzz.R

# Register global variables to silence CMD check notes
utils::globalVariables(c(
  "group", "category", "estimate", "yval", "xval", "wgt", "x",
  "percent", "proportion", "imp", "weights"
))

#' Subset an `scf_mi_survey` Object
#'
#' @description
#' Subsetting refers to the process of retaining only those observations that
#' satisfy a logical (TRUE/FALSE) condition. This function applies such a
#' filter independently to each implicate in an `scf_mi_survey` object created
#' by [scf_design()] via [scf_load()]. The result is a new multiply-imputed,
#' replicate-weighted survey object with appropriately restricted designs.
#'
#' @section Implementation:
#' Use `scf_subset()` to focus analysis on analytically meaningful
#' sub-populations. For example, to analyze only households headed by seniors:
#' 
#' \preformatted{
#' scf2022_seniors <- scf_subset(scf2022, age >= 65)
#' }
#'
#' This is especially useful when analyzing populations such as renters, homeowners, specific age brackets,
#' or any group defined by logical expressions over SCF variables.
#'
#' @section Details:
#' Filtering is conducted separately in each implicate. This preserves valid design structure but means
#' that the same household may fall into or out of the subset depending on imputed values.
#' For example, a household with five different age imputations—say, 64, 66, 63, 65, and 67—would be
#' classified as a senior in only three of five implicates if subsetting on `age >= 65`.
#'
#' Empty subsets in any implicate can cause downstream analysis to fail. Always check subgroup sizes after subsetting.
#'
#' @param scf A `scf_mi_survey` object, typically created by [scf_load()].
#' @param expr A logical expression used to filter rows, evaluated separately in each implicate's variable frame (e.g., `age < 65 & own == 1`).
#'
#' @return A new `scf_mi_survey` object (see [scf_design()])
#'
#' @examples
#' # Do not implement these lines in real analysis:
#' # Use functions `scf_download()` and `scf_load()`
#' td <- tempfile("subset_")
#' dir.create(td)
#'
#' src <- system.file("extdata", "scf2022_mock_raw.rds", package = "scf")
#' file.copy(src, file.path(td, "scf2022.rds"), overwrite = TRUE)
#' scf2022 <- scf_load(2022, data_directory = td)
#'
#' # Example for real analysis: Filter for working-age households with positive net worth
#' scf_sub <- scf_subset(scf2022, age < 65 & networth > 0)
#' scf_mean(scf_sub, ~income)
#' 
#' # Do not implement these lines in real analysis: Cleanup for package check
#' unlink(td, recursive = TRUE, force = TRUE)
#'
#'
#' @seealso [scf_load()], [scf_update()]
#'
#' @export
scf_subset <- function(scf, expr) {
  if (!inherits(scf, "scf_mi_survey")) stop("Input must be a `scf_mi_survey` object.")

  if (isTRUE(attr(scf, "mock"))) {
    warning("Mock data detected. Do not interpret results as valid SCF estimates.", call. = FALSE)
  }

  expr <- substitute(expr)

  new_designs <- vector("list", length(scf$mi_design))
  names(new_designs) <- names(scf$mi_design)

  for (i in seq_along(scf$mi_design)) {
    d <- scf$mi_design[[i]]
    dsub <- subset(d, eval(expr, d$variables, parent.frame()))
    new_designs[[i]] <- dsub
  }

  new_data <- do.call(rbind, lapply(seq_along(new_designs), function(i) {
    df <- new_designs[[i]]$variables
    df$implicate <- i
    df
  }))

  new_implicates <- if (!is.null(scf$implicates)) {
    lapply(scf$implicates, function(df) df[eval(expr, df, parent.frame()), ])
  } else {
    NULL
  }

  result <- structure(
    list(
      mi_design = new_designs,
      data = new_data,
      implicates = new_implicates,
      year = scf$year,
      n_households = sum(new_designs[[1]]$variables$wgt)
    ),
    class = "scf_mi_survey"
  )
  attr(result, "mock") <- isTRUE(attr(scf, "mock"))
  result
}

#' Estimate the Frequencies of a Discrete Variable from SCF Microdata
#'
#' Computes weighted proportions and standard errors for a discrete variable
#' in multiply-imputed SCF data, optionally stratified by a grouping variable.
#' Proportions and standard errors are computed separately within each 
#' implicate using `svymean()`, then averaged across
#' implicates using SCF-recommended pooling logic. Group-wise frequencies are
#' supported, but users may find the features of [scf_xtab()] to be more useful.
#'
#' @description
#' This function estimates the relative frequency (proportion) of each category
#' in a discrete variable from the SCF public-use microdata.  Use this function
#' to discern the univariate distribution of a discrete variable.
#'
#'
#' @section Details:
#' Proportions are estimated within each implicate using `survey::svymean()`, 
#' then pooled using the standard MI formula for proportions. When a grouping 
#' variable is provided via `by`, estimates are produced separately for each
#' group-category combination. Results may be scaled to percentages using the 
#' `percent` argument.
#'
#' Estimates are pooled using the standard formula:
#' - The mean of implicate-level proportions is the point estimate
#' - The standard error reflects both within-implicate variance and across-implicate variation
#'
#' Unlike means or model parameters, category proportions do not use Rubin's full combination rules (e.g., degrees of freedom).
#'
#' @param scf A `scf_mi_survey` object created by [scf_load()]. Must contain five replicate-weighted implicates.
#' @param var A one-sided formula specifying a categorical variable (e.g., `~racecl`).
#' @param by Optional one-sided formula specifying a discrete grouping variable (e.g., `~own`).
#' @param percent Logical. If `TRUE` (default), scales results and standard errors to percentages.
#'
#' @return A list of class `"scf_freq"` with:
#' \describe{
#'   \item{results}{Pooled category proportions and standard errors, by group if specified.}
#'   \item{imps}{A named list of implicate-level proportion estimates.}
#'   \item{aux}{Metadata about the variable and grouping structure.}
#' }
#'
#'
#' @seealso [scf_xtab()], [scf_plot_dist()]
#'
#' @examples
#' # Do not implement these lines in real analysis:
#' # Use functions `scf_download()` and `scf_load()`
#' td <- tempfile("freq_")
#' dir.create(td)
#'
#' src <- system.file("extdata", "scf2022_mock_raw.rds", package = "scf")
#' file.copy(src, file.path(td, "scf2022.rds"), overwrite = TRUE)
#' scf2022 <- scf_load(2022, data_directory = td)
#'
#' # Example for real analysis: Proportions of homeownership
#' scf_freq(scf2022, ~own)
#'
#' # Example for real analysis: Homeownership proportions by education
#' scf_freq(scf2022, ~own, by = ~edcl)
#'
#' # Do not implement these lines in real analysis: Cleanup for package check
#' unlink(td, recursive = TRUE, force = TRUE)
#'
#' @export
scf_freq <- function(scf, var, by = NULL, percent = TRUE) {
  if (!inherits(scf, "scf_mi_survey") ||
      !is.list(scf$mi_design) ||
      !all(sapply(scf$mi_design, inherits, "svyrep.design"))) {
    stop("Input must be an 'scf_mi_survey' object with valid multiply-imputed designs.")
  }

  if (isTRUE(attr(scf, "mock"))) {
    warning("Mock data detected. Do not interpret results as valid SCF estimates.", call. = FALSE)
  }

  varname <- all.vars(var)[1]
  byname <- if (!is.null(by)) all.vars(by)[1] else NULL

  # Ensure discrete variables
  check_discrete <- function(vname) {
    v <- scf$mi_design[[1]]$variables[[vname]]
    if (is.numeric(v) && length(unique(v)) > 25) {
      stop(sprintf("Variable '%s' appears continuous. Use only discrete variables.", vname))
    }
  }
  check_discrete(varname)
  if (!is.null(byname)) check_discrete(byname)

  designs <- scf$mi_design
  imp_out <- vector("list", length(designs))
  names(imp_out) <- paste0("imp", seq_along(designs))

  for (i in seq_along(designs)) {
    d <- designs[[i]]
    d$variables[[varname]] <- factor(d$variables[[varname]])
    if (!is.null(byname)) d$variables[[byname]] <- factor(d$variables[[byname]])

    if (is.null(byname)) {
      est <- survey::svymean(as.formula(paste0("~", varname)), d)
      labs <- gsub(paste0("^", varname), "", names(coef(est)))
      df <- data.frame(
        implicate = i,
        group = NA,
        category = labs,
        est = as.numeric(coef(est)),
        var = diag(vcov(est)),
        stringsAsFactors = FALSE
      )
    } else {
      levels_g <- levels(d$variables[[byname]])
      df <- do.call(rbind, lapply(levels_g, function(g) {
        dsub <- subset(d, d$variables[[byname]] == g)
        est <- try(survey::svymean(as.formula(paste0("~", varname)), dsub), silent = TRUE)
        if (inherits(est, "try-error")) return(NULL)
        labs <- gsub(paste0("^", varname), "", names(coef(est)))
        data.frame(
          implicate = i,
          group = g,
          category = labs,
          est = as.numeric(coef(est)),
          var = diag(vcov(est)),
          stringsAsFactors = FALSE
        )
      }))
    }
    imp_out[[i]] <- df
  }

  long <- do.call(rbind, Filter(Negate(is.null), imp_out))
  combos <- unique(long[, c("group", "category")])
  pooled <- lapply(seq_len(nrow(combos)), function(k) {
    g <- combos$group[k]
    c <- combos$category[k]
    subdf <- subset(long,
                    (is.na(group) & is.na(g) | group == g) & category == c)
    m <- length(unique(subdf$implicate))
    qbar <- mean(subdf$est)
    ubar <- mean(subdf$var)
    b <- var(subdf$est)
    se <- sqrt(ubar + (1 + 1/m) * b)

    data.frame(
      group = g,
      category = c,
      proportion = if (percent) 100 * qbar else qbar,
      se_proportion = if (percent) 100 * se else se,
      stringsAsFactors = FALSE
    )
  })

  out <- list(
    results = do.call(rbind, pooled),
    imps = setNames(imp_out, paste0("imp", seq_along(imp_out))),
    aux = list(variable = varname, group = byname)
  )
  class(out) <- "scf_freq"
  return(out)
}

#' @export
print.scf_freq <- function(x, ...) {
  cat("SCF Frequency Table (Pooled Results)\n\n")
  print(x$results, row.names = FALSE)
  invisible(x)
}

#' @export
summary.scf_freq <- function(object, ...) {
  cat("Summary of SCF Frequency Analysis\n")
  cat("Variable:", object$aux$variable, "\n")
  if (!is.null(object$aux$group)) cat("Grouped by:", object$aux$group, "\n")
  cat("\nPooled Estimates:\n")
  print(object$results, row.names = FALSE)

  cat("\nIndividual Implicate Results:\n")
  for (i in seq_along(object$imps)) {
    cat("\nImplicate", i, ":\n")
    print(object$imps[[i]], row.names = FALSE)
  }
  invisible(object)
}

# tests/testthat/test-scf-package.R

test_that("scf2022 mock data loads as scf_mi_survey object", {
  td  <- tempdir()
  src <- system.file("extdata", "scf2022_mock_raw.rds", package = "scf")
  file.copy(src, file.path(td, "scf2022.rds"), overwrite = TRUE)
  scf2022 <- scf_load(2022, data_directory = td)
  
  expect_s3_class(scf2022, "scf_mi_survey")
  expect_equal(length(scf2022$mi_design), 5)
  
  unlink(file.path(td, "scf2022.rds"), force = TRUE)
})

test_that("Descriptive functions return expected structure", {
  td  <- tempdir()
  src <- system.file("extdata", "scf2022_mock_raw.rds", package = "scf")
  file.copy(src, file.path(td, "scf2022.rds"), overwrite = TRUE)
  scf2022 <- scf_load(2022, data_directory = td)
  
  scf2022 <- scf_update(scf2022, over50 = age > 50)
  
  freq <- scf_freq(scf2022, ~own)
  expect_named(freq, c("results", "imps", "aux"))
  expect_true(inherits(freq, "scf_freq"))
  
  mean <- scf_mean(scf2022, ~income)
  expect_named(mean, c("results", "imps", "aux", "verbose"))
  expect_true(inherits(mean, "scf_mean"))
  
  med <- scf_median(scf2022, ~income)
  expect_true(inherits(med, "scf_percentile"))
  
  unlink(file.path(td, "scf2022.rds"), force = TRUE)
})

test_that("Regression functions return valid model objects", {
  td  <- tempdir()
  src <- system.file("extdata", "scf2022_mock_raw.rds", package = "scf")
  file.copy(src, file.path(td, "scf2022.rds"), overwrite = TRUE)
  scf2022 <- scf_load(2022, data_directory = td)
  
  scf2022 <- scf_update(scf2022,
                        over50 = factor(age > 50),
                        log_inc = log(pmax(income, 1)),
                        own_bin = factor(own == 1)
  )
  
  model <- scf_ols(scf2022, networth ~ income + over50)
  
  # Required components exist
  expect_true(all(c("results", "imps", "fit", "call") %in% names(model)))
  
  # Optional: ensure new component exists
  expect_true("formula" %in% names(model))
  
  expect_true("estimate" %in% names(model$results))
  
  logit <- scf_logit(scf2022, own_bin ~ over50 + log_inc)
  
  expect_true(all(c("results", "imps", "fit", "call") %in% names(logit)))
  expect_true("formula" %in% names(logit))
  
  unlink(file.path(td, "scf2022.rds"), force = TRUE)
})


test_that("Plot functions generate ggplot objects", {
  td  <- tempdir()
  src <- system.file("extdata", "scf2022_mock_raw.rds", package = "scf")
  file.copy(src, file.path(td, "scf2022.rds"), overwrite = TRUE)
  scf2022 <- scf_load(2022, data_directory = td)
  
  p1 <- scf_plot_dbar(scf2022, ~own)
  expect_s3_class(p1, "ggplot")
  
  p2 <- scf_plot_cbar(scf2022, ~networth, ~own)
  expect_s3_class(p2, "ggplot")
  
  unlink(file.path(td, "scf2022.rds"), force = TRUE)
})

#' Histogram of a Continuous Variable in Multiply-Imputed SCF Data
#'
#' @description
#' Produces a histogram of a continuous SCF variable by binning across implicates,
#' pooling weighted bin counts using [scf_freq()], and plotting the result.
#' Values outside `xlim` are clamped into the nearest endpoint to ensure all
#' observations are included and replicate-weighted bins remain stable.
#'
#' @section Implementation:
#' This function bins a continuous variable (after clamping to `xlim` if supplied),
#' applies the same `cut()` breaks across implicates using [scf_update_by_implicate()],
#' and computes Rubin-pooled frequencies with [scf_freq()]. Results are filtered to
#' remove bins with undefined proportions and then plotted using `ggplot2::geom_col()`.
#'
#' The logic here is specific to operations where the bin assignment must be computed
#' **within** each implicate, not after pooling. This approach ensures consistent binning
#' and stable pooled estimation in the presence of multiply-imputed microdata.
#'
#' @param design A `scf_mi_survey` object from [scf_load()].
#' @param variable A one-sided formula indicating the numeric variable to plot.
#' @param bins Number of bins (default: 30).
#' @param xlim Optional numeric range. Values outside will be included in edge bins.
#' @param title Optional plot title.
#' @param xlab Optional x-axis label. Defaults to the variable name.
#' @param ylab Optional y-axis label. Defaults to "Weighted Count".
#' @param fill Fill color for bars (default: `"#0072B2"`).
#'
#' @return A `ggplot2` object representing the Rubin-pooled histogram.
#'
#' @examples
#' # Do not implement these lines in real analysis:
#' # Use functions `scf_download()` and `scf_load()`
#' td <- tempfile("plot_hist_")
#' dir.create(td)
#' 
#' src <- system.file("extdata", "scf2022_mock_raw.rds", package = "scf")
#' file.copy(src, file.path(td, "scf2022.rds"), overwrite = TRUE)
#' scf2022 <- scf_load(2022, data_directory = td)
#'
#' # Example for real analysis: Plot histogram of age
#' scf_plot_hist(scf2022, ~age, bins = 10)
#' 
#' # Do not implement these lines in real analysis: Cleanup for package check
#' unlink(td, recursive = TRUE, force = TRUE)
#'
#' @seealso [scf_freq()], [scf_plot_dbar()], [scf_plot_smooth()], [scf_update_by_implicate()]
#' @export
scf_plot_hist <- function(design, variable,
                          bins = 30,
                          xlim = NULL,
                          title = NULL,
                          xlab = NULL,
                          ylab = "Weighted Count",
                          fill = "#0072B2") {
  stopifnot(inherits(design, "scf_mi_survey"))
  if (isTRUE(attr(design, "mock"))) {
    warning("Mock data detected. Do not interpret results as valid SCF estimates.", call. = FALSE)
  }
  if (!is.numeric(bins) || length(bins) != 1L || !is.finite(bins) || bins < 1 || bins != as.integer(bins)) {
    stop("`bins` must be a single positive integer.")
  }
  
  varname <- all.vars(variable)[1]
  
  # Pull values from mi_design (not implicates)
  all_values <- unlist(lapply(design$mi_design, function(d) d$variables[[varname]]), use.names = FALSE)
  all_values <- all_values[is.finite(all_values)]
  if (length(all_values) == 0L) stop("No usable values found for this variable.")
  
  if (!is.null(xlim)) {
    if (!is.numeric(xlim) || length(xlim) != 2L || !all(is.finite(xlim)) || xlim[1] >= xlim[2]) {
      stop("`xlim` must be a numeric vector of length 2 with xlim[1] < xlim[2].")
    }
  }
  
  rng <- if (!is.null(xlim)) xlim else range(all_values, na.rm = TRUE)
  breaks <- seq(rng[1], rng[2], length.out = bins + 1)
  
  # Clamp and bin within each implicate
  design <- scf_update_by_implicate(design, function(df) {
    x <- df[[varname]]
    x <- pmin(pmax(x, rng[1]), rng[2])
    df$.binvar <- cut(x, breaks = breaks, include.lowest = TRUE, right = TRUE)
    df
  })
  
  # Weighted frequencies across bins
  freq <- scf_freq(design, ~.binvar, percent = FALSE)
  
  # Keep non-empty bins
  res <- freq$results
  res <- res[is.finite(res$proportion) & res$proportion > 0, , drop = FALSE]
  
  # Restore the intended numeric bin order after scf_freq() returns category labels.
  bin_levels <- levels(cut(
    pmin(pmax(all_values, rng[1]), rng[2]),
    breaks = breaks,
    include.lowest = TRUE,
    right = TRUE
  ))
  
  res$category <- factor(res$category, levels = bin_levels, ordered = TRUE)
  
  ggplot2::ggplot(res, ggplot2::aes(x = .data$category, y = .data$proportion)) +
    ggplot2::geom_col(fill = fill) +
    ggplot2::labs(
      title = if (is.null(title)) paste("Histogram of", varname) else title,
      x     = if (is.null(xlab))  varname                      else xlab,
      y     = ylab
    ) +
    scf_theme() +
    ggplot2::theme(axis.text.x = ggplot2::element_text(angle = 45, hjust = 1))
}

#' Extract Implicate-Level Estimates from SCF Results
#'
#' @description
#' Returns implicate-level outputs from SCF result objects produced by functions
#' in the `scf` suite. Supports result objects containing implicate-level data
#' frames, `svystat` summaries, or `svyglm` model fits.
#'
#' @param x A result object containing implicate-level estimates. Three types are supported:
#'   \describe{
#'     \item{Data frame results}{Objects from [scf_freq()], [scf_mean()], [scf_median()],
#'       [scf_percentile()], [scf_corr()], and [scf_xtab()], whose `$imps` slot contains
#'       a list of data frames.}
#'     \item{Survey statistic results}{Objects whose `$imps` slot contains `svystat`
#'       objects (e.g., raw outputs from `survey::svymean()`).}
#'     \item{Model results}{Objects from [scf_ols()], [scf_glm()], [scf_logit()], and
#'       [scf_quantreg()], whose `$imps` or `$models` slot contains `svyglm` fits.}
#'   }
#' @param long Logical. If TRUE, returns stacked data frame. If FALSE, returns list.
#'
#' @section Usage:
#' This function allows users to inspect how estimates vary across the SCF’s five implicates,
#' which is important for diagnostics, robustness checks, and transparent reporting.
#'
#' For example: 
#' ```r
#' scf_implicates(scf_mean(scf2022, ~income))
#' scf_implicates(scf_ols(scf2022, networth ~ age + income), long = TRUE)
#' ```
#' @return A list of implicate-level data frames, or a single stacked data frame if `long = TRUE`.
#'
#' @examples
#' # Do not implement these lines in real analysis:
#' # Use functions `scf_download()` and `scf_load()`
#' td <- tempfile("implicates_")
#' dir.create(td)
#' 
#' src <- system.file("extdata", "scf2022_mock_raw.rds", package = "scf")
#' file.copy(src, file.path(td, "scf2022.rds"), overwrite = TRUE)
#' scf2022 <- scf_load(2022, data_directory = td)
#'
#' # Example for real analysis: Extract implicate-level results
#' out <- scf_freq(scf2022, ~own)
#' scf_implicates(out, long = TRUE)
#' 
#' # Do not implement these lines in real analysis: Cleanup for package check
#' unlink(td, recursive = TRUE, force = TRUE)
#'
#' @importFrom stats coef vcov confint
#' @importFrom survey SE cv
#' @export
scf_implicates <- function(x, long = FALSE) {
  imps <- x$imps
  if (is.null(imps)) stop("No implicate-level estimates found in object.")

  # If all implicates are data.frames (e.g., from scf_freq, scf_xtab)
  if (all(sapply(imps, is.data.frame))) {
    out <- lapply(seq_along(imps), function(i) {
      df <- imps[[i]]
      df$implicate <- i
      # Add basic diagnostics if they exist
      if (all(c("est", "var") %in% names(df))) {
        df$estimate <- df$est
        df$se <- sqrt(df$var)
        df$lower <- df$estimate - 1.96 * df$se
        df$upper <- df$estimate + 1.96 * df$se
        df$cv <- ifelse(df$estimate == 0, NA_real_, df$se / abs(df$estimate))
      }
      df
    })
    return(if (long) do.call(rbind, out) else out)
  }

  # If all implicates are svystat-like objects
  if (all(sapply(imps, function(x) inherits(x, "svystat")))) {
    out <- lapply(seq_along(imps), function(i) {
      est <- imps[[i]]
      data.frame(
        implicate = i,
        estimate = coef(est),
        se = tryCatch(survey::SE(est), error = function(e) NA),
        lower = tryCatch(confint(est)[, 1], error = function(e) NA),
        upper = tryCatch(confint(est)[, 2], error = function(e) NA),
        cv = tryCatch(survey::cv(est), error = function(e) NA)
      )
    })
    return(if (long) do.call(rbind, out) else out)
  }

  # If all implicates are svyglm regression fits
  if (all(sapply(imps, function(x) inherits(x, "svyglm")))) {
    out <- lapply(seq_along(imps), function(i) {
      fit <- imps[[i]]
      coefs <- coef(fit)
      ses <- tryCatch(sqrt(diag(vcov(fit))), error = function(e) NA)
      data.frame(
        implicate = i,
        term = names(coefs),
        estimate = coefs,
        se = ses,
        lower = coefs - 1.96 * ses,
        upper = coefs + 1.96 * ses,
        cv = ifelse(coefs == 0, NA_real_, ses / abs(coefs))
      )
    })
    return(if (long) do.call(rbind, out) else out)
  }

  # Fallback: return as-is
  return(imps)
}

#' Cross-Tabulate Two Discrete Variables in Multiply-Imputed SCF Data
#'
#' @description
#' Computes replicate-weighted two-way cross-tabulations of two discrete variables
#' using multiply-imputed SCF data. Estimates cell proportions and standard errors,
#' with optional scaling of proportions by cell, row, or column. Results are pooled
#' across implicates using Rubin's Rules.
#'
#' @param scf A `scf_mi_survey` object, typically created by [scf_load()]. Must include five implicates with replicate weights.
#' @param rowvar A one-sided formula specifying the row variable (e.g., `~edcl`).
#' @param colvar A one-sided formula specifying the column variable (e.g., `~racecl`).
#' @param scale Character. Proportion basis: "cell" (default), "row", or "col".
#'
#' @return A list of class `"scf_xtab"` with:
#' \describe{
#'   \item{results}{Data frame with one row per cell. Columns: `row`, `col`, `prop`, `se`, `row_share`, `col_share`, `rowvar`, and `colvar`.}
#'   \item{matrices}{List of matrices: `cell` (default proportions), `row`, `col`, and `se`.}
#'   \item{imps}{List of implicate-level cell count tables.}
#'   \item{aux}{List with `rowvar` and `colvar` names.}
#' }
#'
#' @section Statistical Notes:
#' Implicate-level tables are created using `svytable()` on replicate-weighted designs.
#' Proportions are calculated as shares of total population estimates. Variance across
#' implicates is used to estimate uncertainty. Rubin's Rules are applied in simplified form.
#'
#' For technical details on pooling logic, see [scf_MIcombine()] or the SCF package manual.
#'
#' @examples
#' \donttest{
#' # Do not implement these lines in real analysis:
#' # Use functions `scf_download()` and `scf_load()`
#' td <- tempfile("xtab_")
#' dir.create(td)
#'
#' src <- system.file("extdata", "scf2022_mock_raw.rds", package = "scf")
#' file.copy(src, file.path(td, "scf2022.rds"), overwrite = TRUE)
#' scf2022 <- scf_load(2022, data_directory = td)
#'
#' # Example for real analysis: Cross-tabulate ownership by sex
#' suppressWarnings(scf_xtab(scf2022, ~own, ~hhsex, scale = "row"))
#'
#' # Do not implement these lines in real analysis: Cleanup for package check
#' unlink(td, recursive = TRUE, force = TRUE)
#' }
#'
#' @importFrom stats as.formula ave
#' @export
scf_xtab <- function(scf, rowvar, colvar, scale = "cell") {
  scale <- match.arg(scale, choices = c("cell", "row", "col"))

  if (isTRUE(attr(scf, "mock"))) {
    warning("Mock data detected. Do not interpret results as valid SCF estimates.", call. = FALSE)
  }


  if (!inherits(scf, "scf_mi_survey"))
    stop("Input must be an 'scf_mi_survey' object.")

  rowname <- all.vars(rowvar)[1]
  colname <- all.vars(colvar)[1]
  designs <- scf$mi_design
  nimp <- length(designs)

  imp_tables <- vector("list", nimp)
  row_levels <- NULL
  col_levels <- NULL
  row_label <- if (!is.null(attr(scf$data[[rowname]], "label"))) attr(scf$data[[rowname]], "label") else rowname
  col_label <- if (!is.null(attr(scf$data[[colname]], "label"))) attr(scf$data[[colname]], "label") else colname

  for (i in seq_len(nimp)) {
    d <- designs[[i]]
    d$variables[[rowname]] <- factor(d$variables[[rowname]])
    d$variables[[colname]] <- factor(d$variables[[colname]])

    if (is.null(row_levels)) row_levels <- levels(d$variables[[rowname]])
    if (is.null(col_levels)) col_levels <- levels(d$variables[[colname]])

    tbl <- try(survey::svytable(as.formula(paste("~", rowname, "+", colname)), d), silent = TRUE)
    if (inherits(tbl, "try-error")) next

    full_tbl <- matrix(0, nrow = length(row_levels), ncol = length(col_levels),
                       dimnames = list(row_levels, col_levels))
    full_tbl[rownames(tbl), colnames(tbl)] <- tbl
    imp_tables[[i]] <- full_tbl
  }

  imp_tables <- Filter(Negate(is.null), imp_tables)
  if (length(imp_tables) < 2L) stop("Too few valid implicates.")

  pooled_table <- Reduce("+", imp_tables) / length(imp_tables)
  total_pop <- sum(pooled_table)

  grid <- expand.grid(row = row_levels, col = col_levels, stringsAsFactors = FALSE)
  grid$rowvar <- rowname
  grid$colvar <- colname

  imp_counts <- lapply(seq_along(imp_tables), function(i) {
    tab <- imp_tables[[i]]
    data.frame(
      row = rep(rownames(tab), times = ncol(tab)),
      col = rep(colnames(tab), each = nrow(tab)),
      count = as.vector(tab),
      implicate = i,
      stringsAsFactors = FALSE
    )
  })
  long <- do.call(rbind, imp_counts)

  pooled_stats <- lapply(split(long, list(long$row, long$col)), function(df) {
    x <- df$count
    m <- length(x)
    qbar <- mean(x)
    b <- var(x)
    se <- sqrt((1 + 1 / m) * b)
    data.frame(prop = qbar / total_pop, se = se / total_pop, stringsAsFactors = FALSE)
  })

  stat_df <- cbind(grid[, c("row", "col", "rowvar", "colvar")], do.call(rbind, pooled_stats))

  stat_df$row_share <- ave(stat_df$prop, stat_df$row, FUN = function(x) x / sum(x))
  stat_df$col_share <- ave(stat_df$prop, stat_df$col, FUN = function(x) x / sum(x))

  to_matrix <- function(df, value_col) {
    xtabs(df[[value_col]] ~ df$row + df$col)
  }

  matrices <- list(
    cell = to_matrix(stat_df, "prop"),
    row  = to_matrix(stat_df, "row_share"),
    col  = to_matrix(stat_df, "col_share"),
    se   = to_matrix(stat_df, "se")
  )

  out <- list(
    results = stat_df,
    matrices = matrices,
    imps = imp_tables,
    aux = list(rowvar = rowname, colvar = colname, rowlabel = row_label, collabel = col_label, scale = scale)
  )
  class(out) <- "scf_xtab"
  return(out)
}

#' @export
print.scf_xtab <- function(x, ...) {
  cat("SCF Cross-Tabulation\n")
  cat("Row Variable:", x$aux$rowlabel, "| Column Variable:", x$aux$collabel, "\n")
  cat("Displayed as:", x$aux$scale, "proportions (percent)\n\n")

  mat <- round(100 * x$matrices[[x$aux$scale]], 2)
  dimnames(mat) <- list(
    paste0(x$aux$rowlabel, ": ", rownames(mat)),
    paste0(x$aux$collabel, ": ", colnames(mat))
  )
  print(mat)
  invisible(x)
}

#' @export
summary.scf_xtab <- function(object, ...) {
  cat("Summary of SCF Cross-Tabulation\n")
  cat("Row Variable:", object$aux$rowlabel, "\n")
  cat("Col Variable:", object$aux$collabel, "\n")
  cat("Proportion Scale:", object$aux$scale, "\n\n")

  cat("Proportions (Percent):\n")
  print(round(100 * object$matrices[[object$aux$scale]], 2))

  cat("\nStandard Errors (Proportions):\n")
  print(round(object$matrices$se, 4))
  invisible(object)
}

#' Estimate Mean in Multiply-Imputed SCF Data
#'
#' @description
#' Returns the population-level estimate of a continuous variable's weighted
#' mean across the Survey's five implicates. Use this operation to derive an
#' estimate of a population's 'typical' or 'average' score on a continuous
#' variable.
#'
#' @section Details:
#' The mean is a measure of central tendency that represents the arithmetic average of a distribution.
#' It is most appropriate when the distribution is symmetric and not heavily skewed.
#' Unlike the median, the mean is sensitive to extreme values, which may distort interpretation in the presence of outliers.
#' Use this function to describe the “typical” value of a continuous variable in the population or within subgroups.
#'
#' @param scf A scf_mi_survey object created with [scf_load()]. Must contain five replicate-weighted implicates.
#' @param var A one-sided formula identifying the continuous variable to summarize (e.g., ~networth).
#' @param by Optional one-sided formula specifying a discrete grouping variable for stratified means.
#' @param verbose Logical. If TRUE, include implicate-level results in print output. Default is FALSE.
#'
#' @return A list of class "scf_mean" with:
#' \describe{
#'   \item{results}{Pooled estimates with standard errors and range across implicates. One row per group, or one row total.}
#'   \item{imps}{A named list of implicate-level estimates.}
#'   \item{aux}{Variable and group metadata.} 
#' }
#'
#' @examples
#' # Do not implement these lines in real analysis:
#' # Use functions `scf_download()` and `scf_load()`
#' td <- tempfile("mean_")
#' dir.create(td)
#' 
#' src <- system.file("extdata", "scf2022_mock_raw.rds", package = "scf")
#' file.copy(src, file.path(td, "scf2022.rds"), overwrite = TRUE)
#' scf2022 <- scf_load(2022, data_directory = td)
#'
#' # Example for real analysis: Estimate means
#' scf_mean(scf2022, ~networth)
#' scf_mean(scf2022, ~networth, by = ~edcl)
#' 
#' # Do not implement these lines in real analysis: Cleanup for package check
#' unlink(td, recursive = TRUE, force = TRUE)
#'
#'
#' @seealso [scf_median()], [scf_percentile()], [scf_xtab()], [scf_plot_dist()]
#'
#' @export
scf_mean <- function(scf, var, by = NULL, verbose = FALSE) {
  
  if (!inherits(scf, "scf_mi_survey") ||
      !is.list(scf$mi_design) ||
      !all(sapply(scf$mi_design, inherits, "svyrep.design"))) {
    stop("Input must be a 'scf_mi_survey' object with replicate-weighted implicates.")
  }
  
  if (isTRUE(attr(scf, "mock"))) {
    warning("Mock data detected. Do not interpret results as valid SCF estimates.", call. = FALSE)
  }
  
  varname <- all.vars(var)[1]
  byname  <- if (!is.null(by)) all.vars(by)[1] else NULL
  designs <- scf$mi_design
  nimp    <- length(designs)
  
  if (is.null(byname)) {
    
    imp_results <- lapply(seq_len(nimp), function(i) {
      d <- designs[[i]]
      survey::svymean(var, d)
    })
    
    imp_estimates <- lapply(seq_len(nimp), function(i) {
      est <- imp_results[[i]]
      data.frame(
        implicate = i,
        group     = "All",
        estimate  = as.numeric(coef(est)),
        se        = sqrt(diag(vcov(est))),
        stringsAsFactors = FALSE
      )
    })
    
    pooled <- scf_MIcombine(imp_results)
    
    est_pooled <- as.numeric(pooled$coefficients)
    se_pooled  <- sqrt(diag(pooled$variance))
    
    x_all <- sapply(imp_results, function(obj) as.numeric(coef(obj)))
    
    out <- data.frame(
      variable = varname,
      estimate = est_pooled,
      se       = se_pooled,
      min      = min(x_all),
      max      = max(x_all),
      stringsAsFactors = FALSE
    )
    
  } else {
    
    for (i in seq_len(nimp)) {
      designs[[i]]$variables[[byname]] <- factor(designs[[i]]$variables[[byname]])
    }
    groups <- levels(designs[[1]]$variables[[byname]])
    
    imp_results <- lapply(seq_len(nimp), function(i) {
      d <- designs[[i]]
      lapply(groups, function(g) {
        d_g <- subset(d, d$variables[[byname]] == g)
        survey::svymean(var, d_g)
      })
    })
    
    imp_estimates <- lapply(seq_len(nimp), function(i) {
      res_i <- imp_results[[i]]
      do.call(rbind, lapply(seq_along(groups), function(j) {
        est <- res_i[[j]]
        data.frame(
          implicate = i,
          group     = groups[j],
          estimate  = as.numeric(coef(est)),
          se        = sqrt(diag(vcov(est))),
          stringsAsFactors = FALSE
        )
      }))
    })
    
    out <- do.call(rbind, lapply(seq_along(groups), function(j) {
      g <- groups[j]
      
      res_list_g <- lapply(seq_len(nimp), function(i) {
        imp_results[[i]][[j]]
      })
      
      pooled_g <- scf_MIcombine(res_list_g)
      
      est_pooled_g <- as.numeric(pooled_g$coefficients)
      se_pooled_g  <- sqrt(diag(pooled_g$variance))
      
      x_g <- sapply(res_list_g, function(obj) as.numeric(coef(obj)))
      
      data.frame(
        group    = g,
        variable = varname,
        estimate = est_pooled_g,
        se       = se_pooled_g,
        min      = min(x_g),
        max      = max(x_g),
        stringsAsFactors = FALSE
      )
    }))
    
    out <- out[, c("group", "variable", "estimate", "se", "min", "max")]
  }
  
  names(imp_estimates) <- paste0("imp", seq_len(nimp))
  
  out_obj <- list(
    results = out,
    imps    = imp_estimates,
    aux     = list(varname = varname, byname = byname, year = scf$year),
    verbose = verbose
  )
  class(out_obj) <- "scf_mean"
  return(out_obj)
}

#' @export
print.scf_mean <- function(x, ...) {
  dollar_label <- if (isTRUE(attr(x, "deflated")))
    sprintf(" (%d$)", attr(x, "base_year")) else ""
  cat(sprintf(
    "Multiply-Imputed, Replicate-Weighted Mean Estimate%s\n\n",
    dollar_label
  ))
  print(x$results, row.names = FALSE, ...)

  if (isTRUE(x$verbose)) {
    cat("\nImplicate-Level Estimates:\n\n")
    imp_df <- do.call(rbind, x$imps)
    print(imp_df, row.names = FALSE)
  }

  invisible(x)
}
#' @export
summary.scf_mean <- function(object, ...) {
  cat("Summary of Multiply-Imputed Mean Estimate\n\n")

  # Print pooled estimates with standard errors
  cat("Pooled Estimates:\n")
  print(
    format(object$results, digits = 4, nsmall = 2),
    row.names = FALSE,
    ...
  )

  # Optionally print implicate-level estimates
  cat("\nImplicate-Level Estimates:\n")
  imp_df <- do.call(rbind, object$imps)
  print(
    format(imp_df, digits = 4, nsmall = 2),
    row.names = FALSE
  )

  invisible(object)
}

#' Bar Plot of Summary Statistics by Grouping Variable in SCF Data
#'
#' @description
#' Computes and plots a grouped summary statistic (either a mean, median, or
#' quantile) for a continuous variable across a discrete factor. Estimates are
#' pooled across implicates using [scf_mean()], [scf_median()], or
#' [scf_percentile()]. Use this function to visualize the bivariate relationship
#' between a discrete and a continuous variable.
#'
#' @section Implementation:
#' The user specifies a continuous outcome (`yvar`) and a discrete grouping
#' variable (`xvar`) via one-sided formulas. Group means are plotted by default.
#' Medians or other percentiles can be specified via the `stat` argument.
#'
#' Results are plotted using `ggplot2::geom_col()`, styled with [scf_theme()],
#' and optionally customized with additional arguments (e.g., axis labels,
#' color, angles).
#'
#' @param design A `scf_mi_survey` object from [scf_load()].
#' @param yvar One-sided formula for the continuous variable (e.g., `~networth`).
#' @param xvar One-sided formula for the grouping variable (e.g., `~racecl`).
#' @param stat `"mean"` (default), `"median"`, or a quantile (numeric between 0 and 1).
#' @param title Plot title (optional).
#' @param xlab X-axis label (optional).
#' @param ylab Y-axis label (optional).
#' @param fill Bar fill color. Default is `"#0072B2"`.
#' @param angle Angle of x-axis labels. Default is 30.
#' @param label_map Optional named vector to relabel x-axis category labels.
#'
#' @return A `ggplot2` object.
#'
#' @seealso [scf_mean()], [scf_median()], [scf_percentile()], [scf_theme()]
#'
#' @examples
#' # Do not implement these lines in real analysis:
#' # Use functions `scf_download()` and `scf_load()`
#' td <- tempfile("plot_cbar_")
#' dir.create(td)
#' 
#' src <- system.file("extdata", "scf2022_mock_raw.rds", package = "scf")
#' file.copy(src, file.path(td, "scf2022.rds"), overwrite = TRUE)
#' scf2022 <- scf_load(2022, data_directory = td)
#'
#' # Example for real analysis: Plot mean net worth by education level
#' scf_plot_cbar(scf2022, ~networth, ~edcl, stat = "mean")
#'
#' # Example for real analysis: Visualize 90th percentile of income by education
#' scf_plot_cbar(scf2022, ~income, ~edcl, stat = 0.9, fill = "#D55E00")
#' 
#' # Do not implement these lines in real analysis: Cleanup for package check
#' unlink(td, recursive = TRUE, force = TRUE)
#'
#' @export
scf_plot_cbar <- function(design, yvar, xvar,
                          stat = "mean",
                          title = NULL,
                          xlab = NULL,
                          ylab = NULL,
                          fill = "#0072B2",
                          angle = 30,
                          label_map = NULL) {

  stopifnot(inherits(design, "scf_mi_survey"))
  stopifnot(inherits(yvar, "formula"), inherits(xvar, "formula"))

  if (isTRUE(attr(design, "mock"))) {
    warning("Mock data detected. Do not interpret results as valid SCF estimates.", call. = FALSE)
  }


  `%||%` <- function(a, b) if (!is.null(a)) a else b

  yname <- all.vars(yvar)[1]
  xname <- all.vars(xvar)[1]

  # Validate that xvar is discrete
  xvals <- design$mi_design[[1]]$variables[[xname]]
  if (is.numeric(xvals) && length(unique(xvals)) > 25) {
    stop("Grouping variable appears continuous. Please use a factor or discrete variable.")
  }

  # Capture factor levels from first implicate
  x_levels <- if (is.factor(xvals)) levels(xvals) else unique(xvals)

  # Dispatch to appropriate estimator
  results <- switch(
    as.character(stat),
    mean   = scf_mean(design, yvar, by = xvar),
    median = scf_median(design, yvar, by = xvar),
    {
      if (is.numeric(stat) && stat > 0 && stat < 1) {
        scf_percentile(design, yvar, q = stat, by = xvar)
      } else {
        stop("`stat` must be 'mean', 'median', or a quantile between 0 and 1.")
      }
    }
  )

  df <- results$results
  if (!all(c("group", "estimate") %in% names(df))) {
    stop("Estimation failed: expected 'group' and 'estimate' columns not found.")
  }

  # Coerce to factor and preserve ordering
  df$group <- factor(df$group, levels = x_levels)

  # Optional x-axis relabeling
  if (!is.null(label_map)) {
    df$group <- factor(df$group,
                       levels = names(label_map),
                       labels = unname(label_map))
  }

  # Auto-generate y-axis label if not provided
  y_label <- ylab %||% switch(
    as.character(stat),
    "mean"   = paste("Mean of", yname),
    "median" = paste("Median of", yname),
    paste0(100 * stat, "th Percentile of ", yname)
  )

  # Construct ggplot
  ggplot2::ggplot(df, ggplot2::aes(x = group, y = estimate)) +
    ggplot2::geom_col(fill = fill) +
    ggplot2::labs(
      title = title %||% paste("Distribution of", yname, "by", xname),
      x = xlab %||% xname,
      y = y_label
    ) +
    scf_theme() +
    ggplot2::theme(axis.text.x = .scf_axis_text_x(angle))
}

#' Test a Proportion in SCF Data
#'
#' @description
#' Tests a binary variable's proportion against a null hypothesis (one-sample),
#' or compares proportions across two groups (two-sample). Supports two-sided,
#' less-than, or greater-than alternatives. 
#'
#' @param design A `scf_mi_survey` object created by [scf_load()]. Must contain replicate-weighted implicates.
#' @param var A one-sided formula indicating a binary variable (e.g., `~rich`).
#' @param group Optional one-sided formula indicating a binary grouping variable (e.g., `~female`). If omitted, a one-sample test is performed.
#' @param p Null hypothesis value. Defaults to `0.5` for one-sample, `0` for two-sample tests.
#' @param alternative Character. One of `"two.sided"` (default), `"less"`, or `"greater"`.
#' @param conf.level Confidence level for the confidence interval. Default is `0.95`.
#'
#' @return An object of class `"scf_prop_test"` with:
#' \describe{
#'   \item{results}{A data frame with the pooled estimate, standard error, z-statistic, p-value, confidence interval, and significance stars.}
#'   \item{proportions}{(Only in two-sample tests) A data frame of pooled proportions by group.}
#'   \item{fit}{A list describing the method, null value, alternative hypothesis, and confidence level.}
#' }
#'
#' @section Statistical Notes:
#' Proportions are computed in each implicate using weighted means, and variances are approximated under the binomial model.
#' Rubin’s Rules are applied to pool point estimates and standard errors. For pooling details, see [scf_MIcombine()].
#'
#' @examples
#' # Do not implement these lines in real analysis:
#' # Use functions `scf_download()` and `scf_load()`
#' td <- tempfile("proptest_")
#' dir.create(td)
#' 
#' src <- system.file("extdata", "scf2022_mock_raw.rds", package = "scf")
#' file.copy(src, file.path(td, "scf2022.rds"), overwrite = TRUE)
#' scf2022 <- scf_load(2022, data_directory = td)
#'
#' # Wrangle data for example
#' scf2022 <- scf_update(scf2022,
#'   rich   = networth > 1e6,
#'   female = factor(hhsex, levels = 1:2, labels = c("Male","Female")),
#'   over50 = age > 50
#' )
#'
#' # Example for real analysis: One-sample test
#' scf_prop_test(scf2022, ~rich, p = 0.10)
#'
#' # Example for real analysis: Two-sample test
#' scf_prop_test(scf2022, ~rich, ~female, alternative = "less")
#' 
#' # Do not implement these lines in real analysis: Cleanup for package check
#' unlink(td, recursive = TRUE, force = TRUE)
#'
#' 
#' @seealso [scf_ttest()], [scf_mean()], [scf_MIcombine()]
#'
#' @export


scf_prop_test <- function(design, var, group = NULL, p = 0.5,
                          alternative = c("two.sided", "less", "greater"),
                          conf.level = 0.95) {

  stopifnot(inherits(design, "scf_mi_survey"))
  stopifnot(inherits(var, "formula"))
  alternative <- match.arg(alternative)
  varname <- all.vars(var)[1]

  if (any(sapply(design$mi_design, function(d) {
    v <- d$variables[[varname]]
    !is.logical(v) && !all(v %in% c(0, 1, NA))
  }))) {
    stop("Test variable must be logical or 0/1 coded.")
  }

  if (isTRUE(attr(design, "mock"))) {
    warning("Mock data detected. Do not interpret results as valid SCF estimates.", call. = FALSE)
  }


  if (is.null(group)) {
    # One-sample
    props <- sapply(design$mi_design, function(d) {
      x <- d$variables[[varname]]
      w <- weights(d, type = "sampling")
      valid <- !is.na(x)
      x <- x[valid]
      w <- w[valid]
      p_hat <- weighted.mean(x, w)
      var_hat <- p_hat * (1 - p_hat) / sum(w)
      c(p_hat = p_hat, var = var_hat)
    })
    est <- mean(props["p_hat", ])
    within <- mean(props["var", ])
    between <- var(props["p_hat", ])
    total_var <- within + (1 + 1 / length(design$mi_design)) * between
    se <- sqrt(total_var)
    zval <- (est - p) / se
    pval <- switch(alternative,
                   "two.sided" = 2 * pnorm(-abs(zval)),
                   "less" = pnorm(zval),
                   "greater" = 1 - pnorm(zval)
    )
    ci <- switch(alternative,
                 "two.sided" = est + c(-1, 1) * qnorm(1 - (1 - conf.level) / 2) * se,
                 "less" = c(-Inf, est + qnorm(conf.level) * se),
                 "greater" = c(est - qnorm(conf.level) * se, Inf)
    )
    prop_df <- NULL

  } else {
    # Two-sample
    stopifnot(inherits(group, "formula"))
    groupname <- all.vars(group)[1]

    levels_detected <- levels(factor(design$mi_design[[1]]$variables[[groupname]]))
    if (length(levels_detected) != 2) {
      stop("Grouping variable must have exactly two levels.")
    }

    stats <- lapply(design$mi_design, function(d) {
      df <- d$variables
      w <- weights(d, type = "sampling")
      g <- factor(df[[groupname]], levels = levels_detected)
      x <- df[[varname]]
      valid <- !is.na(x) & !is.na(g)
      x <- x[valid]
      g <- g[valid]
      w <- w[valid]

      if (length(x) == 0) return(NULL)

      means <- tapply(seq_along(x), g, function(idx) weighted.mean(x[idx], w[idx]))
      ns <- tapply(seq_along(x), g, function(idx) sum(w[idx]))
      vars <- means * (1 - means) / ns

      if (anyNA(means) || anyNA(vars) || length(means) != 2 || length(vars) != 2) return(NULL)

      c(p1 = unname(means[1]), p2 = unname(means[2]), diff = unname(means[1] - means[2]), var = unname(sum(vars)))

    })

    stats <- Filter(Negate(is.null), stats)
    if (length(stats) < 2) stop("Too few valid implicates.")

    ests <- do.call(rbind, stats)
    est <- mean(ests[, "diff"])
    within <- mean(ests[, "var"])
    between <- var(ests[, "diff"])
    total_var <- within + (1 + 1 / length(ests)) * between
    se <- sqrt(total_var)
    zval <- (est - p) / se
    pval <- switch(alternative,
                   "two.sided" = 2 * pnorm(-abs(zval)),
                   "less" = pnorm(zval),
                   "greater" = 1 - pnorm(zval)
    )
    ci <- switch(alternative,
                 "two.sided" = est + c(-1, 1) * qnorm(1 - (1 - conf.level) / 2) * se,
                 "less" = c(-Inf, est + qnorm(conf.level) * se),
                 "greater" = c(est - qnorm(conf.level) * se, Inf)
    )
    prop_df <- data.frame(
      group = levels_detected,
      proportion = c(mean(ests[, "p1"]), mean(ests[, "p2"]))
    )
  }

  results <- data.frame(
    estimate = est,
    std.error = se,
    z.value = zval,
    p.value = pval,
    conf.low = ci[1],
    conf.high = ci[2],
    stars = cut(pval,
                breaks = c(-Inf, 0.001, 0.01, 0.05, 0.10, Inf),
                labels = c("***", "**", "*", "^", ""), right = FALSE
    )
  )

  structure(list(
    results = results,
    proportions = prop_df,
    fit = list(
      method = if (is.null(group)) "One-sample proportion test" else "Two-sample proportion test",
      null.value = p,
      conf.level = conf.level,
      alternative = alternative
    )
  ), class = "scf_prop_test")
}
#' @export
print.scf_prop_test <- function(x, digits = 4, ...) {
  cat("\n", x$fit$method, "\n", sep = "")
  cat("Null hypothesis: proportion ",
      if (x$fit$method == "Two-sample proportion test") "difference = " else "= ",
      x$fit$null.value, "\n", sep = "")
  cat("Alternative hypothesis: ", x$fit$alternative, "\n", sep = "")
  cat("Confidence level: ", x$fit$conf.level * 100, "%\n\n", sep = "")

  num_cols <- sapply(x$results, is.numeric)
  rounded <- x$results
  rounded[num_cols] <- round(rounded[num_cols], digits = digits)
  print(rounded, row.names = FALSE)

  if (!is.null(x$proportions)) {
    cat("\nEstimated group proportions:\n")
    prop_rounded <- x$proportions
    num_cols <- sapply(prop_rounded, is.numeric)
    prop_rounded[num_cols] <- round(prop_rounded[num_cols], digits = digits)
    print(prop_rounded, row.names = FALSE)
  }

  invisible(x)
}

#' Estimate a Quantile Regression Model on SCF Microdata
#'
#' Fits a survey-weighted quantile regression to each implicate of
#' multiply-imputed SCF data and pools coefficients and standard errors using
#' Rubin's Rules.
#'
#' @description
#' `scf_quantreg()` estimates a linear quantile regression at a user-specified
#' quantile (`tau`) across the five SCF implicates. Each implicate is fit
#' independently via [quantreg::rq()] using the SCF final sampling weights.
#' Coefficient estimates and variance-covariance matrices are then pooled across
#' implicates using [scf_MIcombine()].
#'
#' Unlike [scf_ols()], which models the conditional mean, quantile regression
#' models the conditional quantile of the outcome distribution. This makes it
#' especially useful for analyzing wealth and income data, which are highly
#' right-skewed: the conditional median (`tau = 0.5`) and upper quantiles
#' (`tau = 0.75`, `tau = 0.90`) describe the distribution more completely than
#' the mean alone.
#'
#' @section Standard Error Methods:
#'
#' The `se` argument controls how within-implicate variance is estimated.
#' All methods produce a variance-covariance matrix passed to [scf_MIcombine()].
#'
#' \describe{
#'   \item{`"replicate"` (recommended)}{Replication-based variance estimation.
#'     For each implicate, the model is re-fit using each of the SCF's 999
#'     replicate weight vectors. Variance is accumulated as the weighted sum of
#'     squared deviations from the full-weight estimate, matching the SCF's own
#'     published variance methodology via [survey::withReplicates()].
#'     This method is theoretically preferred for quantile regression because
#'     replication-based variance estimators are consistent for sample quantiles,
#'     whereas analytical (sandwich) estimators are not guaranteed consistent for
#'     nonsmooth statistics (Rust and Rao, 1996).
#'     Computationally intensive (~5,000 model fits for five implicates).}
#'   \item{`"nid"` (default)}{Non-iid sandwich estimator. Allows the
#'     conditional sparsity (density of the error distribution at the quantile)
#'     to vary across observations. Appropriate when the shape of the error
#'     distribution differs across the covariate space, which is typical for
#'     skewed outcomes such as wealth and income. Unreliable near quantiles
#'     with high mass points (e.g., `tau <= 0.25` when net worth has substantial
#'     mass at zero); use `se = "replicate"` in such cases.}
#'   \item{`"iid"`}{Assumes identically distributed errors, i.e., constant
#'     sparsity across all observations. Implements the covariance formula from
#'     Koenker and Bassett (1978, Theorem 4.2): \eqn{[\theta(1-\theta)/f(\xi(\theta))^2] Q^{-1}},
#'     where \eqn{f(\xi(\theta))} is the density of the error distribution at its
#'     \eqn{\theta}-quantile and \eqn{Q = \lim T^{-1}X'X}. Fastest analytical
#'     option; subject to the same mass-point caveat as `"nid"`.}
#'   \item{`"ker"`}{Kernel smoothing estimate of the conditional sparsity.
#'     More data-adaptive than `"nid"` but slower. Suitable for large samples.}
#'   \item{`"boot"`}{Pairs bootstrap over observations. Provides
#'     distribution-free variance estimates but is the slowest analytical
#'     option. Useful for robustness checks.}
#' }
#'
#' @section Goodness of Fit:
#'
#' The `fit` component of the returned object contains per-quantile goodness-of-fit
#' measures following Koenker and Machado (1999). These are computed by comparing
#' the full model to an intercept-only null model at the same `tau`.
#'
#' \describe{
#'   \item{`rho`}{Mean minimized weighted sum of absolute residuals
#'     (\eqn{\sum \rho_\tau(y_i - x_i'\hat\beta)}) from the full model, averaged
#'     across implicates. This is the quantile regression analog of the residual
#'     sum of squares.}
#'   \item{`rho_null`}{Same quantity from the intercept-only null model. Larger
#'     values relative to `rho` indicate greater explanatory power.}
#'   \item{`r1`}{The Koenker-Machado R\eqn{^1(\tau)} statistic:
#'     \eqn{1 - \bar{V}(\tau) / \tilde{V}(\tau)}, where \eqn{\bar{V}} is the
#'     mean full-model objective and \eqn{\tilde{V}} is the mean null-model
#'     objective, each averaged across implicates. Ranges from 0 to 1. Measures
#'     the proportional reduction in the weighted sum of absolute residuals due
#'     to the covariates at quantile `tau`. This is a *local* measure for the
#'     specific quantile estimated; it is not a global summary of fit across
#'     the distribution.}
#'   \item{`r1_adj`}{Degrees-of-freedom-adjusted R\eqn{^1(\tau)}:
#'     \eqn{1 - (1 - R^1) \cdot n / (n - p)}, where \eqn{n} is the mean
#'     number of observations and \eqn{p} is the number of estimated parameters.
#'     Penalizes model complexity analogously to adjusted R\eqn{^2} in OLS.
#'     Note: this adjustment is not derived from the asymptotic theory in
#'     Koenker and Machado (1999) and should be interpreted descriptively.}
#'   \item{`nobs`}{Integer vector of per-implicate sample sizes.}
#'   \item{`nobs_mean`}{Mean sample size across successful implicates.}
#' }
#'
#' @section Implementation:
#' For each implicate:
#' \enumerate{
#'   \item Extracts the survey data and final sampling weights from the
#'     `svyrep.design` object.
#'   \item Fits [quantreg::rq()] at quantile `tau` with the final weights.
#'   \item Estimates the within-implicate variance-covariance matrix using
#'     the method specified by `se`.
#'   \item Fits an intercept-only [quantreg::rq()] at the same `tau` and
#'     weights to obtain the null-model objective value for goodness-of-fit.
#' }
#' SCF public-use microdata contain no missing values; each implicate is a
#' complete dataset. Pooled estimates follow Rubin's Rules (see
#' [scf_MIcombine()]): the total variance combines within-implicate sampling
#' uncertainty and between-implicate imputation uncertainty.
#'
#' @param object A `scf_mi_survey` object created with [scf_load()] or
#'   [scf_design()].
#' @param formula A model formula specifying the outcome and predictors,
#'   e.g., `networth ~ age + factor(edcl)`. The outcome should be numeric.
#' @param tau Numeric scalar in (0, 1) specifying the quantile to estimate.
#'   Defaults to `0.5` (median regression). To estimate multiple quantiles,
#'   call `scf_quantreg()` separately for each. Analytical SE methods (`"nid"`,
#'   `"iid"`, `"ker"`, `"boot"`) may be unreliable at quantiles with high mass
#'   points; prefer `se = "replicate"` in such cases.
#' @param se Character string specifying the standard error estimation method
#'   for within-implicate variance. One of `"nid"` (default), `"iid"`,
#'   `"ker"`, `"boot"`, or `"replicate"`. See the **Standard Error Methods**
#'   section for details. `"replicate"` is theoretically preferred for quantile
#'   regression but is computationally intensive.
#' @param ... Additional arguments passed to [quantreg::rq()].
#'
#' @return An object of class `"scf_quantreg"` and `"scf_model_result"` with:
#' \describe{
#'   \item{`results`}{A data frame of pooled coefficients, standard errors,
#'     t-values, p-values, and significance stars.}
#'   \item{`tau`}{The quantile estimated.}
#'   \item{`se_method`}{The SE method used.}
#'   \item{`fit`}{A list of goodness-of-fit statistics. See the
#'     **Goodness of Fit** section for details. Components: `rho`,
#'     `rho_null`, `r1`, `r1_adj`, `nobs`, `nobs_mean`.}
#'   \item{`models`}{A list of implicate-level `rq` model objects for
#'     direct inspection.}
#'   \item{`fit_errors`}{Character vector of any implicate-level errors.}
#'   \item{`call`}{The matched call.}
#'   \item{`formula`}{The model formula.}
#' }
#'
#' @examples
#' # Do not implement these lines in real analysis:
#' # Use functions `scf_download()` and `scf_load()`
#' td <- tempfile("qreg_")
#' dir.create(td)
#'
#' src <- system.file("extdata", "scf2022_mock_raw.rds", package = "scf")
#' file.copy(src, file.path(td, "scf2022.rds"), overwrite = TRUE)
#' scf2022 <- scf_load(2022, data_directory = td)
#'
#' # Example for real analysis: median regression of net worth on age and education
#' m_med <- scf_quantreg(scf2022, networth ~ age + factor(edcl), tau = 0.5)
#' summary(m_med)
#'
#' # Access goodness-of-fit statistics
#' m_med$fit$r1
#' m_med$fit$r1_adj
#'
#' # Example for real analysis: 75th-percentile regression
#' m_75 <- scf_quantreg(scf2022, networth ~ age + factor(edcl), tau = 0.75)
#' summary(m_75)
#'
#' # Do not implement these lines in real analysis: Cleanup for package check
#' unlink(td, recursive = TRUE, force = TRUE)
#'
#' @seealso [scf_ols()], [scf_glm()], [scf_MIcombine()], [quantreg::rq()]
#'
#' @references
#' Koenker R, Bassett G. Regression quantiles. \emph{Econometrica}.
#'   1978;46(1):33--50. \doi{10.2307/1913643}
#'
#' Koenker R, Machado JAF. Goodness of fit and related inference processes
#'   for quantile regression. \emph{Journal of the American Statistical
#'   Association}. 1999;94(448):1296--1310. \doi{10.2307/2669943}
#'
#' Rust KF, Rao JNK. Variance estimation for complex surveys using replication
#'   techniques. \emph{Statistical Methods in Medical Research}.
#'   1996;5(3):283--310. \doi{10.1177/096228029600500305}
#'
#' @importFrom stats weights pt coef as.formula
#' @importFrom survey withReplicates
#' @export
scf_quantreg <- function(object, formula, tau = 0.5,
                         se = c("nid", "iid", "ker", "boot", "replicate"),
                         ...) {
  
  if (!inherits(object, "scf_mi_survey"))
    stop("Input must be of class 'scf_mi_survey'.")
  if (!inherits(formula, "formula"))
    stop("'formula' must be a formula object.")
  if (!requireNamespace("quantreg", quietly = TRUE))
    stop("Package 'quantreg' is required.")
  if (!is.numeric(tau) || length(tau) != 1 || tau <= 0 || tau >= 1)
    stop("'tau' must be a single numeric value strictly between 0 and 1.")
  
  se <- match.arg(se)
  
  if (isTRUE(attr(object, "mock")))
    warning("Mock data detected. Do not interpret results as valid SCF estimates.",
            call. = FALSE)
  
  # Construct intercept-only (null) formula from the LHS of the full formula.
  # This is used to compute the Koenker-Machado R1(tau) goodness-of-fit statistic.
  lhs_chr   <- deparse(formula[[2]])
  null_formula <- stats::as.formula(paste(lhs_chr, "~ 1"))
  
  coefs_list <- list()
  vars_list  <- list()
  models     <- list()
  fit_errors <- character(0)
  
  # Per-implicate storage for goodness-of-fit components
  rho_vec      <- numeric(0)   # full model objective values
  rho_null_vec <- numeric(0)   # null model objective values
  nobs_vec     <- integer(0)   # sample sizes
  
  for (i in seq_along(object$mi_design)) {
    imp      <- object$mi_design[[i]]
    df       <- imp$variables
    df$.wts  <- as.numeric(stats::weights(imp, type = "sampling"))
    
    fit <- tryCatch(
      quantreg::rq(formula, tau = tau, data = df, weights = .wts,
                   method = "fn", ...),
      error = function(e) e
    )
    
    if (inherits(fit, "error")) {
      fit_errors <- c(fit_errors,
                      paste0("implicate ", i, ": rq failed: ",
                             conditionMessage(fit)))
      next
    }
    
    nm <- names(stats::coef(fit))
    
    if (se == "replicate") {
      p    <- length(nm)
      df_i <- df
      theta_fn <- function(w, ...) {
        df_i$.wts <- w
        r <- tryCatch(
          suppressWarnings(
            quantreg::rq(formula, tau = tau, data = df_i, weights = .wts)
          ),
          error = function(e) NULL
        )
        if (is.null(r)) return(rep(NA_real_, p))
        out <- rep(NA_real_, p)
        names(out) <- nm
        matched <- intersect(names(stats::coef(r)), nm)
        out[matched] <- stats::coef(r)[matched]
        out
      }
      cov_i <- stats::vcov(survey::withReplicates(imp, theta = theta_fn))
    } else {
      s     <- suppressWarnings(
        quantreg::summary.rq(fit, se = se, covariance = TRUE)
      )
      cov_i <- s$cov
    }
    
    dimnames(cov_i) <- list(nm, nm)
    coefs_list[[length(coefs_list) + 1]] <- stats::coef(fit)
    vars_list[[length(vars_list) + 1]]   <- cov_i
    models[[length(models) + 1]]         <- fit
    
    # --- Goodness-of-fit: full model objective value ---
    # sum(rho_tau(residuals)) where rho_tau is the check function.
    # quantreg stores the minimized objective in fit$rho (a scalar for single tau).
    rho_i <- if (!is.null(fit$rho)) {
      as.numeric(fit$rho)
    } else {
      # Fallback: compute directly from residuals and weights.
      resid_i <- as.numeric(stats::residuals(fit))
      wts_i   <- df$.wts
      sum(wts_i * ifelse(resid_i >= 0, tau * resid_i, (tau - 1) * resid_i))
    }
    
    # --- Goodness-of-fit: null model objective value ---
    fit_null <- tryCatch(
      quantreg::rq(null_formula, tau = tau, data = df, weights = .wts,
                   method = "fn"),
      error = function(e) NULL
    )
    
    rho_null_i <- if (!is.null(fit_null)) {
      if (!is.null(fit_null$rho)) {
        as.numeric(fit_null$rho)
      } else {
        resid_null <- as.numeric(stats::residuals(fit_null))
        wts_i      <- df$.wts
        sum(wts_i * ifelse(resid_null >= 0,
                           tau * resid_null, (tau - 1) * resid_null))
      }
    } else {
      NA_real_
    }
    
    rho_vec      <- c(rho_vec,      rho_i)
    rho_null_vec <- c(rho_null_vec, rho_null_i)
    nobs_vec     <- c(nobs_vec,     nrow(df))
  }
  
  if (length(models) < 2) {
    msg <- c("Too few successful implicate-level models to pool (need >= 2).",
             if (length(fit_errors)) c("Implicate errors:", fit_errors))
    stop(paste(msg, collapse = "\n"))
  }
  
  if (length(fit_errors) > 0)
    warning(paste(c("Some implicates failed:", fit_errors), collapse = "\n"),
            call. = FALSE)
  
  common_terms <- Reduce(intersect, lapply(coefs_list, names))
  if (length(common_terms) == 0)
    stop("No common coefficient terms found across implicates.")
  
  coefs_list <- lapply(coefs_list, function(x) x[common_terms])
  vars_list  <- lapply(vars_list, function(v) {
    v[common_terms, common_terms, drop = FALSE]
  })
  
  pooled <- scf_MIcombine(coefs_list, vars_list)
  
  est   <- pooled$coefficients
  se_v  <- sqrt(diag(pooled$variance))
  tval  <- est / se_v
  pval  <- 2 * stats::pt(-abs(tval), df = pooled$df)
  
  coef_table <- data.frame(
    term      = names(est),
    estimate  = unname(est),
    std.error = unname(se_v),
    t.value   = unname(tval),
    p.value   = unname(pval),
    stars     = as.character(cut(
      pval,
      breaks = c(-Inf, 0.001, 0.01, 0.05, 0.10, Inf),
      labels = c("***", "**", "*", "^", ""),
      right  = FALSE
    )),
    stringsAsFactors = FALSE
  )
  
  # --- Pool goodness-of-fit statistics across successful implicates ---
  # rho and rho_null are averaged across implicates; R1 is computed from
  # the averages. This mirrors the pooling logic for coefficients (Rubin's
  # Rules average within-implicate quantities before combining).
  # NA values arise only if the null model failed for a given implicate;
  # those implicates are excluded from the fit statistic means.
  rho_mean      <- mean(rho_vec,      na.rm = TRUE)
  rho_null_mean <- mean(rho_null_vec, na.rm = TRUE)
  
  r1 <- if (!is.na(rho_null_mean) && rho_null_mean > 0) {
    1 - rho_mean / rho_null_mean
  } else {
    NA_real_
  }
  
  # Degrees-of-freedom-adjusted R1, analogous to adjusted R-squared.
  # n_mean = mean observations across implicates; p = number of estimated
  # parameters (length of common_terms, which includes the intercept).
  n_mean <- mean(nobs_vec)
  p      <- length(common_terms)
  
  r1_adj <- if (!is.na(r1) && (n_mean - p) > 0) {
    1 - (1 - r1) * n_mean / (n_mean - p)
  } else {
    NA_real_
  }
  
  fit_stats <- list(
    rho       = rho_mean,
    rho_null  = rho_null_mean,
    r1        = r1,
    r1_adj    = r1_adj,
    nobs      = nobs_vec,
    nobs_mean = n_mean
  )
  
  out <- list(
    results    = coef_table,
    tau        = tau,
    se_method  = se,
    fit        = fit_stats,
    vcov       = pooled$variance,
    models     = models,
    fit_errors = fit_errors,
    call       = match.call(),
    formula    = formula
  )
  class(out) <- c("scf_quantreg", "scf_model_result")
  return(out)
}


#' @export
#' @method print scf_quantreg
print.scf_quantreg <- function(x, digits = 4, ...) {
  cat(sprintf("Quantile Regression Results (tau = %.2f, Multiply-Imputed SCF)\n",
              x$tau))
  cat("------------------------------------------------------------------\n")
  
  df <- x$results
  df$estimate   <- round(df$estimate,   digits)
  df$std.error  <- round(df$std.error,  digits)
  df$t.value    <- round(df$t.value,    digits)
  df$p.value    <- format.pval(df$p.value, digits = digits)
  
  print(df[, c("term", "estimate", "std.error", "t.value", "p.value", "stars")],
        row.names = FALSE)
  
  cat(sprintf("\nSE method: %s | Implicates pooled: %d\n",
              x$se_method, length(x$models)))
  
  # Print goodness-of-fit if available
  fit <- x$fit
  if (!is.null(fit) && length(fit) > 0 && !is.null(fit$r1)) {
    cat(sprintf("R1(tau):   %-8s  R1 adj:  %s\n",
                if (is.na(fit$r1))     "NA" else formatC(fit$r1,     format = "f", digits = 4),
                if (is.na(fit$r1_adj)) "NA" else formatC(fit$r1_adj, format = "f", digits = 4)))
  }
  
  if (length(x$fit_errors) > 0)
    cat(sprintf("Warning: %d implicate(s) failed. See object$fit_errors.\n",
                length(x$fit_errors)))
  
  cat("Implicate-level rq objects stored in `object$models`.\n")
  invisible(x)
}


#' @export
#' @method summary scf_quantreg
summary.scf_quantreg <- function(object, digits = 4, ...) {
  cat(sprintf("SCF Quantile Regression Summary (tau = %.2f)\n", object$tau))
  cat("------------------------------------------------------------------\n")
  
  df <- object$results
  df$estimate   <- round(df$estimate,   digits)
  df$std.error  <- round(df$std.error,  digits)
  df$t.value    <- round(df$t.value,    digits)
  df$p.value    <- format.pval(df$p.value, digits = digits)
  
  cat("Pooled Coefficient Estimates:\n")
  print(df[, c("term", "estimate", "std.error", "t.value", "p.value", "stars")],
        row.names = FALSE)
  
  cat(sprintf(
    "\nQuantile:        %.2f\nSE method:       %s\nImplicates used: %d\n",
    object$tau, object$se_method, length(object$models)
  ))
  
  # Goodness-of-fit block
  fit <- object$fit
  if (!is.null(fit) && length(fit) > 0) {
    cat("\nGoodness of Fit (Koenker-Machado, 1999):\n")
    cat(sprintf("  Rho (full model):  %.4f\n", fit$rho))
    cat(sprintf("  Rho (null model):  %.4f\n", fit$rho_null))
    r1_str     <- if (is.na(fit$r1))     "NA" else sprintf("%.4f", fit$r1)
    r1_adj_str <- if (is.na(fit$r1_adj)) "NA" else sprintf("%.4f", fit$r1_adj)
    cat(sprintf("  R1(tau):           %s\n", r1_str))
    cat(sprintf("  R1(tau) adjusted:  %s\n", r1_adj_str))
    cat(sprintf("  Mean N (implicates): %.0f\n", fit$nobs_mean))
    cat("  Note: R1 is a local fit measure at tau; it is not a global\n")
    cat("  summary of fit across the conditional distribution.\n")
    cat("  R1 adjusted uses a df-penalty not derived from asymptotic\n")
    cat("  theory; interpret it descriptively.\n")
  }
  
  if (length(object$fit_errors) > 0) {
    cat("\nImplicate errors:\n")
    cat(paste(" ", object$fit_errors, collapse = "\n"), "\n")
  }
  
  cat("\nCall:\n")
  print(object$call)
  invisible(object)
}

#' Format and Display Regression Results from Multiply-Imputed SCF Models
#'
#' This function formats and aligns coefficient estimates, standard errors, and
#' significance stars from one or more SCF regression model objects
#' (e.g., from \code{scf_ols()}, \code{scf_logit()}, \code{scf_quantreg()}, or \code{scf_glm()}). 
#'
#' It compiles a side-by-side table with terms matched across models, appends
#' model fit statistics (sample size N, R-squared or pseudo-R-squared, quantile tau,
#' and AIC where applicable), and outputs the results as console text, Markdown for
#' R Markdown documents, LaTeX for PDF compilation, or a CSV file.
#'
#' @param ... One or more SCF regression model objects, or a single list of such models.
#' @param model.names Optional character vector naming the models. Defaults to
#'   \code{"Model 1"}, \code{"Model 2"}, etc.
#' @param digits Integer specifying decimal places for numeric formatting when
#'   \code{auto_digits = FALSE}. Default is 0.
#' @param auto_digits Logical; if \code{TRUE}, uses adaptive decimal places:
#'   0 digits for large numbers (>= 1000), 2 digits for moderate (>= 1),
#'   and 3 digits for smaller values.
#' @param labels Optional named character vector or labeling function to replace
#'   term names with descriptive labels.
#' @param output Output format: one of \code{"console"} (print to console),
#'   \code{"markdown"} (print Markdown table for R Markdown), \code{"latex"}
#'   (print LaTeX table for PDF compilation), or \code{"csv"}
#'   (write CSV file).
#' @param file File path for CSV output; required if \code{output = "csv"}.
#'
#' @return Invisibly returns a data frame with formatted regression results and fit statistics.
#'
#' @details
#' The function aligns all unique coefficient terms across provided models, formats
#' coefficients with significance stars and standard errors, appends model fit
#' statistics as additional rows, and renders output in the specified format.
#'
#' Fit statistics rows are automatically selected based on model class:
#' \describe{
#'   \item{All models}{Sample size (N)}
#'   \item{OLS models}{R-squared and AIC}
#'   \item{Logit/GLM models}{Pseudo-R-squared and AIC}
#'   \item{Quantile regression}{Quantile tau, R1(tau), and adjusted R1(tau)}
#' }
#' 
#' It avoids external dependencies by using base R formatting and simple text,
#' Markdown, LaTeX, or CSV output.
#'
#' @examples
#' # Do not implement these lines in real analysis:
#' # Use functions `scf_download()` and `scf_load()`
#' td <- tempfile("regtable_")
#' dir.create(td)
#' 
#' src <- system.file("extdata", "scf2022_mock_raw.rds", package = "scf")
#' file.copy(src, file.path(td, "scf2022.rds"), overwrite = TRUE)
#' scf2022 <- scf_load(2022, data_directory = td)
#'
#' # Wrangle data for example:  Perform OLS regression 
#' m1 <- scf_ols(scf2022, income ~ age)
#'
#' # Example for real analysis: Print regression results as a console table
#' scf_regtable(m1, digits = 2)
#' 
#' # Do not implement these lines in real analysis: Cleanup for package check
#' unlink(td, recursive = TRUE, force = TRUE)
#'
#' @export
scf_regtable <- function(...,
                         model.names = NULL,
                         digits = 0,
                         auto_digits = FALSE,
                         labels = NULL,
                         output = c("console", "markdown", "latex", "csv"),
                         file = NULL) {
  
  models <- list(...)
  
  if (length(models) == 1 && is.list(models[[1]]) &&
      (inherits(models[[1]][[1]], "scf_model_result"))) {
    models <- models[[1]]
  }
  
  output <- match.arg(output)
  n_models <- length(models)
  
  if (is.null(model.names)) model.names <- paste("Model", seq_len(n_models))
  stopifnot(length(model.names) == n_models)
  
  all_terms <- unique(unlist(lapply(models, function(m) m$results$term)))
  all_terms <- sort(all_terms)
  out <- data.frame(term = all_terms, stringsAsFactors = FALSE)
  
  # Helper: format estimates with adaptive digits or fixed digits
  format_estimate <- function(est) {
    if (!auto_digits) {
      formatC(est, format = "f", digits = digits)
    } else {
      sapply(est, function(x) {
        absx <- abs(x)
        if (absx >= 1000) {
          formatC(x, format = "f", digits = 0)
        } else if (absx >= 1) {
          formatC(x, format = "f", digits = 2)
        } else if (absx > 0) {
          formatC(x, format = "f", digits = 3)
        } else {
          "0"
        }
      }, USE.NAMES = FALSE)
    }
  }
  
  # Fill main regression results into the output data.frame
  for (i in seq_along(models)) {
    res <- models[[i]]$results
    est_str <- format_estimate(res$estimate)
    se_str <- format_estimate(res$std.error)
    stars <- as.character(res$stars)
    formatted <- paste0(est_str, stars, " (", se_str, ")")
    vals <- rep("--", length(all_terms))
    names(vals) <- all_terms
    vals[res$term] <- formatted
    out[[model.names[i]]] <- vals
  }
  
  colnames(out)[1] <- "Term"
  
  # Apply custom labels if provided
  if (!is.null(labels)) {
    if (is.function(labels)) {
      out$Term <- labels(out$Term)
    } else if (is.character(labels)) {
      out$Term <- sapply(out$Term, function(t) ifelse(!is.na(labels[t]), labels[t], t))
    }
  }
  
  # ========================================================================
  # DETERMINE FIT STATISTICS ROWS DYNAMICALLY
  # ========================================================================
  # Check if any model is quantile regression
  is_quantreg <- any(sapply(models, function(m) inherits(m, "scf_quantreg")))
  is_logit <- any(sapply(models, function(m) inherits(m, "scf_logit")))
  
  # Build fit terms list dynamically
  fit_terms <- c("N")
  if (!is_quantreg) {
    if (is_logit) {
      fit_terms <- c(fit_terms, "PseudoR2")
    } else {
      fit_terms <- c(fit_terms, "R2")
    }
    fit_terms <- c(fit_terms, "AIC")
  } else {
    # For quantreg, add tau column
    fit_terms <- c(fit_terms, "Tau", "R1", "R1(adj)")
  }
  
  fit_stats_mat <- matrix("--", nrow = length(fit_terms), ncol = n_models,
                          dimnames = list(fit_terms, model.names))
  
  for (i in seq_along(models)) {
    m <- models[[i]]
    
    # Fetch N
    n <- tryCatch({
      length(stats::residuals(m))
    }, error = function(e) {
      # Fallback if S3 method fails: reach into stored implicate models directly
      if (!is.null(m$models) && length(m$models) > 0 && !is.null(m$models[[1]])) {
        length(stats::residuals(m$models[[1]]))
      } else if (!is.null(m$imps) && length(m$imps) > 0 && !is.null(m$imps[[1]])) {
        length(stats::residuals(m$imps[[1]]))
      } else {
        NA_integer_
      }
    })
    
    fit_stats_mat["N", i] <- if (!is.na(n)) as.character(n) else "--"
    
    # Quantile regression: add tau
    if (inherits(m, "scf_quantreg")) {
      tau_val <- if (!is.null(m$tau)) m$tau else NA_real_
      r1_val     <- if (!is.null(m$fit$r1))     m$fit$r1     else NA_real_
      r1_adj_val <- if (!is.null(m$fit$r1_adj)) m$fit$r1_adj else NA_real_
      
      fit_stats_mat["R1", i] <-
        if (!is.na(r1_val))     formatC(r1_val,     digits = 3, format = "f") else "--"
      fit_stats_mat["R1(adj)", i] <-
        if (!is.na(r1_adj_val)) formatC(r1_adj_val, digits = 3, format = "f") else "--"
      fit_stats_mat["Tau", i] <- if (!is.na(tau_val)) formatC(tau_val, digits = 2, format = "f") else "--"
    } else {
      # Standard models: add R2/PseudoR2 and AIC
      
      # Fetch AIC
      aic_val <- tryCatch({
        stats::AIC(m)
      }, error = function(e) {
        if (!is.null(m$fit$AIC)) m$fit$AIC else NA_real_
      })
      
      # Detect binomial/logit family for pseudo-R2 usage 
      is_binomial <- FALSE
      if (!is.null(m$family)) {
        is_binomial <- inherits(m$family, "binomial")
      }
      if (!is_binomial && !is.null(m$models) && length(m$models) > 0) {
        is_binomial <- inherits(m$models[[1]], "glm") &&
          stats::family(m$models[[1]])$family == "binomial"
      }
      
      # Get R2 or pseudo_R2 
      r2_val <- NA_real_
      r2_label <- "R2"
      if (inherits(m, "scf_logit") || is_binomial) {
        r2_val <- if (!is.null(m$fit$pseudo_r2)) m$fit$pseudo_r2 else NA_real_
        r2_label <- "PseudoR2"
      } else {
        r2_val <- if (!is.null(m$fit$r.squared)) m$fit$r.squared else NA_real_
      }
      
      fit_stats_mat[r2_label, i] <- if (!is.na(r2_val)) formatC(r2_val, digits = 3, format = "f") else "--"
      fit_stats_mat["AIC", i] <- if (!is.na(aic_val)) formatC(aic_val, digits = 0, format = "f") else "--"
    }
  }
  
  fit_stats_df <- data.frame(Term = fit_terms, fit_stats_mat, stringsAsFactors = FALSE)
  
  # Convert all columns to character to safely bind
  out[] <- lapply(out, as.character)
  fit_stats_df[] <- lapply(fit_stats_df, as.character)
  
  # Align column names for binding
  colnames(fit_stats_df) <- colnames(out)
  
  # Append fit stats below main table
  out <- rbind(out, fit_stats_df)
  
  # Output depending on user choice
  if (output == "console") {
    max_len <- max(nchar(out$Term))
    for (i in seq_len(nrow(out))) {
      cat(sprintf(paste0("%-", max_len, "s"), out$Term[i]))
      for (j in 2:ncol(out)) {
        cat("  ", format(out[i, j], justify = "right"))
      }
      cat("\n")
    }
    invisible(out)
    
  } else if (output == "csv") {
    if (is.null(file)) stop("Please provide a file path for CSV output.")
    write.csv(out, file = file, row.names = FALSE)
    invisible(out)
    
  } else if (output == "markdown") {
    # Escape LaTeX special characters across all columns
    out_escaped <- out
    out_escaped$Term <- gsub("_", "\\_", out_escaped$Term, fixed = TRUE)
    out_escaped[] <- lapply(out_escaped, function(col) {
      gsub("^", "\\^{}", col, fixed = TRUE)
    })
    
    header <- paste0("| ", paste(colnames(out_escaped), collapse = " | "), " |")
    separator <- paste0("|", paste(rep("---", ncol(out_escaped)), collapse = "|"), "|")
    rows <- apply(out_escaped, 1, function(r) paste0("| ", paste(r, collapse = " | "), " |"))
    md_table <- paste(c(header, separator, rows), collapse = "\n")
    cat(md_table, "\n")
    invisible(md_table)
    
  } else if (output == "latex") {
    # LaTeX table format using booktabs for nice horizontal lines
    cat("\\begin{table}\n")
    cat("\\centering\n")
    cat("\\begin{tabular}{l", paste(rep("r", ncol(out) - 1), collapse = ""), "}\n", sep = "")
    cat("\\toprule\n")
    
    # Header row
    header <- paste(colnames(out), collapse = " & ")
    cat(header, " \\\\\n")
    cat("\\midrule\n")
    
    # Data rows - insert midrule before fit stats
    n_coef <- nrow(out) - length(fit_terms)
    for (i in seq_len(nrow(out))) {
      row_str <- paste(out[i, ], collapse = " & ")
      cat(row_str, " \\\\\n")
      if (i == n_coef) cat("\\midrule\n")
    }
    
    cat("\\bottomrule\n")
    cat("\\end{tabular}\n")
    cat("\\end{table}\n")
    invisible(out)
  }
}

#' Create or Alter SCF Variables
#'
#' @description
#' Use this function to create or alter SCF variables once the raw data set has
#' been loaded into memory using the `scf_load()` function. This function
#' updates an `scf_mi_survey` object by evaluating transformations within each
#' implicate, and then returning a new object with the new or amended variables.
#'
#' Most of the time, you can use `scf_update()` to define variables based on
#' simple logical conditions, arithmetic transformations, or categorical
#' binning. These rules are evaluated separately in each implicate, using the
#' same formula. However, if the transformation you want to apply depends on the
#' distribution of the data within each implicate, such as computing an
#' average percentile or ranking households across all implicates, 
#' this function will not suffice. In those cases, use
#' [scf_update_by_implicate()] to write a custom function that operates on each
#' implicate individually.
#'
#' @section Usage:
#' Use `scf_update()` during data wrangling to clean, create, or alter variables before calculating
#' statistics or running models. The function is useful when the analyst wishes to:
#' - Recode missing values that are coded as numeric data
#' - Recast variables that are not in the desired format (e.g., converting a numeric variable to a factor)
#' - Create new variables based on existing ones (e.g., calculating ratios, differences, or indicators)
#'
#' @param object A `scf_mi_survey` object, typically created by [scf_load()].
#' @param ... Named expressions assigning new or modified variables using `=` syntax.
#'   Each expression must return a vector of the same length as the implicate data frame.
#'
#' @return The input `scf_mi_survey` object with `mi_design` updated to reflect
#' the new or modified variables. All other attributes (`year`, `n_households`,
#' `mock`) are preserved unchanged.
#'
#' @examples
#' # Do not implement these lines in real analysis:
#' # Use functions `scf_download()` and `scf_load()`
#' td <- tempfile("update_")
#' dir.create(td)
#' 
#' src <- system.file("extdata", "scf2022_mock_raw.rds", package = "scf")
#' file.copy(src, file.path(td, "scf2022.rds"), overwrite = TRUE)
#' scf2022 <- scf_load(2022, data_directory = td)
#'
#' # Example for real analysis: Create a binary indicator for being over age 50
#' scf2022 <- scf_update(scf2022,
#'   over50 = age > 50
#' )
#'
#' # Example: Create a log-transformed income variable
#' scf2022 <- scf_update(scf2022,
#'   log_income = log(income + 1)
#' )
#' 
#' # Do not implement these lines in real analysis: Cleanup for package check
#' unlink(td, recursive = TRUE, force = TRUE)
#'
#' @seealso [scf_load()], [scf_update_by_implicate()], [survey::svrepdesign()]
#'
#' @export
scf_update <- function(object, ...) {
  
  if (!inherits(object, "scf_mi_survey")) {
    stop("Input must be of class 'scf_mi_survey'.")
  }
  
  exprs <- rlang::enquos(...)

  if (is.null(object$mi_design) || length(object$mi_design) == 0)
    stop("'mi_design' is empty. Cannot apply update.")

  updated_designs <- vector("list", length(object$mi_design))

  for (i in seq_along(object$mi_design)) {
    df <- object$mi_design[[i]]$variables

    for (j in seq_along(exprs)) {
      varname <- names(exprs)[j]
      try({
        df[[varname]] <- rlang::eval_tidy(exprs[[j]], data = df)
      }, silent = TRUE)
    }

    rep_cols <- grep("^wt1b", names(df), value = TRUE)
    updated_designs[[i]] <- survey::svrepdesign(
      weights          = ~wgt,
      repweights       = as.matrix(df[, rep_cols]),
      data             = df,
      type             = "other",
      scale            = 1,
      rscales          = rep(1 / 998, 999),
      mse              = TRUE,
      combined.weights = TRUE
    )
  }

  object$mi_design <- updated_designs
  
  if (!inherits(object, "scf_mi_survey")) {
    class(object) <- "scf_mi_survey"
  }
  if (is.null(attr(object, "mock"))) {
    attr(object, "mock") <- FALSE
  }
  
  return(object)
}

# tests/testthat/test-scf_logit.R

# NOTE: This test may trigger the benign warning:
#   'non-integer #successes in a binomial glm!'
# This occurs in replicate-weighted logistic regression using `svyglm(family = binomial())`.
# It does not affect estimates or inference.
# See: https://stackoverflow.com/questions/12953045/warning-non-integer-successes-in-a-binomial-glm-survey-packages

test_that("scf_logit runs and returns expected structure (with known warning)", {
  skip_on_cran()

  td  <- tempdir()
  src <- system.file("extdata", "scf2022_mock_raw.rds", package = "scf")
  file.copy(src, file.path(td, "scf2022.rds"), overwrite = TRUE)
  scf2022 <- scf_load(2022, data_directory = td)
  
  scf2022 <- scf_update(scf2022,
                    rich = as.integer(networth > 1e6),
                    log_income = log(pmax(income, 1)))

  model <- suppressWarnings(scf_logit(scf2022, rich ~ age + log_income))

  expect_s3_class(model, "scf_logit")
  expect_true("results" %in% names(model))
  expect_true(is.data.frame(model$results))
  expect_true(all(c("term", "estimate", "std.error") %in% names(model$results)))
  
  unlink(file.path(td, "scf2022.rds"), force = TRUE)
})

#' Load SCF Data as Multiply-Imputed Survey Designs
#'
#' @description
#' Converts SCF `.rds` files prepared by `scf_download()` into `scf_mi_survey`
#' objects. Each object wraps five implicates per year in replicate-weighted,
#' multiply-imputed survey designs suitable for use with `scf_` functions.
#'
#' @section Implementation:
#' Provide a year or range and either (1) a directory containing `scf<year>.rds`
#' files, or (2) a full path to a single `.rds` file. Files must contain five
#' implicate data frames with columns `wgt` and `wt1b1..wt1bK` (typically K=999).
#'
#' @param min_year Integer. First SCF year to load (1989–2022, divisible by 3).
#' @param max_year Integer. Last SCF year to load. Defaults to `min_year`.
#' @param data_directory Character. Directory containing `.rds` files or a 
#'   full path to a single `.rds` file. Defaults to the current working directory `"."`.
#'   For examples and tests, use `tempdir()` to avoid leaving files behind.
#'
#' @return Invisibly returns a `scf_mi_survey` (or named list if multiple years).
#' Attributes: `mock` (logical), `year`, `n_households`.
#'
#' @seealso [scf_download()], [scf_design()], [scf_update()], [survey::svrepdesign()]
#'
#' @examples
#' # Using with CRAN-compliant mock data:
#' # Use functions `scf_download()` and `scf_load()`
#' td <- tempfile("load_")
#' dir.create(td)
#' 
#' src <- system.file("extdata", "scf2022_mock_raw.rds", package = "scf")
#' file.copy(src, file.path(td, "scf2022.rds"), overwrite = TRUE)
#' scf2022 <- scf_load(2022, data_directory = td)
#' 
#' # Do not implement these lines in real analysis: Cleanup for package check
#' unlink(td, recursive = TRUE, force = TRUE)
#'
#' @export
scf_load <- function(min_year,
                     max_year = min_year,
                     data_directory = ".") {

  if (!requireNamespace("survey", quietly = TRUE)) stop("Package 'survey' is required.")

  household_totals <- c(
    `1989` = 92830000, `1992` = 95670000, `1995` = 98990000,
    `1998` = 102530000, `2001` = 108210000, `2004` = 112000000,
    `2007` = 116100000, `2010` = 117540000, `2013` = 122460000,
    `2016` = 125820000, `2019` = 128580000, `2022` = 131202000
  )

  years <- intersect(seq(min_year, max_year, by = 3), names(household_totals))
  if (length(years) == 0) stop("No valid SCF years selected.")
  
  results <- list()
  
  for (year in years) {
    file_path <- file.path(data_directory, paste0("scf", year, ".rds"))
    if (!file.exists(file_path)) {
      warning("File not found: ", file_path)
      next
    }
    
    imp_list <- readRDS(file_path)
    
    if (!is.list(imp_list) || length(imp_list) != 5) {
      warning("File does not contain 5 implicates: ", file_path)
      next
    }
    
    imp_list <- lapply(imp_list, function(df) {
      rep_cols <- grep("^wt1b", names(df), value = TRUE)
      df[rep_cols] <- lapply(df[rep_cols], as.numeric)
      df
    })
    
    rep_cols <- grep("^wt1b", names(imp_list[[1]]), value = TRUE)
    
    imp_designs <- lapply(imp_list, function(df) {
      survey::svrepdesign(
        weights = ~wgt,
        repweights = as.matrix(df[, rep_cols]),
        data = df,
        type = "other",
        scale = 1,
        rscales = rep(1 / 998, 999),
        mse = TRUE,
        combined.weights = TRUE
      )
    })
    
    mi_obj <- scf_design(
      design = imp_designs,
      year = as.integer(year),
      n_households = sum(imp_list[[1]]$wgt)
    )
    
    attr(mi_obj, "mock") <- FALSE
    
    results[[as.character(year)]] <- mi_obj
  }
  
  if (length(results) == 0L) stop("No valid SCF files loaded.")
  if (length(results) == 1L) {
    invisible(results[[1]])
  } else {
    invisible(results)
  }
}

#' Plot Bar Chart of a Discrete Variable from SCF Data
#'
#' @description
#' Creates a bar chart that visualizes the distribution of a discrete variable.
#'
#' @section Implementation:
#' This function internally calls [scf_freq()] to compute population proportion
#' estimates, which are then plotted using `ggplot2::geom_col()`. The default
#' output is scaled to percent and can be customized via title, axis labels,
#' angle, and color.
#'
#' @section Details:
#' Produces a bar chart of category proportions from a one-way tabulation,
#' pooled across SCF implicates using [scf_freq()]. This function summarizes
#' weighted sample composition and communicates categorical distributions
#' effectively in descriptive analysis.
#'
#' @param design A `scf_mi_survey` object created by [scf_load()]. Must contain valid implicates.
#' @param variable A one-sided formula specifying a categorical variable (e.g., `~racecl`).
#' @param title Optional character string for the plot title. Default: `"Distribution of <variable>"`.
#' @param xlab Optional x-axis label. Default: variable name.
#' @param ylab Optional y-axis label. Default: `"Percent"`.
#' @param angle Integer. Rotation angle for x-axis labels. Default is `30`.
#' @param fill Fill color for bars. Default is `"#0072B2"`.
#' @param label_map Optional named vector to relabel x-axis category labels.
#'
#' @return A `ggplot2` object representing the pooled bar chart.
#'
#' @section Dependencies: 
#' Requires the `ggplot2` package.
#'
#' @seealso [scf_freq()], [scf_plot_bbar()], [scf_xtab()]
#'
#' @examples
#' # Do not implement these lines in real analysis:
#' # Use functions `scf_download()` and `scf_load()`
#' td <- tempfile("plot_dbar_")
#' dir.create(td)
#' 
#' src <- system.file("extdata", "scf2022_mock_raw.rds", package = "scf")
#' file.copy(src, file.path(td, "scf2022.rds"), overwrite = TRUE)
#' scf2022 <- scf_load(2022, data_directory = td)
#'
#' # Example for real analysis: Bar chart of education categories
#' scf_plot_dbar(scf2022, ~edcl)
#' 
#' # Do not implement these lines in real analysis: Cleanup for package check
#' unlink(td, recursive = TRUE, force = TRUE)
#' 
#' @export
scf_plot_dbar <- function(design, variable,
                          title = NULL,
                          xlab = NULL,
                          ylab = "Percent",
                          angle = 30,
                          fill = "#0072B2",
                          label_map = NULL) {

  if (!inherits(variable, "formula")) {
    stop("`variable` must be a one-sided formula, e.g., ~edcl")
  }

  if (!inherits(design, "scf_mi_survey")) {
    stop("Input must be a `scf_mi_survey` object.")
  }

  if (isTRUE(attr(design, "mock"))) {
    warning("Mock data detected. Do not interpret results as valid SCF estimates.", call. = FALSE)
  }


  `%||%` <- function(a, b) if (!is.null(a)) a else b
  varname <- all.vars(variable)[1]

  freq <- scf_freq(design, variable, percent = TRUE)
  df <- freq$results

  if (!"proportion" %in% names(df)) {
    stop("`scf_freq()` output must include a `proportion` column.")
  }

  # Optional relabeling
  if (!is.null(label_map)) {
    df$category <- factor(df$category,
                          levels = names(label_map),
                          labels = unname(label_map))
  }

  df$yval <- df$proportion

  p <- ggplot2::ggplot(df, ggplot2::aes(x = as.factor(category), y = yval)) +
    ggplot2::geom_col(fill = fill) +
    ggplot2::labs(
      title = title %||% paste("Distribution of", varname),
      x = xlab %||% varname,
      y = ylab
    ) +
    scf_theme() +
    ggplot2::theme(axis.text.x = .scf_axis_text_x(angle))

  return(p)
}

#' Estimate an Ordinary Least Squares Regression on SCF Microdata
#'
#' Fits a replicate-weighted linear regression model to each implicate of
#' multiply-imputed SCF data and pools coefficients and standard errors using
#' Rubin’s Rules.
#'
#' @description
#' Computes an OLS regression on SCF data using `svyglm()` across the SCF's
#' five implicates.  Returns coefficient estimates, standard errors, test
#' statistics, and model diagnostics.
#'
#' @section Implementation:
#' Ordinary least squares (OLS) regression estimates the linear relationship
#' between a continuous outcome and one or more predictor variables. Each
#' coefficient represents the expected change in the outcome for a one-unit
#' increase in the corresponding predictor, holding all other predictors
#' constant. 
#'
#' Use this function to model associations between SCF variables while
#' accounting for complex survey design and multiple imputation.
#'
#' This function takes a `scf_mi_survey` object and a model formula. Internally,
#' it fits a weighted linear regression to each implicate using
#' `survey::svyglm()`, extracts coefficients and variance-covariance matrices,
#' and pools them via [scf_MIcombine()].
#'
#' @param object A `scf_mi_survey` object created with [scf_load()] and [scf_design()]. Must contain five implicates with replicate weights.
#' @param formula A model formula specifying a continuous outcome and predictor variables (e.g., `networth ~ income + age`).
#'
#' @return An object of class `"scf_ols"` and `"scf_model_result"` with:
#' \describe{
#'   \item{results}{A data frame of pooled coefficients, standard errors, t-values, p-values, and significance stars.}
#'   \item{fit}{A list of model diagnostics including mean AIC, standard deviation of AIC, mean R-squared, and its standard deviation.}
#'   \item{imps}{A list of implicate-level `svyglm` model objects.}
#'   \item{call}{The matched call used to produce the model.}
#' }
#'
#' @examples
#' # Do not implement these lines in real analysis:
#' # Use functions `scf_download()` and `scf_load()`
#' td <- tempfile("ols_")
#' dir.create(td)
#' 
#' src <- system.file("extdata", "scf2022_mock_raw.rds", package = "scf")
#' file.copy(src, file.path(td, "scf2022.rds"), overwrite = TRUE)
#' scf2022 <- scf_load(2022, data_directory = td)
#'
#' # Example for real analysis: Run OLS model
#' model <- scf_ols(scf2022, networth ~ income + age)
#' summary(model)
#' 
#' # Do not implement these lines in real analysis: Cleanup for package check
#' unlink(td, recursive = TRUE, force = TRUE)
#'
#' @seealso [scf_glm()], [scf_logit()], [scf_MIcombine()]
#' @importFrom stats coef vcov pt sd AIC deviance
#'
#' @export
scf_ols <- function(object, formula) {
  if (!inherits(object, "scf_mi_survey"))
    stop("Input must be of class 'scf_mi_survey'")
  if (!inherits(formula, "formula"))
    stop("Model must be specified as a formula")

  if (isTRUE(attr(object, "mock"))) {
    warning("Mock data detected. Do not interpret results as valid SCF estimates.", call. = FALSE)
  }


  models <- lapply(object$mi_design, function(imp) {
    tryCatch(survey::svyglm(formula, design = imp), error = function(e) NULL)
  })

  models <- Filter(Negate(is.null), models)
  if (length(models) < 2)
    stop("Too few successful implicate-level models to pool.")

  coefs_list <- lapply(models, coef)
  vars_list  <- lapply(models, vcov)
  common_terms <- Reduce(intersect, lapply(coefs_list, names))
  coefs_list <- lapply(coefs_list, function(x) x[common_terms])
  vars_list  <- lapply(vars_list, function(v) v[common_terms, common_terms, drop = FALSE])

  # Drop the survey design from each model object after extracting what is
  # needed. svyglm stores the full svrepdesign (data + 999 replicate weights)
  # in $survey.design; keeping it multiplies object size by ~5x for no benefit.
  models <- lapply(models, function(m) { m$survey.design <- NULL; m })

  pooled <- scf_MIcombine(coefs_list, vars_list)

  est  <- coef(pooled)
  se   <- SE(pooled)
  tval <- est / se
  pval <- 2 * pt(-abs(tval), df = pooled$df)

  coefs <- data.frame(
    term = names(est),
    estimate = est,
    std.error = se,
    t.value = tval,
    p.value = pval,
    stars = cut(pval,
                breaks = c(-Inf, 0.001, 0.01, 0.05, 0.10, Inf),
                labels = c("***", "**", "*", "^", ""),
                right = FALSE),
    stringsAsFactors = FALSE
  )

  aics <- sapply(models, AIC)
  r2s  <- sapply(models, function(m) {
    if (is.null(m$null.deviance) || m$null.deviance == 0) return(NA_real_)
    1 - deviance(m) / m$null.deviance
  })


  diagnostics <- list(
    AIC = mean(aics),
    AIC.sd = sd(aics),
    r.squared = mean(r2s),
    r.squared.sd = sd(r2s)
  )

  out <- list(
    results = coefs,
    fit = diagnostics,
    vcov = pooled$variance,
    imps = models,
    call = match.call(),
    formula = formula
  )
  class(out) <- c("scf_ols", "scf_model_result")
  return(out)
}
#' @export
#' @method summary scf_ols
summary.scf_ols <- function(object, digits = 4, ...) {
  cat("SCF OLS Regression Summary\n")
  cat("--------------------------------------------------\n")

  df <- object$results
  df$estimate   <- round(df$estimate,   digits)
  df$std.error  <- round(df$std.error,  digits)
  df$t.value    <- round(df$t.value,    digits)
  df$p.value    <- format.pval(df$p.value, digits = digits)

  cat("Pooled Coefficient Estimates:\n")
  print(df[, c("term", "estimate", "std.error", "t.value", "p.value", "stars")],
        row.names = FALSE)

  cat("\nModel Fit Statistics:\n")
  with(object$fit, {
    if (!is.null(r.squared) && !is.na(r.squared)) {
      cat("  Mean R-squared: ", round(r.squared, digits),
          " (SD: ", round(r.squared.sd, digits), ")\n", sep = "")
    }
    cat("  Mean AIC:       ", round(AIC, digits),
        " (SD: ", round(AIC.sd, digits), ")\n", sep = "")
  })

  cat("\nCall:\n")
  print(object$call)
  invisible(object)
}

#' @export
#' @method print scf_ols
print.scf_ols <- function(x, digits = 4, ...) {
  cat("OLS Regression Results (Multiply-Imputed SCF)\n")
  cat("--------------------------------------------------\n")
  print(x$results, digits = digits, row.names = FALSE)

  cat("\nModel Fit Statistics:\n")
  with(x$fit, {
    if (!is.null(r.squared)) {
      cat("  Mean R-squared: ", round(r.squared, digits),
          " (SD: ", round(r.squared.sd, digits), ")\n", sep = "")
    }
    cat("  Mean AIC:       ", round(AIC, digits),
        " (SD: ", round(AIC.sd, digits), ")\n", sep = "")
  })

  cat("\nNote: Implicate-level model objects are stored in `object$imps`\n")
  cat("      Use `summary(object$imps[[1]])` to inspect them.\n")
  invisible(x)
}

#' Stacked Bar Chart of Two Discrete Variables in SCF Data
#'
#' @description
#' Visualizes a discrete-discrete bivariate distribution using stacked bars
#' based on pooled cross-tabulations from [scf_xtab()]. Use this function to
#' visualize the relationship between two discrete variables.
#'
#' @section Implementation:
#' This function calls [scf_xtab()] to estimate the joint distribution of two
#' categorical variables across multiply-imputed SCF data. The result is translated
#' into a `ggplot2` stacked bar chart using pooled counts or normalized percentages.
#'
#' @param design A `scf_mi_survey` object created by [scf_load()]. Must contain five implicates with replicate weights.
#' @param rowvar A one-sided formula for the x-axis grouping variable (e.g., `~edcl`).
#' @param colvar A one-sided formula for the stacked fill variable (e.g., `~racecl`).
#' @param scale Character. One of `"percent"` (default) or `"count"`.
#' @param percent_by Character. One of `"total"` (default), `"row"`, or `"col"` — determines normalization base when `scale = "percent"`.
#' @param title Optional character string for the plot title.
#' @param xlab Optional character string for the x-axis label.
#' @param ylab Optional character string for the y-axis label.
#' @param fill_colors Optional vector of fill colors to pass to `ggplot2::scale_fill_manual()`.
#' @param row_labels Optional named vector to relabel `row` categories (x-axis).
#' @param col_labels Optional named vector to relabel `col` categories (legend).
#'
#' @return A `ggplot2` object. 
#'
#' @examples
#' # Do not implement these lines in real analysis:
#' # Use functions `scf_download()` and `scf_load()`
#' td <- tempfile("plot_bbar_")
#' dir.create(td)
#' 
#' src <- system.file("extdata", "scf2022_mock_raw.rds", package = "scf")
#' file.copy(src, file.path(td, "scf2022.rds"), overwrite = TRUE)
#' scf2022 <- scf_load(2022, data_directory = td)
#'
#' # Example for real analysis: Stacked bar chart: education by ownership
#' scf_plot_bbar(scf2022, ~own, ~edcl)
#'
#' # Example for real analysis: Column percentages instead of total percent
#' scf_plot_bbar(scf2022, ~own, ~edcl, percent_by = "col")
#'
#' # Example for real analysis: Raw counts (estimated number of households)
#' scf_plot_bbar(scf2022, ~own, ~edcl, scale = "count")
#' 
#' # Do not implement these lines in real analysis: Cleanup for package check
#' unlink(td, recursive = TRUE, force = TRUE)
#' 
#' @export
scf_plot_bbar <- function(design, rowvar, colvar,
                          scale = c("percent", "count"),
                          percent_by = c("total", "row", "col"),
                          title = NULL,
                          xlab = NULL,
                          ylab = NULL,
                          fill_colors = NULL,
                          row_labels = NULL,
                          col_labels = NULL) {

  scale <- match.arg(scale)
  percent_by <- match.arg(percent_by)

  if (isTRUE(attr(design, "mock"))) {
    warning("Mock data detected. Do not interpret results as valid SCF estimates.", call. = FALSE)
  }

  if (!requireNamespace("ggplot2", quietly = TRUE)) {
    stop("Package 'ggplot2' is required for plotting.")
  }

  `%||%` <- function(a, b) if (!is.null(a)) a else b
  rowname <- deparse(rowvar[[2]])
  colname <- deparse(colvar[[2]])

  xtab <- scf_xtab(design, rowvar, colvar)
  df <- xtab$results

  # Choose y variable and label
  if (scale == "count") {
    df$yval <- df$prop * sum(sapply(xtab$imps, sum))
    ylab <- ylab %||% "Estimated Count"
  } else {
    ylab <- ylab %||% "Percent"
    df$yval <- switch(percent_by,
                      total = df$prop,
                      row = df$row_share,
                      col = df$col_share) * 100
  }

  # Convert to factor preserving order of appearance
  df$row <- factor(df$row, levels = unique(df$row))
  df$col <- factor(df$col, levels = unique(df$col))

  # Relabel if requested
  if (!is.null(row_labels)) {
    levels(df$row) <- unname(row_labels[levels(df$row)])
  }
  if (!is.null(col_labels)) {
    levels(df$col) <- unname(col_labels[levels(df$col)])
  }

  p <- ggplot2::ggplot(df, ggplot2::aes(x = row, y = yval, fill = col)) +
    ggplot2::geom_col(position = "stack", color = "white") +
    ggplot2::labs(
      title = title,
      x = xlab %||% rowname,
      y = ylab,
      fill = colname
    ) +
    scf_theme()

  if (!is.null(fill_colors)) {
    p <- p + ggplot2::scale_fill_manual(values = fill_colors)
  }

  return(p)
}

# Internal helper for x-axis text alignment in SCF plots.
.scf_axis_text_x <- function(angle) {
  is_zero_angle <- isTRUE(all.equal(angle, 0))
  
  ggplot2::element_text(
    angle = angle,
    hjust = if (is_zero_angle) 0.5 else 1,
    vjust = if (is_zero_angle) 0.5 else 1
  )
}

#' Analyzing Survey of Consumer Finances Public-Use Microdata
#'
#' @author Joseph N. Cohen, CUNY Queens College
#'
#' @description
#'
#' This package provides functions to analyze the U.S. Federal Reserve's
#' Survey of Consumer Finances (SCF) public-use microdata. It encapsulates
#' the SCF’s multiply-imputed, replicate-weighted structure in a custom
#' object class (`scf_mi_survey`) and supports estimation of population-level
#' statistics, including univariate and bivariate distributions,
#' hypothesis tests, data visualizations, and regression models.
#'
#' Designed for generalist analysts, the package assumes familiarity with
#' standard statistical methods but not with the complexities of
#' multiply-imputed or survey-weighted data. All functions prioritize
#' transparency, reproducibility, and pedagogical clarity.
#'
#' @section Methodological Background:
#'
#' The SCF is one of the most detailed and methodologically rigorous sources of
#' data on U.S. household finances. It is nationally representative, includes an
#' oversample of high-wealth households and households in predominantly Black
#' communities, and provides multiply-imputed estimates for item nonresponse.
#' These features increase the analytical value of the data set but also
#' introduce methodological complexity. Valid inference requires attention to:
#'
#' - **Survey Weights:** The SCF employs a dual-frame, stratified, and clustered
#'   probability sample. Analysts must apply the provided sampling weights to
#'   produce population-representative estimates.
#' - **Replicate Weights:** Each observation is associated with 999 replicate weights,
#'   generated using a custom replication method developed by the Federal
#'   Reserve. These are used to estimate sampling variance.
#' - **Multiple Imputation:** The SCF uses multiple imputation to address item nonresponse,
#'   providing five implicates per household. Estimates must be pooled across
#'   implicates to obtain valid point estimates and standard errors.
#'
#' The `scf` package provides a structured, user-friendly interface for handling
#' these design complexities, enabling applied researchers and generalist
#' analysts to conduct principled and reproducible analysis of SCF microdata
#' using familiar statistical workflows.
#'
#' @section Package Architecture and Workflow:
#'
#' This section recommends a sequence of operations enacted through the package's
#' functions.  For an in-depth discussion of the methodological considerations
#' involved in these functions' formulation, see Cohen (2026).
#'
#' 1. **Data Acquisition**:  Download the data from Federal Reserve servers to your working directory using [scf_download()].
#' 2. **Data Loading**: Load the data into R using [scf_load()]. This function returns an `scf_mi_survey` object (described below).
#' 3. **Data Wrangling**: Use [scf_update()] to modify the data, or [scf_subset()] to filter it. These functions return new `scf_mi_survey` objects.
#' 4. **Descriptive Statistics**: Compute univariate and bivariate statistics using functions like [scf_mean()], [scf_median()], [scf_percentile()], [scf_freq()], [scf_xtab()], and [scf_corr()].
#' 5. **Basic Inferential Tests**: Conduct hypothesis tests using [scf_ttest()] for means and [scf_prop_test()] for proportions.
#' 6. **Regression Modeling**: Fit regression models using [scf_ols()] for linear regression, [scf_logit()] for logistic regression, and [scf_glm()] for generalized linear models.
#' 7. **Data Visualization**: Create informative visualizations using [scf_plot_dist()] for distributions, [scf_plot_cbar()] and [scf_plot_bbar()] for categorical data, [scf_plot_smooth()] for smoothers, and [scf_plot_hex()] for hexbin plots.
#' 8. **Diagnostics and Infrastructure**: Use [scf_MIcombine()] to pool results across implicates.
#'
#' @section Core Data Object and Its Structure:
#'
#' This suite of functions operate from a custom object class, `scf_mi_survey`,
#' which is created by [scf_design()] via [scf_load()].  Specifically, the
#' object is a structured list containing the elements:
#'
#' - `mi_design`: A list of five `survey::svrepdesign()` objects (one per implicate)
#' - `year`: Year of survey
#' - `n_households`: Estimated number of U.S. households in that year, per data from the Federal Reserve Economic Data (FRED) series TTLHH, accessed 6/17/2025.
#'
#' @section Imputed Missing Data:
#'
#' The SCF addresses item nonresponse using multiple imputation
#' (see Kennickell 1998). This procedure generates five completed data sets,
#' each containing distinct but plausible values for the missing entries. The
#' method applies a predictive model to the observed data, simulates variation
#' in both model parameters and residuals, and generates five independent
#' estimates for each missing value. These completed data sets—called
#' *implicates*—reflect both observed relationships and the uncertainty in
#' estimating them. See [scf_MIcombine()] for details.
#' 
#' @section Mock Data for Testing:
#' 
#' A mock SCF dataset (`scf2022_mock_raw.rds`) is bundled in `inst/extdata/` for 
#' internal testing purposes. It is a structurally valid `scf_mi_survey` object 
#' created by retaining only the first ~200 rows per implicate and only variables
#' used in examples and tests.
#' 
#' This object is intended solely for package development and documentation rendering. 
#' It is **not suitable for analytical use or valid statistical inference.**
#' 
#'
#' @section Theming and Visual Style:
#' All built-in graphics follow a common aesthetic set by [scf_theme()]. Users
#' may modify the default theme by calling `scf_theme()` explicitly within their
#' scripts. See [scf_theme()] documentation for customization options.
#'
#' @section Pedagogical Design:
#'
#' The package is designed to support instruction in advanced methods courses
#' on complex survey analysis and missing data. It promotes pedagogical
#' transparency through several features:
#'
#' - Each implicate’s design object is accessible via `scf_mi_survey$mi_design[[i]]`
#' - Raw implicate-level data can be viewed directly through `scf_mi_survey$mi_design[[i]]$variables`
#' - Users can execute analyses on individual implicates or combine them using Rubin’s Rules
#' - Key functions implement design-based estimation strategies explicitly, such as replicate-weight variance estimation
#' - Minimal abstraction is used, so each step remains visible and tractable
#'
#' These features allow instructors to demonstrate how survey weights, replicate
#' designs, and multiple imputation contribute to final results. Students can
#' follow the full analytic path from raw inputs to pooled estimates using
#' transparent, inspectable code and data structures.
#'
#' @references
#' Cohen JN. Analyzing the Survey of Consumer Finances with \pkg{scf}. 2026.
#'   <https://osf.io/azrsn>
#'
#' Barnard J, Rubin DB. Small-sample degrees of freedom with multiple imputation.
#'   \doi{10.1093/biomet/86.4.948}.
#'
#' Bricker J, Henriques AM, Moore KB. Updates to the sampling of wealthy families in the Survey of Consumer Finances.
#'   Finance and Economics Discussion Series 2017-114.
#'   https://www.federalreserve.gov/econres/scfindex.htm
#'
#' Kennickell AB, McManus DA, Woodburn RL. Weighting design for the 1992 Survey of Consumer Finances.
#'   U.S. Federal Reserve.
#'   <https://www.federalreserve.gov/Pubs/OSS/oss2/papers/weight92.pdf>
#'
#' Kennickell AB. Multiple imputation in the Survey of Consumer Finances.
#'   Statistical Journal of the IAOS 33(1):143-151.
#'   \doi{10.3233/SJI-160278}.
#'
#' Little RJA, Rubin DB. Statistical analysis with missing data.
#'   ISBN: 9780470526798.
#'
#' Lumley T. survey: Analysis of complex survey samples. R package version 4.1-1.
#'   <https://CRAN.R-project.org/package=survey>
#'
#' Lumley T. Analysis of complex survey samples.
#'   \doi{10.18637/jss.v009.i08}.
#'
#' Lumley T. Complex surveys: A guide to analysis using R. ISBN: 9781118210932.
#'
#' U.S. Federal Reserve. Codebook for 2022 Survey of Consumer Finances.
#'   https://www.federalreserve.gov/econres/scfindex.htm
#'
#' @docType package
#' @name scf
"_PACKAGE"

#' Plot a Univariate Distribution of an SCF Variable
#'
#' @description
#' This function provides a unified plotting interface for visualizing the
#' distribution of a single variable from multiply-imputed SCF data. Discrete
#' variables produce bar charts of pooled proportions; continuous variables
#' produce binned histograms.  Use this function to visualize the univariate
#' distribution of an SCF variable.
#'
#' @section Implementation:
#' For discrete variables (factor or numeric with <= 25 unique values), the
#' function uses [scf_freq()] to calculate category proportions and produces a
#' bar chart.  For continuous variables, it bins values across implicates and
#' estimates Rubin-pooled frequencies for each bin.
#'
#' Users may supply a named vector of custom axis labels using the `labels` argument.
#'
#' @param design A `scf_mi_survey` object created by [scf_load()].
#' @param variable A one-sided formula specifying the variable to plot.
#' @param bins Number of bins for continuous variables. Default is 30.
#' @param title Optional plot title. 
#' @param xlab Optional x-axis label.
#' @param ylab Optional y-axis label. Default is "Percent".
#' @param angle Angle for x-axis tick labels. Default is 30.
#' @param fill Fill color for bars. Default is `"#0072B2"`.
#' @param labels Optional named vector of custom axis labels (for discrete variables only).
#'
#' @return A `ggplot2` object.
#'
#' @seealso [scf_theme()]
#'
#' @examples
#' # Do not implement these lines in real analysis:
#' # Use functions `scf_download()` and `scf_load()`
#' td <- tempfile("plot_dist_")
#' dir.create(td)
#'
#' src <- system.file("extdata", "scf2022_mock_raw.rds", package = "scf")
#' file.copy(src, file.path(td, "scf2022.rds"), overwrite = TRUE)
#' scf2022 <- scf_load(2022, data_directory = td)
#'
#' # Example for real analysis: Distribution of homeownership
#' scf_plot_dist(scf2022, ~own)
#'
#' # Example for real analysis: Distribution of age
#' scf_plot_dist(scf2022, ~age, bins = 10)
#'
#' # Do not implement these lines in real analysis: Cleanup for package check
#' unlink(td, recursive = TRUE, force = TRUE)
#'
#' @export
scf_plot_dist <- function(design, variable, bins = 30,
                          title = NULL, xlab = NULL, ylab = "Percent",
                          angle = 30, fill = "#0072B2", labels = NULL) {

  stopifnot(inherits(design, "scf_mi_survey"))
  stopifnot(inherits(variable, "formula"))

  if (isTRUE(attr(design, "mock"))) {
    warning("Mock data detected. Do not interpret results as valid SCF estimates.", call. = FALSE)
  }


  `%||%` <- function(a, b) if (!is.null(a)) a else b
  varname <- all.vars(variable)[1]
  xlab <- xlab %||% varname
  title <- title %||% paste("Distribution of", varname)

  values <- design$mi_design[[1]]$variables[[varname]]

  n_distinct <- function(x) length(unique(x[!is.na(x)]))
  is_discrete <- is.factor(values) || (is.numeric(values) && n_distinct(values) <= 25)

  if (is_discrete) {
    freq <- scf_freq(design, variable, percent = TRUE)
    df <- freq$results
    df$yval <- df$proportion
    df$xval <- factor(df$category, levels = unique(df$category))


    # Apply custom labels if supplied
    if (!is.null(labels)) {
      df$xval <- factor(as.character(df$xval),
                        levels = names(labels),
                        labels = unname(labels))
    }


  } else {
    # Continuous: bin and pool
    all_vals <- unlist(lapply(design$mi_design, function(d) d$variables[[varname]]))
    rng <- range(all_vals, na.rm = TRUE)
    cutpoints <- pretty(rng, bins)
    labels_seq <- paste(head(cutpoints, -1), tail(cutpoints, -1), sep = "-")

    binned <- lapply(seq_along(design$mi_design), function(i) {
      d <- design$mi_design[[i]]
      x <- d$variables[[varname]]
      bin <- cut(x, breaks = cutpoints, include.lowest = TRUE, labels = labels_seq)
      d$variables$bin <- bin
      est <- survey::svymean(~bin, d)
      data.frame(bin = labels_seq,
                 prop = as.numeric(est),
                 var = diag(vcov(est)),
                 implicate = i,
                 stringsAsFactors = FALSE)
    })

    long <- do.call(rbind, binned)
    pooled <- lapply(split(long, long$bin), function(df) {
      qbar <- mean(df$prop)
      ubar <- mean(df$var)
      b <- var(df$prop)
      m <- length(df$implicate)
      se <- sqrt(ubar + (1 + 1/m) * b)
      data.frame(xval = df$bin[1], yval = 100 * qbar, se = 100 * se)
    })
    df <- do.call(rbind, pooled)
  }

  ggplot2::ggplot(df, ggplot2::aes(x = xval, y = yval)) +
    ggplot2::geom_col(fill = fill) +
    ggplot2::labs(title = title, x = xlab, y = ylab) +
    scf_theme() +
    ggplot2::theme(axis.text.x = .scf_axis_text_x(angle))
}

#' Estimate Correlation Between Two Continuous Variables in SCF Microdata
#'
#' Computes the Pearson correlation coefficient between two continuous variables using
#' multiply-imputed, replicate-weighted SCF data. Returns pooled estimates and standard errors
#' using Rubin’s Rules.
#'
#' @description
#' This function estimates the linear association between two continuous variables
#' using Pearson's correlation. Estimates are computed within each implicate and then
#' pooled across implicates to account for imputation uncertainty.
#'
#' @section Implementation:
#' - Inputs: an `scf_mi_survey` object and two one-sided formulas (e.g., `~income`)
#' - Correlation computed using `cor(..., use = "complete.obs")` within each implicate
#' - Rubin’s Rules applied to pool results across implicates
#'
#' @section Interpretation:
#' Pearson’s `$r$` ranges from -1 to +1 and reflects the strength and
#' direction of a linear bivariate association between two continuous variables.
#' Values near 0 indicate weak linear association. Note that the operation is
#' sensitive to outliers and does not capture nonlinear relationships nor adjust
#' for covariates.
#'
#' @param scf An `scf_mi_survey` object, created by [scf_load()]
#' @param var1 One-sided formula specifying the first variable
#' @param var2 One-sided formula specifying the second variable
#'
#'
#' @section Statistical Notes:
#' Correlation is computed within each implicate using complete cases. Rubin’s
#' Rules are applied manually to pool estimates and calculate total variance.
#' This function does not use [scf_MIcombine()], which is intended
#' for vector-valued estimates; direct pooling is more appropriate for
#' scalar statistics like correlation coefficients.
#'
#' @seealso [scf_plot_hex()], [scf_ols()]
#'
#' @return An object of class `scf_corr`, containing: 
#' \describe{
#'   \item{results}{Data frame with pooled correlation estimate, standard error, 
#'     t-statistic, degrees of freedom, p-value, and minimum/maximum values across implicates.}
#'   \item{imps}{Named vector of implicate-level correlations.}
#'   \item{aux}{Variable names used in the estimation.}
#' }
#'
#' @note Degrees of freedom are approximated using a simplified Barnard–Rubin 
#' adjustment, since correlation is a scalar quantity. Interpret cautiously with 
#' few implicates.
#'
#' @examples
#' # Do not implement these lines in real analysis:
#' # Use functions `scf_download()` and `scf_load()`
#' td <- tempfile("corr_")
#' dir.create(td)
#'
#' src <- system.file("extdata", "scf2022_mock_raw.rds", package = "scf")
#' file.copy(src, file.path(td, "scf2022.rds"), overwrite = TRUE)
#' scf2022 <- scf_load(2022, data_directory = td)
#'
#' # Example for real analysis: Correlate income and net worth
#' corr <- scf_corr(scf2022, ~income, ~networth)
#' print(corr)
#' summary(corr)
#'
#' # Do not implement these lines in real analysis: Cleanup for package check
#' unlink(td, recursive = TRUE, force = TRUE)
#' 
#' @export
scf_corr <- function(scf, var1, var2) {
  if (isTRUE(attr(scf, "mock"))) {
    warning("Mock data detected. Do not interpret results as valid SCF estimates.", call. = FALSE)
  }

  if (!inherits(scf, "scf_mi_survey") ||
      !is.list(scf$mi_design) ||
      !all(sapply(scf$mi_design, inherits, "svyrep.design"))) {
    stop("Input must be an 'scf_mi_survey' object with valid replicate-weighted designs.")
  }

  v1 <- all.vars(var1)[1]
  v2 <- all.vars(var2)[1]
  designs <- scf$mi_design
  nimp <- length(designs)

  cors <- sapply(seq_along(designs), function(i) {
    d <- designs[[i]]
    dvars <- d$variables
    stats::cor(dvars[[v1]], dvars[[v2]], use = "complete.obs")
  })
  names(cors) <- paste0("imp", seq_len(nimp))

  qbar <- mean(cors)
  b <- var(cors)
  se <- sqrt(b * (1 + 1/nimp))

  # Rubin degrees of freedom
  r <- (1 + 1/nimp) * b / (b / nimp)
  df <- (nimp - 1) * (1 + 1/r)^2
  tval <- qbar / se
  pval <- 2 * stats::pt(-abs(tval), df = df)

  out <- list(
    results = data.frame(
      correlation = qbar,
      se = se,
      t = tval,
      df = df,
      p.value = pval,
      min = min(cors),
      max = max(cors),
      stringsAsFactors = FALSE
    ),
    imps = cors,
    aux = list(var1 = v1, var2 = v2)
  )
  class(out) <- "scf_corr"
  return(out)
}

#' @export
print.scf_corr <- function(x, ...) {
  cat("SCF Correlation Estimate\n")
  cat(sprintf("Variables: %s and %s\n\n", x$aux$var1, x$aux$var2))
  print(x$results, row.names = FALSE)
  invisible(x)
}

#' @rdname scf_corr
#' @param object A `scf_corr` object returned by [scf_corr()].
#' @param ... Currently unused; included for S3 generic compatibility.
#' @export
summary.scf_corr <- function(object, ...) {
  cat("Summary of SCF Correlation Analysis\n")
  cat("Variables:", object$aux$var1, "and", object$aux$var2, "\n\n")
  cat("Pooled Correlation Estimate:\n")
  print(object$results, row.names = FALSE)
  cat("\nImplicate-level Correlations:\n")
  print(data.frame(
    implicate = names(object$imps),
    correlation = object$imps,
    row.names = NULL
  ))
  invisible(object)
}

#' Combine Estimates Across SCF Implicates Using Rubin's Rules
#'
#' This function implements **Rubin’s Rules** for combining multiply-imputed
#' survey model results in the `scf` package. It pools point estimates,
#' variance-covariance matrices, and degrees of freedom across the SCF’s
#' five implicates.
#' 
#' @section Scope:
#' `scf_MIcombine()` is used for **model-based analyses** such as
#' [scf_ols()], [scf_glm()], and [scf_logit()], where each implicate’s model
#' output includes both parameter estimates and replicate-weighted sampling
#' variances.
#'
#' Descriptive estimators—functions such as [scf_mean()], [scf_percentile()],
#' and [scf_median()]—do **not** apply Rubin’s Rules. They follow the Survey of
#' Consumer Finances convention used in the Federal Reserve Board’s SAS macro,
#' combining (i) the replicate-weight sampling variance from implicate 1 with
#' (ii) the between-implicate variance scaled by (m + 1)/m.
#' 
#' This separation is intentional: descriptive statistics in `scf` aim to
#' reproduce the Survey of Consumer Finances' published standard errors,
#' whereas model-based functions use Rubin's Rules.
#'
#' @section Implementation: 
#' `scf_MIcombine()` pools a set of implicate-level estimates and their
#' associated variance-covariance matrices using Rubin’s Rules.
#'
#' This includes:
#' - Calculation of pooled point estimates
#' - Total variance from within- and between-imputation components
#' - Degrees of freedom via Barnard-Rubin method
#' - Fraction of missing information
#'
#' Inputs are typically produced by modeling functions such as [scf_ols()], 
#' [scf_glm()], or [scf_logit()], which return implicate-level coefficient 
#' vectors and variance-covariance matrices.
#'
#' This function is primarily used internally, but may be called directly by
#' advanced users constructing custom estimation routines from implicate-level
#' results.
#'
#' @section Details:
#' The SCF provides five implicates per survey wave, each a plausible version
#' of the population under a specific missing-data model. Analysts conduct the
#' same statistical procedure on each implicate, producing a set of five
#' estimates \eqn{ Q_1, Q_2, ..., Q_5 }. These are then combined using Rubin’s
#' Rules, a procedure to combine results across these implicates with an
#' attempt to account for:
#'
#' - **Within-imputation variance**: Uncertainty from complex sample design
#' - **Between-imputation variance**: Uncertainty due to missing data
#'
#'For a scalar quantity \eqn{ Q }, the pooled estimate and
#' total variance are calculated as:
#'
#' \deqn{ \bar{Q} = \frac{1}{M} \sum Q_m }
#' \deqn{ \bar{U} = \frac{1}{M} \sum U_m }
#' \deqn{ B = \frac{1}{M - 1} \sum (Q_m - \bar{Q})^2 }
#' \deqn{ T = \bar{U} + \left(1 + \frac{1}{M} \right) B }
#'
#' Where:
#' - \eqn{ M } is the number of implicates (typically 5 for SCF)
#' - \eqn{ Q_m } is the estimate from implicate \eqn{ m }
#' - \eqn{ U_m } is the sampling variance of \eqn{ Q_m }, accounting for replicate weights and design
#'
#' The total variance \eqn{ T } reflects both within-imputation uncertainty (sampling error)
#' and between-imputation uncertainty (missing-data imputation).
#'
#' The standard error of the pooled estimate is \eqn{ \sqrt{T} }. Degrees of freedom are
#' adjusted using the Barnard-Rubin method:
#'
#' \deqn{ \nu = (M - 1) \left(1 + \frac{\bar{U}}{(1 + \frac{1}{M}) B} \right)^2 }
#'
#' The fraction of missing information (FMI) is also reported:
#' it reflects the proportion of total variance attributable to imputation uncertainty.
#'
#' @param results A list of implicate-level model outputs. Each element must be a named numeric vector
#' or an object with methods for `coef()` and `vcov()`. Typically generated internally by modeling functions.
#' @param variances Optional list of variance-covariance matrices. If omitted, extracted using `vcov()`.
#' @param call Optional. The originating function call. Defaults to `sys.call()`.
#' @param df.complete Optional degrees of freedom for the complete-data model. Used for small-sample
#' corrections. Defaults to `Inf`, assuming large-sample asymptotics.
#' @param object A pooled result object of class \code{"scf_MIresult"} (for \code{SE()}).
#' @param ... Not used.
#'
#' @return An object of class `"scf_MIresult"` with components:
#' \describe{
#'   \item{coefficients}{Pooled point estimates across implicates.}
#'   \item{variance}{Pooled variance-covariance matrix.}
#'   \item{df}{Degrees of freedom for each parameter, adjusted using Barnard-Rubin formula.}
#'   \item{missinfo}{Estimated fraction of missing information for each parameter.}
#'   \item{nimp}{Number of implicates used in pooling.}
#'   \item{call}{Function call recorded for reproducibility.}
#' }
#'
#' Supports `coef()`, [SE()], `confint()`, and `summary()` methods.
#'
#' @examples
#' # Do not implement these lines in real analysis:
#' # Use functions `scf_download()` and `scf_load()`
#' td <- tempfile("MIcombine_")
#' dir.create(td)
#' 
#' src <- system.file("extdata", "scf2022_mock_raw.rds", package = "scf")
#' file.copy(src, file.path(td, "scf2022.rds"), overwrite = TRUE)
#' scf2022 <- scf_load(2022, data_directory = td)
#'
#' # Example for real analysis: Pool simple survey mean for mock data
#' outlist <- lapply(scf2022$mi_design, function(d) survey::svymean(~I(age >= 65), d))
#' pooled  <- scf_MIcombine(outlist)     # vcov/coef extracted automatically
#' SE(pooled); coef(pooled)
#' 
#' unlink(td, recursive = TRUE, force = TRUE)
#'
#' @references
#'
#' Barnard J, Rubin DB. Small-sample degrees of freedom with multiple imputation.
#'   \doi{10.1093/biomet/86.4.948}.
#'   
#' Little RJA, Rubin DB. Statistical analysis with missing data.
#'   ISBN: 9780470526798.
#'   
#' U.S. Federal Reserve. Codebook for 2022 Survey of Consumer Finances.
#'   https://www.federalreserve.gov/econres/scfindex.htm
#'   
#' @export
scf_MIcombine <- function(results, variances, call = sys.call(), df.complete = Inf) {
  m <- length(results)
  if (missing(variances)) {
    variances <- suppressWarnings(lapply(results, vcov))
    results <- lapply(results, coef)
  }

  vbar <- Reduce("+", variances) / m
  cbar <- Reduce("+", results) / m
  evar <- var(do.call("rbind", results))

  r <- (1 + 1/m) * evar / vbar
  df <- (m - 1) * (1 + 1/r)^2
  if (is.matrix(df)) df <- diag(df)

  if (is.finite(df.complete)) {
    dfobs <- ((df.complete + 1)/(df.complete + 3)) * df.complete * vbar / (vbar + evar)
    if (is.matrix(dfobs)) dfobs <- diag(dfobs)
    df <- 1 / (1/dfobs + 1/df)
  }

  missinfo <- (r + 2/(df + 3)) / (r + 1)

  structure(
    list(
      coefficients = cbar,
      variance = vbar + (1 + 1/m) * evar,
      call = call,
      nimp = m,
      df = df,
      missinfo = missinfo
    ),
    class = "scf_MIresult"
  )
}

#' @rdname scf_MIcombine
#' @export
SE <- function(object, ...) UseMethod("SE")

#' @rdname scf_MIcombine
#' @export
SE.scf_MIresult <- function(object, ...) {
  sqrt(diag(object$variance))
}

#' @export
coef.scf_MIresult <- function(object, ...) {
  object$coefficients
}

#' @export
vcov.scf_MIresult <- function(object, ...) {
  object$variance
}

#' Hexbin Plot of Two Continuous SCF Variables

#' @description
#' Visualizes the bivariate relationship between two continuous SCF variables
#' using hexagonal bins. 
#'
#' @section Implementation:
#' The function stacks all implicates into one data frame, retains replicate weights,
#' and uses `ggplot2::geom_hex()` to produce a density-style scatterplot. The color
#' intensity of each hexagon reflects the Rubin-pooled weighted count of households
#' in that cell. Missing values are excluded.
#'
#' This plot is especially useful for visualizing joint distributions with large
#' samples and skewed marginals, such as net worth vs. income.
#'
#' @section Aesthetic Guidance:
#' This plot uses a log-scale fill and `viridis` palette to highlight variation
#' in density. To adjust the visual style globally, use [scf_theme()] or set it
#' explicitly with `ggplot2::theme_set(scf_theme())`. For mobile-friendly or
#' publication-ready appearance, export the plot at 5.5 x 5.5 inches, 300 dpi.
#'
#' @param design A `scf_mi_survey` object created by [scf_load()].
#' @param x A one-sided formula for the x-axis variable (e.g., `~income`).
#' @param y A one-sided formula for the y-axis variable (e.g., `~networth`).
#' @param bins Integer. Number of hexagonal bins along the x-axis. Default is `50`.
#' @param title Optional character string for the plot title.
#' @param xlab Optional x-axis label. Defaults to the variable name.
#' @param ylab Optional y-axis label. Defaults to the variable name.
#'
#' @return A `ggplot2` object displaying a Rubin-pooled hexbin plot.
#'
#' @section Dependencies:
#' Requires the `ggplot2` package. The fill scale uses `scale_fill_viridis_c()` from `ggplot2`.
#' Requires the \pkg{hexbin} package. The function will stop with an error if it is not installed.
#'
#'
#' @examples
#' # Do not implement these lines in real analysis:
#' # Use functions `scf_download()` and `scf_load()`
#' td <- tempfile("plot_hex_")
#' dir.create(td)
#' 
#' src <- system.file("extdata", "scf2022_mock_raw.rds", package = "scf")
#' file.copy(src, file.path(td, "scf2022.rds"), overwrite = TRUE)
#' scf2022 <- scf_load(2022, data_directory = td)
#'
#' # Example for real analysis: Plot hexbin of income vs. net worth
#' # Note: mock data has ~75 rows per implicate; hexbin output will be sparse.
#' # Results on full SCF data will show a meaningful joint density.
#' scf_plot_hex(scf2022, ~income, ~networth)
#' 
#' # Do not implement these lines in real analysis: Cleanup for package check
#' unlink(td, recursive = TRUE, force = TRUE)
#'
#' @seealso [scf_corr()], [scf_plot_smooth()], [scf_theme()]
#'
#' @export
scf_plot_hex <- function(design, x, y,
                         bins = 50,
                         title = NULL,
                         xlab = NULL,
                         ylab = NULL) {

  if (!requireNamespace("ggplot2", quietly = TRUE)) {
    stop("Package 'ggplot2' is required. Please install it.")
  }

  `%||%` <- function(a, b) if (!is.null(a)) a else b

  if (isTRUE(attr(design, "mock"))) {
    warning("Mock data detected. Do not interpret results as valid SCF estimates.", call. = FALSE)
  }

  if (!inherits(x, "formula") || !inherits(y, "formula")) {
    stop("Both `x` and `y` must be one-sided formulas, e.g., ~income, ~networth")
  }

  if (!requireNamespace("hexbin", quietly = TRUE)) {
    stop("The 'hexbin' package is required for scf_plot_hex(). Please install it.")
  }

  xname <- as.character(x[[2]])
  yname <- as.character(y[[2]])

  data_list <- lapply(seq_along(design$mi_design), function(i) {
    d <- design$mi_design[[i]]
    df <- data.frame(
      x = d$variables[[xname]],
      y = d$variables[[yname]],
      wgt = as.numeric(weights(d)),
      imp = i
    )
    df
  })

  data_all <- do.call(rbind, data_list)

  data_all <- data_all[!is.na(data_all$x) & !is.na(data_all$y) & !is.na(data_all$wgt), ]

  ggplot2::ggplot(data_all, ggplot2::aes(x = x, y = y)) +
    ggplot2::geom_hex(ggplot2::aes(weight = wgt), bins = bins) +
    ggplot2::scale_fill_viridis_c(trans = "log10", name = "Weighted Count") +
    ggplot2::labs(
      title = title %||% paste("Weighted Hexbin:", yname, "vs", xname),
      x = xlab %||% xname,
      y = ylab %||% yname
    ) +
    scf_theme()
}

#' @importFrom stats formula residuals coef vcov AIC predict family
NULL

#' S3 Methods for scf_model_result Objects
#'
#' @description
#' Generic S3 methods dispatched on objects of class \code{"scf_model_result"},
#' as returned by \code{\link{scf_ols}}, \code{\link{scf_glm}},
#' \code{\link{scf_logit}}, and \code{\link{scf_quantreg}}.
#'
#' \describe{
#'   \item{\code{coef()}}{Pooled coefficient estimates (Rubin's Rules).}
#'   \item{\code{vcov()}}{Pooled variance-covariance matrix.}
#'   \item{\code{AIC()}}{Mean AIC across implicates.}
#'   \item{\code{residuals()}}{Residuals from the first implicate model (diagnostic use only).}
#'   \item{\code{predict()}}{Mean predictions pooled across all five implicate models.}
#'   \item{\code{formula()}}{The model formula.}
#' }
#'
#' @param object An object of class \code{"scf_model_result"}.
#' @param x An object of class \code{"scf_model_result"} (for \code{formula}).
#' @param k Penalty term passed to \code{AIC()} (default 2 for AIC, \code{log(n)} for BIC).
#' @param newdata Optional data frame of new observations for \code{predict()}.
#'   If missing, predictions are made on the original training data.
#' @param type Prediction scale for \code{predict()}: \code{"link"} (default)
#'   or \code{"response"}.
#' @param ... Additional arguments (not used by most methods).
#'
#' @name scf_model_result_methods
#' @aliases coef.scf_model_result vcov.scf_model_result AIC.scf_model_result
#'   residuals.scf_model_result predict.scf_model_result formula.scf_model_result
NULL

#' @rdname scf_model_result_methods
#' @export
formula.scf_model_result <- function(x, ...) {
  if (is.null(x$formula)) {
    stop("Formula not found in the model object. Ensure the model function saves the formula argument.")
  }
  return(x$formula)
}

# ----------------------------------------------------------------------

#' @rdname scf_model_result_methods
#' @export
residuals.scf_model_result <- function(object, ...) {
  # Implicate models may be stored under $models (scf_glm, scf_quantreg) or
  # $imps (scf_ols, scf_logit). Check both to ensure uniform behavior.
  imp_models <- if (!is.null(object$models) && length(object$models) > 0) {
    object$models
  } else if (!is.null(object$imps) && length(object$imps) > 0) {
    object$imps
  } else {
    NULL
  }
  if (is.null(imp_models)) {
    stop("No underlying models found to extract residuals.")
  }
  return(stats::residuals(imp_models[[1]]))
}

# ----------------------------------------------------------------------

#' @rdname scf_model_result_methods
#' @export
coef.scf_model_result <- function(object, ...) {
  # Estimates are stored in the results data frame
  est <- object$results$estimate
  names(est) <- object$results$term
  return(est)
}

# ----------------------------------------------------------------------

#' @rdname scf_model_result_methods
#' @export
vcov.scf_model_result <- function(object, ...) {
  if (is.null(object$vcov)) {
    stop("Pooled variance-covariance matrix not found. Re-fit the model with the current version of the package.")
  }
  object$vcov
}

# ----------------------------------------------------------------------

#' @rdname scf_model_result_methods
#' @export
AIC.scf_model_result <- function(object, k = 2, ...) {
  if (is.null(object$fit$AIC) || is.na(object$fit$AIC)) {
    stop("AIC not available in model diagnostics ('fit$AIC').")
  }
  # For multiply imputed models, we typically use the mean AIC across implicates.
  # Note: This is an approximation and does not fully conform to MI literature on AIC.
  return(object$fit$AIC) 
}


# ----------------------------------------------------------------------

#' @rdname scf_model_result_methods
#' @export
predict.scf_model_result <- function(object, newdata, type = "link", ...) {

  # Implicate models may be stored under $models (scf_glm, scf_quantreg) or
  # $imps (scf_ols, scf_logit). Resolve once and use throughout.
  imp_models <- if (!is.null(object$models) && length(object$models) > 0) {
    object$models
  } else if (!is.null(object$imps) && length(object$imps) > 0) {
    object$imps
  } else {
    NULL
  }
  if (is.null(imp_models)) {
    stop("No underlying models found to generate predictions.")
  }

  # --- 1. Determine Model Type and Prediction Scale ---

  pred_type <- match.arg(type, c("link", "response"))

  # Try to infer family (necessary for predict.glm to handle links/responses)
  model_family <- tryCatch({
    stats::family(imp_models[[1]])
  }, error = function(e) list(family = "gaussian"))

  # --- 2. Predict on each Implicate Model ---

  all_preds <- lapply(imp_models, function(m) {
    # If no new data is provided, use the original data attached to the model
    local_newdata <- if (missing(newdata)) {
      m$model 
      # Fallback to model.frame if m$model is missing (e.g. for svyglm objects)
      # More reliably, if no newdata is provided, the goal is to predict on original data.
    } else {
      newdata
    }
    
    # If using OLS (gaussian) or a model like Poisson, 'link' and 'response' scale predictions
    # can be directly averaged. For binomial, we almost always average on the 'response' scale.
    
    # Run the prediction using the chosen type
    p <- tryCatch({
      stats::predict(m, newdata = local_newdata, type = pred_type, ...)
    }, error = function(e) {
      stop(paste("Prediction failed for an underlying model. Error:", conditionMessage(e)))
    })
    
    # Convert prediction to a vector if it came back as a matrix or data frame
    as.vector(p)
  })
  
  # --- 3. Pool Predictions ---
  
  if (length(unique(sapply(all_preds, length))) > 1) {
    stop("Prediction lengths are inconsistent across implicates. Check 'newdata'.")
  }
  
  # Stack predictions into a matrix (one column per implicate)
  pred_matrix <- do.call(cbind, all_preds)
  
  # The pooled prediction is simply the mean across all implicates for each observation.
  pooled_prediction <- rowMeans(pred_matrix, na.rm = TRUE)
  
  return(pooled_prediction)
}

#' Modify Each Implicate Individually in SCF Data
#'
#' Applies a user-defined transformation to each implicate's data frame separately.
#' This is useful when you need to compute values that depend on the distribution
#' within each implicate — such as ranks, percentiles, or groupwise comparisons —
#' which cannot be computed reliably using [scf_update()].
#'
#' @description
#' Each household in SCF data is represented by five *implicates*, which reflect
#' uncertainty from the imputation process. Most transformations — such as computing
#' log income or assigning categorical bins — can be applied uniformly across implicates
#' using [scf_update()]. However, some operations depend on the *internal distribution*
#' of variables within each implicate. For those, you need to modify each one separately.
#'
#' This function extracts each implicate from the replicate-weighted survey design,
#' applies your transformation, and rebuilds the survey design objects accordingly.
#'
#' @section Use this When:
#' - You need implicate-specific quantiles (e.g., flag households in the top 10% of wealth)
#' - You want to assign percentile ranks (e.g., income percentile by implicate)
#' - You are computing statistics within groups (e.g., groupwise z-scores)
#' - You need to derive a variable based on implicate-specific thresholds or bins
#'
#' @param object A `scf_mi_survey` object from [scf_load()].
#' @param f A function that takes one implicate's data frame as its sole argument
#'   and returns a modified data frame with the same number of rows. The function
#'   signature should be \code{function(df) \{ ... \}}. The returned data frame is
#'   used to rebuild the replicate-weighted survey design.
#'   
#'
#' @return A modified `scf_mi_survey` object with updated implicate-level designs.
#'
#' @examples
#' # Do not implement these lines in real analysis:
#' # Use functions `scf_download()` and `scf_load()`
#' td <- tempfile("update_by_implicate_")
#' dir.create(td)
#' src <- system.file("extdata", "scf2022_mock_raw.rds", package = "scf")
#' file.copy(src, file.path(td, "scf2022.rds"), overwrite = TRUE)
#' scf2022 <- scf_load(2022, data_directory = td)
#'
#' # Example for real analysis: flag households in the top 10% of net worth,
#' # using the unweighted implicate-specific 90th percentile as the threshold.
#' scf2022 <- scf_update_by_implicate(scf2022, function(df) {
#'   threshold <- stats::quantile(df$networth, probs = 0.90, na.rm = TRUE)
#'   df$top10nw <- df$networth >= threshold
#'   df
#' })
#'
#' # Example for real analysis: compute implicate-specific z-scores of income
#' scf2022 <- scf_update_by_implicate(scf2022, function(df) {
#'   mu <- mean(df$income, na.rm = TRUE)
#'   sigma <- stats::sd(df$income, na.rm = TRUE)
#'   df$z_income <- (df$income - mu) / sigma
#'   df
#' })
#'
#' # Verify new variable exists
#' head(scf2022$mi_design[[1]]$variables$z_income)
#' 
#' # Do not implement these lines in real analysis: Cleanup for package check
#' unlink(td, recursive = TRUE, force = TRUE)
#'
#' @seealso [scf_update()]
#' @export
scf_update_by_implicate <- function(object, f) {
  if (!inherits(object, "scf_mi_survey")) {
    stop("Input must be a 'scf_mi_survey' object.")
  }
  if (!is.function(f)) {
    stop("Argument 'f' must be a function that accepts a data frame and returns a data frame.")
  }

  updated_designs    <- vector("list", length(object$mi_design))

  for (i in seq_along(object$mi_design)) {
    design <- object$mi_design[[i]]
    df     <- design$variables
    new_df <- f(df)
    
    if (!is.data.frame(new_df)) {
      stop(sprintf("Function `f` must return a data.frame. Implicate %d returned: %s",
                   i, class(new_df)))
    }
    if (nrow(new_df) != nrow(df)) {
      stop(sprintf("Row count mismatch in implicate %d: original = %d, new = %d",
                   i, nrow(df), nrow(new_df)))
    }
    
    rep_cols <- grep("^wt1b", names(new_df), value = TRUE)
    if (length(rep_cols) == 0) {
      stop("Could not find replicate weight columns in implicate.")
    }
    
    updated_designs[[i]] <- survey::svrepdesign(
      weights = ~wgt,
      repweights = as.matrix(new_df[, rep_cols]),
      data = new_df,
      type = "other",
      scale = 1,
      rscales = rep(1 / (length(rep_cols) - 1), length(rep_cols)),
      mse = TRUE,
      combined.weights = TRUE
    )
  }
  
  object$mi_design <- updated_designs
  return(object)
}

#' Estimate Percentiles in SCF Microdata
#'
#' This function estimates a weighted percentile of a continuous variable
#' in the Survey of Consumer Finances (SCF). Two methods are available.
#' The default \code{"implicate"} method estimates the percentile separately
#' within each implicate using [survey::svyquantile()] and pools the results
#' following the SCF Bulletin variance convention. The \code{"stack"} method
#' replicates the Federal Reserve's published convention by pooling all five
#' implicates with weights divided by five and computing a single weighted
#' quantile from the combined sample. The two methods will generally produce
#' similar but not identical results. Use \code{method = "stack"} when exact
#' replication of Federal Reserve published figures is required.
#'
#' The implicate method computes estimates as follows:
#' 1. For each implicate, estimate the requested percentile using
#'    [survey::svyquantile()] with `se = TRUE`.
#' 2. The reported point estimate is the mean of the M implicate-specific
#'    percentile estimates.
#' 3. The standard error follows the SCF Bulletin SAS macro convention:
#'
#'        V_total = V1 + ((M + 1) / M) * B
#'
#'    where:
#'    - V1 is the replicate-weight sampling variance of the percentile
#'      from the first implicate only.
#'    - B  is the between-implicate variance of the percentile estimates.
#'
#'    The reported standard error is sqrt(V_total).
#'
#' 4. If a grouping variable is supplied, the same logic is applied
#'    separately within each group.
#'
#' The stack method returns no standard error, as the estimate is a
#' deterministic weighted quantile rather than a model-based estimate.
#'
#' @param scf A `scf_mi_survey` object created with [scf_load()]. Must
#'   contain the list of replicate-weighted designs for each implicate in
#'   `scf$mi_design`.
#' @param var A one-sided formula naming the continuous variable to
#'   summarize (for example `~networth`).
#' @param q Numeric percentile in between 0 and 1. Default 0.5 (median).
#' @param method Character. One of \code{"implicate"} (default) or
#'   \code{"stack"}. The implicate method averages percentile estimates
#'   across the five implicates using the SCF Bulletin variance convention.
#'   The stack method pools all five implicates into a single dataset with
#'   weights divided by five and computes a single weighted quantile,
#'   replicating the Federal Reserve's published figures. When
#'   \code{method = "stack"} no standard error is returned, as the stacked
#'   estimate is a deterministic weighted quantile rather than a
#'   model-based estimate.
#' @param by Optional one-sided formula naming a categorical grouping
#'   variable. If supplied, the percentile is estimated separately within
#'   each group.
#' @param verbose Logical. If TRUE, include implicate-level estimates in
#'   the returned object for inspection. Default FALSE.
#'
#' @return An object of class `"scf_percentile"` containing:
#' \describe{
#'   \item{results}{A data frame containing pooled percentile estimates, pooled
#'     standard errors, and implicate min/max values. One row per group (if
#'     `by` is supplied) or one row otherwise.}
#'   \item{imps}{A list of implicate-level percentile estimates and standard errors.}
#'   \item{aux}{A list containing the variable name, optional group variable name,
#'     and the quantile requested.}
#'   \item{verbose}{Logical flag indicating whether implicate-level estimates
#'     should be printed by `print()` or `summary()`.}
#' }
#'
#' @examples
#' # Do not implement these lines in real analysis:
#' # Use functions `scf_download()` and `scf_load()` for actual SCF data
#' td <- tempfile("percentile_")
#' dir.create(td)
#' 
#' src <- system.file("extdata", "scf2022_mock_raw.rds", package = "scf")
#' file.copy(src, file.path(td, "scf2022.rds"), overwrite = TRUE)
#' scf2022 <- scf_load(2022, data_directory = td)
#'
#' # Estimate the 75th percentile of net worth
#' scf_percentile(scf2022, ~networth, q = 0.75)
#'
#' # Estimate the median net worth by ownership group
#' scf_percentile(scf2022, ~networth, q = 0.5, by = ~own)
#'
#' # Do not implement these lines in real analysis: Cleanup for package check
#' unlink(td, recursive = TRUE, force = TRUE)
#' rm(scf2022)
#'
#' @references
#' Federal Reserve Board. 2023c. "SAS Macro: Variable Definitions."
#' https://www.federalreserve.gov/econres/files/bulletin.macro.txt
#'
#' @seealso [scf_median()], [scf_mean()]
#'
#' @export
scf_percentile <- function(scf, var, q = 0.5, by = NULL, 
                           verbose = FALSE,
                           method = c("implicate", "stack")) {
  
  method <- match.arg(method)
  
  # Basic checks
  if (!inherits(scf, "scf_mi_survey")) {
    stop("`scf` must be an object of class 'scf_mi_survey'.", call. = FALSE)
  }
  if (!is.numeric(q) || length(q) != 1L || q < 0 || q > 1) {
    stop("`q` must be a single numeric value in [0, 1].", call. = FALSE)
  }
  
  # Warn if mock data is being used
  if (isTRUE(attr(scf, "mock"))) {
    warning(
      "Mock data detected. Do not interpret results as valid SCF estimates.",
      call. = FALSE
    )
  }
  
  # Extract variable names from formulas
  varname <- all.vars(var)[1]
  byname  <- if (!is.null(by)) all.vars(by)[1] else NULL
  
  designs <- scf$mi_design
  M       <- length(designs)
  if (M < 1L) {
    stop("`scf$mi_design` must contain at least one implicate design.", call. = FALSE)
  }
  
  # If grouped, harmonize factor levels across implicates
  if (!is.null(byname)) {
    for (i in seq_len(M)) {
      designs[[i]]$variables[[byname]] <- factor(designs[[i]]$variables[[byname]])
    }
    groups <- levels(designs[[1]]$variables[[byname]])
  }
  
  # Internal utility: estimate percentile for one design, optional group subset,
  # with controlled handling of the known replicate-weight warning.
  get_quantile_obj <- function(dsgn, vname, qval, g = NULL, gname = NULL) {
    if (!is.null(g)) {
      dsgn <- subset(dsgn, dsgn$variables[[gname]] == g)
    }
    
    form <- stats::as.formula(paste0("~", vname))
    
    suppressMessages(
      suppressWarnings(
        survey::svyquantile(
          form,
          dsgn,
          quantiles     = qval,
          se            = TRUE,
          interval.type = "quantile"
        )
      )
    )
  }
  
  # --- Stack method (Federal Reserve convention) ---
  if (method == "stack") {
    all_vars <- do.call(rbind, lapply(designs, function(d) d$variables))
    if (!is.null(byname)) {
      groups <- levels(factor(all_vars[[byname]]))
      out_df <- do.call(rbind, lapply(groups, function(g) {
        idx <- all_vars[[byname]] == g
        est <- .scf_wtd_quantile(x     = all_vars[[varname]][idx],
                                 w     = all_vars$wgt[idx] / M,
                                 probs = q)
        data.frame(group = g, variable = varname, quantile = q,
                   estimate = est, se = NA_real_,
                   min = NA_real_, max = NA_real_,
                   stringsAsFactors = FALSE)
      }))
    } else {
      est <- .scf_wtd_quantile(x     = all_vars[[varname]],
                               w     = all_vars$wgt / M,
                               probs = q)
      out_df <- data.frame(variable = varname, quantile = q,
                           estimate = est, se = NA_real_,
                           min = NA_real_, max = NA_real_,
                           stringsAsFactors = FALSE)
    }
    return(structure(
      list(results = out_df, imps = NULL,
           aux     = list(varname = varname, byname = byname, quantile = q),
           verbose = verbose),
      class = "scf_percentile"
    ))
  }
  
  if (is.null(byname)) {
    ## -------------------------------------------------
    ## Ungrouped case (whole population)
    ## -------------------------------------------------
    
    imp_objs <- lapply(seq_len(M), function(i) {
      get_quantile_obj(designs[[i]], varname, q)
    })
    
    imp_estimates <- lapply(seq_len(M), function(i) {
      obj_i <- imp_objs[[i]]
      data.frame(
        implicate = i,
        group     = "All",
        quantile  = q,
        estimate  = as.numeric(stats::coef(obj_i)),
        se        = sqrt(diag(stats::vcov(obj_i))),
        stringsAsFactors = FALSE
      )
    })
    
    point_vec <- sapply(imp_objs, function(o) as.numeric(stats::coef(o)))
    qbar      <- mean(point_vec)
    
    V1 <- stats::vcov(imp_objs[[1]])      # 1 x 1
    B  <- stats::var(point_vec)          # scalar
    
    V_total   <- as.numeric(V1) + ((M + 1) / M) * B
    se_pooled <- sqrt(V_total)
    
    out_df <- data.frame(
      variable = varname,
      quantile = q,
      estimate = qbar,
      se       = se_pooled,
      min      = min(point_vec),
      max      = max(point_vec),
      stringsAsFactors = FALSE
    )
    
  } else {
    ## -------------------------------------------------
    ## Grouped case (within each category of `by`)
    ## -------------------------------------------------
    
    imp_objs <- lapply(seq_len(M), function(i) {
      dsgn_i <- designs[[i]]
      lapply(groups, function(g) {
        get_quantile_obj(dsgn_i, varname, q, g = g, gname = byname)
      })
    })
    
    imp_estimates <- lapply(seq_len(M), function(i) {
      objs_i <- imp_objs[[i]]
      do.call(rbind, lapply(seq_along(groups), function(j) {
        obj_ij <- objs_i[[j]]
        data.frame(
          implicate = i,
          group     = groups[j],
          quantile  = q,
          estimate  = as.numeric(stats::coef(obj_ij)),
          se        = sqrt(diag(stats::vcov(obj_ij))),
          stringsAsFactors = FALSE
        )
      }))
    })
    
    out_df <- do.call(rbind, lapply(seq_along(groups), function(j) {
      group_objs <- lapply(seq_len(M), function(i) imp_objs[[i]][[j]])
      point_vec  <- sapply(group_objs, function(o) as.numeric(stats::coef(o)))
      
      qbar_g <- mean(point_vec)
      V1_g   <- stats::vcov(group_objs[[1]])
      B_g    <- stats::var(point_vec)
      
      V_total_g   <- as.numeric(V1_g) + ((M + 1) / M) * B_g
      se_pooled_g <- sqrt(V_total_g)
      
      data.frame(
        group    = groups[j],
        variable = varname,
        quantile = q,
        estimate = qbar_g,
        se       = se_pooled_g,
        min      = min(point_vec),
        max      = max(point_vec),
        stringsAsFactors = FALSE
      )
    }))
    
    out_df <- out_df[, c("group", "variable", "quantile",
                         "estimate", "se", "min", "max")]
  }
  
  names(imp_estimates) <- paste0("imp", seq_len(M))
  
  structure(
    list(
      results = out_df,
      imps    = imp_estimates,
      aux     = list(
        varname  = varname,
        byname   = byname,
        quantile = q,
        year     = scf$year
      ),
      verbose = verbose
    ),
    class = "scf_percentile"
  )
}

#' @export
print.scf_percentile <- function(x, ...) {
  cat("SCF Percentile Estimate\n\n")
  dollar_label <- if (isTRUE(attr(x, "deflated")))
    sprintf(" (%d$)", attr(x, "base_year")) else ""
  cat(sprintf("SCF Percentile Estimate (SCF Bulletin convention)%s\n\n", dollar_label))

  print(x$results, row.names = FALSE, ...)
  if (all(is.na(x$results$se))) {
    cat("\nNote: Standard error not available for stack method.\n")
  }
  if (isTRUE(x$verbose) && !is.null(x$imps)) {
    cat("\nImplicate-Level Estimates:\n\n")
    imp_df <- do.call(rbind, x$imps)
    print(imp_df, row.names = FALSE)
  }
  invisible(x)
}

#' @export
summary.scf_percentile <- function(object, ...) {
  cat("Summary of SCF Percentile Estimate\n\n")
  cat("Pooled Estimates:\n")
  print(format(object$results, digits = 4, nsmall = 2),
        row.names = FALSE, ...)
  if (!is.null(object$imps)) {
    cat("\nImplicate-Level Estimates:\n")
    imp_df <- do.call(rbind, object$imps)
    print(format(imp_df, digits = 4, nsmall = 2), row.names = FALSE)
  } else {
    cat("\nImplicate-level estimates not available for stack method.\n")
  }
  invisible(object)
}

# Internal helper: weighted quantile.
#
# Finds the first observation at which the cumulative weighted population
# share reaches or exceeds each requested probability.
.scf_wtd_quantile <- function(x, w, probs) {
  ok <- is.finite(x) & is.finite(w) & w > 0
  
  x <- x[ok]
  w <- w[ok]
  
  if (length(x) == 0L) {
    return(rep(NA_real_, length(probs)))
  }
  
  ord <- order(x)
  x <- x[ord]
  w <- w[ord]
  
  cum_w <- cumsum(w) / sum(w)
  
  as.numeric(vapply(probs, function(p) {
    idx <- which(cum_w >= p)[1L]
    if (is.na(idx)) NA_real_ else x[idx]
  }, numeric(1)))
}


# Internal helper: weighted median.
#
# Used by scf_pctile_sum(method = "stack", stat = "median").
.scf_wtd_median <- function(x, w) {
  .scf_wtd_quantile(x = x, w = w, probs = 0.5)
}

#' Smoothed Distribution Plot of a Continuous Variable in SCF Data
#'
#' @description
#' Draws a smoothed distribution plot of a continuous variable in the SCF. Use
#' this function to visualize a single continuous variable's distribution. 
#'
#' @section Implementation:
#' Visualizes the weighted distribution of a continuous SCF variable by stacking implicates,
#' binning observations, and smoothing pooled proportions. This function is useful for
#' examining distribution shape, skew, or modality in variables like income or wealth.
#'
#' All implicates are stacked and weighted, binned across a data-driven or user-specified
#' bin width. Each bin's weight share is calculated, and a smoothing curve is fit to
#' the resulting pseudo-density.
#'
#' @param design A `scf_mi_survey` object created by [scf_load()].
#' @param variable A one-sided formula specifying a continuous variable (e.g., `~networth`).
#' @param binwidth Optional bin width. Default uses Freedman–Diaconis rule.
#' @param xlim Optional numeric vector of length 2 to truncate axis.
#' @param method Character. Smoothing method: `"loess"` (default) or `"lm"`.
#' @param span Numeric LOESS span. Default is `0.2`. Ignored if `method = "lm"`.
#' @param color Line color. Default is `"blue"`.
#' @param xlab Optional label for x-axis. Defaults to the variable name.
#' @param ylab Optional label for y-axis. Defaults to `"Percent of Households"`.
#' @param title Optional plot title.
#'
#' @return A `ggplot2` object.
#'
#' @seealso [scf_plot_hist()], [scf_plot_dist()], [scf_theme()]
#'
#' @examples
#' # Do not implement these lines in real analysis:
#' # Use functions `scf_download()` and `scf_load()`
#' td <- tempfile("plot_smooth_")
#' dir.create(td)
#' 
#' src <- system.file("extdata", "scf2022_mock_raw.rds", package = "scf")
#' file.copy(src, file.path(td, "scf2022.rds"), overwrite = TRUE)
#' scf2022 <- scf_load(2022, data_directory = td)
#'
#' # Example for real analysis: Plot smoothed distribution
#' scf_plot_smooth(scf2022, ~networth, xlim = c(0, 2e6),
#'                 method = "loess", span = 0.25)
#'      
#' # Do not implement these lines in real analysis: Cleanup for package check           
#' unlink(td, recursive = TRUE, force = TRUE)
#'
#' @export
scf_plot_smooth <- function(design,
                            variable,
                            binwidth = NULL,
                            xlim = NULL,
                            method = "loess",
                            span = 0.2,
                            color = "blue",
                            xlab = NULL,
                            ylab = "Percent of Households",
                            title = NULL) {

  stopifnot(inherits(design, "scf_mi_survey"))
  stopifnot(inherits(variable, "formula"))

  if (isTRUE(attr(design, "mock"))) {
    warning("Mock data detected. Do not interpret results as valid SCF estimates.", call. = FALSE)
  }


  `%||%` <- function(a, b) if (!is.null(a)) a else b
  varname <- as.character(variable[[2]])

  # Stack implicates
  stacked <- lapply(seq_along(design$mi_design), function(i) {
    d <- design$mi_design[[i]]
    df <- d$variables
    data.frame(
      x = df[[varname]],
      wgt = as.numeric(weights(d)),
      imp = i
    )
  })
  df <- do.call(rbind, stacked)
  df <- df[!is.na(df$x) & !is.na(df$wgt), ]

  if (!is.null(xlim)) {
    df <- df[df$x >= xlim[1] & df$x <= xlim[2], ]
  }
  if (nrow(df) == 0) stop("No valid data after subsetting.")

  # Binwidth estimation
  if (is.null(binwidth)) {
    iqr <- IQR(df$x, na.rm = TRUE)
    n <- nrow(df)
    binwidth <- 2 * iqr / n^(1/3)
    if (!is.finite(binwidth) || binwidth <= 0) {
      binwidth <- diff(range(df$x, na.rm = TRUE)) / 50
    }
  }

  # Create bins
  breaks <- seq(floor(min(df$x) / binwidth) * binwidth,
                ceiling(max(df$x) / binwidth) * binwidth,
                by = binwidth)
  mids <- head(breaks, -1) + binwidth / 2
  df$bin_index <- cut(df$x, breaks = breaks, include.lowest = TRUE, labels = FALSE)
  df$bin_center <- mids[df$bin_index]

  # Aggregate proportions
  agg <- aggregate(wgt ~ bin_center, data = df, sum)
  names(agg)[1] <- "x"
  agg$percent <- 100 * agg$wgt / sum(agg$wgt)

  # Plot
  ggplot2::ggplot(agg, ggplot2::aes(x = x, y = percent)) +
    ggplot2::geom_smooth(
      method = method,
      span = if (method == "loess") span else NULL,
      se = FALSE,
      color = color
    ) +
    ggplot2::labs(
      title = title,
      x = xlab %||% varname,
      y = ylab
    ) +
    scf_theme()
}

#' Estimate the Population Median of a Continuous SCF Variable
#'
#' @description
#' Estimates the median (50th percentile) of a continuous SCF variable. Use this
#' operation to characterize a typical or average value.  In contrast to
#' [scf_mean()], this function is both uninfluenced by, and insensitive to,
#' outliers.
#'
#' @section Implementation:
#' This function wraps [scf_percentile()] with `q = 0.5`. The user supplies a
#' `scf_mi_survey` object and a one-sided formula for the variable of interest,
#' with an optional grouping formula. Output includes pooled medians,
#' standard errors, min/max across implicates, and implicate-level values.
#' Point estimates are the mean of the five implicate medians. Standard errors
#' are computed using the Survey of Consumer Finances convention described
#' below, not Rubin’s Rules.
#' 
#' @section Statistical Notes:
#' Median estimates follow the Federal Reserve Board’s SCF variance convention.
#' For each implicate, the median is computed with replicate weights via
#' [survey::svyquantile()]. The pooled estimate is the average of the five
#' implicate medians. The pooled variance is
#'   V_total = V1 + ((m + 1) / m) * B,
#' where V1 is the replicate-weight sampling variance from the first implicate
#' and B is the between-implicate variance of the five implicate medians, with
#' m = 5 implicates. The reported standard error is sqrt(V_total). This matches
#' the Federal Reserve Board's published SAS macro for SCF descriptive
#' statistics and is not Rubin’s Rules.

#'
#' @param scf A `scf_mi_survey` object created by [scf_load()]. Must contain five implicates.
#' @param var A one-sided formula specifying the continuous variable of interest (e.g., `~networth`).
#' @param by Optional one-sided formula for a categorical grouping variable.
#' @param verbose Logical; if TRUE, show implicate-level results.
#'
#' @return A list of class `"scf_median"` with:
#' \describe{
#'   \item{results}{A data frame with pooled medians, standard errors, and range across implicates.}
#'   \item{imps}{A list of implicate-level results.}
#'   \item{aux}{Variable and grouping metadata.}
#' }
#'
#' @examples
#' # Do not implement these lines in real analysis:
#' # Use functions `scf_download()` and `scf_load()`
#' td <- tempfile("median_")
#' dir.create(td)
#' 
#' src <- system.file("extdata", "scf2022_mock_raw.rds", package = "scf")
#' file.copy(src, file.path(td, "scf2022.rds"), overwrite = TRUE)
#' scf2022 <- scf_load(2022, data_directory = td)
#'
#' # Example for real analysis: Estimate medians
#' scf_median(scf2022, ~networth)
#' scf_median(scf2022, ~networth, by = ~edcl)
#' 
#' # Do not implement these lines in real analysis: Cleanup for package check
#' unlink(td, recursive = TRUE, force = TRUE)
#'
#' @seealso [scf_percentile()], [scf_mean()]
#'
#' @export
scf_median <- function(scf, var, by = NULL, verbose = FALSE) {
  out <- scf_percentile(scf, var, q = 0.5, by = by, verbose = verbose)
  out$aux$quantile <- NULL
  class(out) <- c("scf_median", "scf_percentile")
  out
}


#' @export
print.scf_median <- function(x, ...) {
  dollar_label <- if (isTRUE(attr(x, "deflated")))
    sprintf(" (%d$)", attr(x, "base_year")) else ""
  cat(sprintf("Multiply-Imputed Median Estimate%s\n\n", dollar_label))
  print(x$results, row.names = FALSE, ...)
  if (isTRUE(x$verbose)) {
    cat("\nImplicate-Level Estimates:\n\n")
    imp_df <- do.call(rbind, x$imps)
    print(imp_df, row.names = FALSE)
  }
  invisible(x)
}

#' @export
summary.scf_median <- function(object, ...) {
  cat("Summary of Multiply-Imputed Median Estimate\n\n")
  cat("Pooled Estimates:\n")
  print(format(object$results, digits = 4, nsmall = 2), row.names = FALSE, ...)
  cat("\nImplicate-Level Estimates:\n")
  imp_df <- do.call(rbind, object$imps)
  print(format(imp_df, digits = 4, nsmall = 2), row.names = FALSE)
  invisible(object)
}

#' Estimate Logistic Regression Model using SCF Microdata
#'
#' @description
#' Fits a replicate-weighted logistic regression model to multiply-imputed SCF data,
#' returning pooled coefficients or odds ratios with model diagnostics. Use this
#' function to model a binary variable as a function of predictors.
#'
#' @section Details:
#' This function internally calls `scf_glm()` with `family = binomial()` and optionally
#' exponentiates pooled log-odds to odds ratios.
#'
#' Logistic regression models the probability of a binary outcome using the
#' logit link.
#'
#' Coefficients reflect the change in log-odds associated with a one-unit change
#' in the predictor. 
#'
#' When `odds = TRUE`, the coefficient estimates and standard errors are
#' transformed from log-odds to odds ratios and approximate SEs.
#'
#' @param object A `scf_mi_survey` object created with [scf_load()] and [scf_design()].
#' @param formula A model formula specifying a binary outcome and predictors, e.g., `rich ~ age + factor(edcl)`.
#' @param odds Logical. If `TRUE` (default), exponentiates coefficient estimates to produce odds ratios for interpretability.
#' @param ... Additional arguments passed to [scf_glm()].
#'
#' @return An object of class `"scf_logit"` and `"scf_model_result"` with:
#' \describe{
#'   \item{results}{A data frame of pooled estimates (log-odds or odds ratios), standard errors, and test statistics.}
#'   \item{fit}{Model diagnostics including AIC and pseudo-R-Squared (for binomial family).}
#'   \item{models}{List of implicate-level `svyglm` model objects.}
#'   \item{call}{The matched function call.}
#' }
#'
#' @examples
#' \donttest{
#' # Do not implement these lines in real analysis:
#' # Use functions `scf_download()` and `scf_load()`
#' td <- tempfile("logit_")
#' dir.create(td)
#' 
#' src <- system.file("extdata", "scf2022_mock_raw.rds", package = "scf")
#' file.copy(src, file.path(td, "scf2022.rds"), overwrite = TRUE)
#' scf2022 <- scf_load(2022, data_directory = td)
#'
#' # Example for real analysis: Run logistic regression
#' model <- suppressWarnings(scf_logit(scf2022, own ~ hhsex))
#' summary(model)
#'
#' # Do not implement these lines in real analysis: Cleanup for package check
#' unlink(td, recursive = TRUE, force = TRUE)
#' }
#' 
#' @section Warning:
#' When modeling binary outcomes using survey-weighted logistic regression,
#' users may encounter the warning:
#'
#'   `"non-integer #successes in a binomial glm!"`
#'
#' This message is benign. It results from replicate-weighted survey designs
#' where the implied number of "successes" is non-integer. The model is
#' estimated correctly. Coefficients are valid and consistent with
#' maximum likelihood.
#'
#' For background, see:
#' https://stackoverflow.com/questions/12953045/warning-non-integer-successes-in-a-binomial-glm-survey-packages
#'
#' @seealso [scf_glm()], [scf_ols()], [scf_MIcombine()]
#'
#' @export
scf_logit <- function(object, formula, odds = TRUE, ...) {
  model <- scf_glm(object, formula, family = binomial(), ...)

  if (isTRUE(attr(object, "mock"))) {
    warning("Mock data detected. Do not interpret results as valid SCF estimates.", call. = FALSE)
  }


  if (odds) {
    logit_est  <- model$results$estimate
    logit_se   <- model$results$std.error
    zval       <- logit_est / logit_se
    pval       <- 2 * pnorm(-abs(zval))
    stars      <- cut(pval,
                      breaks = c(-Inf, 0.001, 0.01, 0.05, 0.10, Inf),
                      labels = c("***", "**", "*", ".", ""),
                      right = FALSE)
    
    model$results$estimate   <- exp(logit_est)
    model$results$std.error  <- model$results$estimate * logit_se
    model$results$t.value    <- zval
    model$results$p.value    <- pval
    model$results$stars      <- stars
    attr(model$results, "scale") <- "odds"
  } else {
    attr(model$results, "scale") <- "logit"
  }

  out <- list(
    results = model$results,
    imps = model$models,
    fit = model$fit,
    vcov = model$vcov,
    call = match.call(),
    formula = formula
  )
  class(out) <- c("scf_logit", "scf_model_result")
  return(out)
}

#' @export
summary.scf_logit <- function(object, digits = 4, ...) {
  cat("SCF Logistic Regression Summary\n")
  cat("---------------------------------\n")
  scale <- attr(object$results, "scale")
  est_label <- if (!is.null(scale) && scale == "odds") "Odds Ratio" else "Log-Odds"

  cat("Outcome modeled using logit link.\n")
  cat("Estimates are on the", est_label, "scale.\n\n")

  df <- object$results
  df$estimate <- round(df$estimate, digits)
  df$std.error <- round(df$std.error, digits)
  df$t.value <- round(df$t.value, digits)
  df$p.value <- format.pval(df$p.value, digits = digits)

  cat("Coefficient Table:\n")
  print(df[, c("term", "estimate", "std.error", "t.value", "p.value", "stars")],
        row.names = FALSE)

  cat("\nModel Fit:\n")
  if (!is.null(object$fit$pseudo_r2)) {
    cat("  Pseudo R-squared: ", formatC(object$fit$pseudo_r2, digits = 3, format = "f"), "\n", sep = "")
  }
  if (!is.null(object$fit$AIC)) {
    cat("  Mean AIC: ", formatC(object$fit$AIC, digits = 0, format = "f"), "\n", sep = "")
  }

  cat("\nCall:\n")
  print(object$call)
  invisible(object)
}

#' @export
print.scf_logit <- function(x, digits = 4, ...) {
  cat("Logistic Regression Results (Multiply-Imputed SCF)\n")
  cat("--------------------------------------------------\n")

  df <- x$results
  scale <- attr(df, "scale")
  est_label <- if (!is.null(scale) && scale == "odds") "Odds Ratio" else "Log-Odds"

  num_cols <- sapply(df, is.numeric)
  df[num_cols] <- lapply(df[num_cols], round, digits)

  if (all(c("t.value", "p.value") %in% names(df))) {
    df$p.value <- format.pval(df$p.value, digits = digits)
    print(df[, c("term", "estimate", "std.error", "t.value", "p.value", "stars")], row.names = FALSE)
  } else {
    print(df[, c("term", "estimate", "std.error")], row.names = FALSE)
  }

  cat("\nModel Fit Diagnostics:\n")
  if (!is.null(x$fit$pseudo_r2)) {
    cat("  Pseudo R-squared: ", round(x$fit$pseudo_r2, digits), "\n")
  }
  if (!is.null(x$fit$AIC)) {
    cat("  Mean AIC:         ", round(x$fit$AIC, digits), "\n")
  }

  cat("\nNotes:\n")
  cat(" - Estimates are reported on the", est_label, "scale.\n")
  cat(" - Implicate-level models are stored in `object$imps`\n")
  invisible(x)
}

#' Download and Prepare SCF Microdata for Local Analysis
#'
#' Downloads SCF public-use microdata from official servers.  For each year,
#' this function retrieves five implicates, merges them with replicate weights
#' and official summary variables, and saves them as `.rds` files ready for use
#' with [scf_load()].
#'
#' @section Implementation:
#' This function downloads from official servers three types of files for each
#' year:
#' - five versions of the dataset (one per implicate), each stored as a separate data frame in a list
#' - a table of replicate weights, and
#' - a data table with official derivative variables
#'
#' These tables are collected to a list and saved to an `.rds` format file in
#' the working directory.  By default, the function downloads all available
#' years.
#'
#' @section Details:
#' The SCF employs multiply-imputed data sets to address unit-level missing
#' data. Each household appears in one of five implicates. This function ensures
#' all implicates are downloaded, merged, and prepared for downstream analysis
#' using [scf_load()], [scf_design()], and the `scf` workflow.
#'
#' @param years Integer vector of SCF years to download (e.g., `c(2016, 2019)`). Must be triennial from 1989 to 2022.
#' @param overwrite Logical. If `TRUE`, re-download and overwrite existing `.rds` files. Default is `FALSE`.
#' @param verbose Logical. If `TRUE`, display progress messages. Default is `TRUE`.
#'
#' @return A character vector of paths to the `.rds` files written to disk (one per year).
#' Each file contains a list of five implicate data frames ready for use with [scf_load()].
#'
#' @seealso [scf_load()], [scf_design()], [scf_update()]
#' 
#' @examples
#' if (FALSE) {
#'   # Download and prepare SCF data for 2022
#'   td <- tempfile("download_")
#'   dir.create(td)
#' 
#'   old <- getwd()
#'   setwd(td)
#'   scf_download(2022)
#'
#'   # Load into a survey design object
#'   scf2022 <- scf_load(2022, data_directory = td)
#'
#'   # Cleanup for package check
#'   unlink(td, recursive = TRUE, force = TRUE)
#'   setwd(old)
#' }

#' 
#' @references
#' U.S. Federal Reserve. Codebook for 2022 Survey of Consumer Finances.
#'   https://www.federalreserve.gov/econres/scfindex.htm
#'
#' @export
scf_download <- function(years = seq(1989, 2022, 3), overwrite = FALSE, verbose = TRUE) {
  pkgs <- c("httr", "haven", "utils")
  missing <- pkgs[!vapply(pkgs, requireNamespace, FUN.VALUE = logical(1), quietly = TRUE)]
  if (length(missing)) {
     stop(
       sprintf(
             "Missing package%s: %s. Please install before using scf_download().",
             if (length(missing) > 1) "s" else "",
             paste(missing, collapse = ", ")
           ),
         call. = FALSE
       )
    }

  years <- intersect(years, seq(1989, 2022, 3))
  output_files <- character()

  url_map <- list(
    `1989` = list(
      main = "https://www.federalreserve.gov/econresdata/scf/files/scfp1989s.zip",
      summary = "https://www.federalreserve.gov/econresdata/scf/files/scf89s.zip",
      rw = "https://www.federalreserve.gov/econresdata/scf/files/scf89rw1s.zip"
    ),
    `1992` = list(
      main = "https://www.federalreserve.gov/econresdata/scf/files/scfp1992s.zip",
      summary = "https://www.federalreserve.gov/econresdata/scf/files/scf92s.zip",
      rw = "https://www.federalreserve.gov/econresdata/scf/files/scf92rw1s.zip"
    ),
    `1995` = list(
      main = "https://www.federalreserve.gov/econresdata/scf/files/scfp1995s.zip",
      summary = "https://www.federalreserve.gov/econresdata/scf/files/scf95s.zip",
      rw = "https://www.federalreserve.gov/econresdata/scf/files/scf95rw1s.zip"
    ),
    `1998` = list(
      main = "https://www.federalreserve.gov/econresdata/scf/files/scfp1998s.zip",
      summary = "https://www.federalreserve.gov/econresdata/scf/files/scf98s.zip",
      rw = "https://www.federalreserve.gov/econresdata/scf/files/scf98rw1s.zip"
    ),
    `2001` = list(
      main = "https://www.federalreserve.gov/econres/files/scf01s.zip",
      summary = "https://www.federalreserve.gov/econres/files/scfp2001s.zip",
      rw = "https://www.federalreserve.gov/econres/files/scf2001rw1s.zip"
    )
  )

  for (year in years) {
    if (verbose) message("Processing SCF ", year)
    out_file <- paste0("scf", year, ".rds")
    if (file.exists(out_file) && !overwrite) {
      if (verbose) message("File exists, skipping: ", out_file)
      output_files <- c(output_files, out_file)
      next
    }

    tmpdir <- tempdir()
    year_short <- sprintf("%02d", year %% 100)

    if (year <= 2001) {
      urls <- url_map[[as.character(year)]]
      zip_main <- basename(urls$main)
      zip_summary <- basename(urls$summary)
      zip_rw <- basename(urls$rw)
    } else {
      zip_main <- paste0("scfp", year, "s.zip")
      zip_summary <- paste0("scf", year, "s.zip")
      zip_rw <- paste0("scf", year, "rw1s.zip")
      urls <- list(
        main = paste0("https://www.federalreserve.gov/econres/files/", zip_main),
        summary = paste0("https://www.federalreserve.gov/econres/files/", zip_summary),
        rw = paste0("https://www.federalreserve.gov/econres/files/", zip_rw)
      )
    }

    download_and_unzip <- function(url, zip_name) {
      zip_path <- file.path(tmpdir, zip_name)
      tryCatch({
        httr::GET(url, httr::write_disk(zip_path, overwrite = TRUE))
        unzip(zip_path, exdir = tmpdir)
        unlink(zip_path)
        TRUE
      }, error = function(e) {
        stop("Download failed: ", url, "\n", conditionMessage(e))
      })
    }

    download_and_unzip(urls$main, zip_main)
    download_and_unzip(urls$summary, zip_summary)
    download_and_unzip(urls$rw, zip_rw)

    main_file <- file.path(tmpdir, paste0("rscfp", year, ".dta"))
    summary_file <- switch(as.character(year),
                           `1989` = "p89i6.dta", `1992` = "p92i4.dta", `1995` = "p95i6.dta",
                           `1998` = "p98i6.dta", `2001` = "p01i6.dta",
                           paste0("p", year_short, "i6.dta")
    )
    summary_file <- file.path(tmpdir, summary_file)

    rw_file <- if (year == 2001) "scf2001rw1s.dta" else paste0("p", year_short, "_rw1.dta")
    rw_file <- file.path(tmpdir, rw_file)

    if (!file.exists(main_file) || !file.exists(summary_file) || !file.exists(rw_file)) {
      stop("Missing one or more required .dta files for ", year)
    }

    main <- haven::read_dta(main_file)
    summary <- haven::read_dta(summary_file)
    rw <- haven::read_dta(rw_file)

    names(main) <- tolower(names(main))
    names(summary) <- tolower(names(summary))
    names(rw) <- tolower(names(rw))

    if (year == 1989) {
      names(main)[names(main) == "x1"] <- "y1"
      names(main)[names(main) == "xx1"] <- "yy1"
      names(summary)[names(summary) == "x1"] <- "y1"
      names(summary)[names(summary) == "xx1"] <- "yy1"
      names(rw)[names(rw) == "x1"] <- "y1"
      names(rw)[names(rw) == "xx1"] <- "yy1"
    }

    if ("yy1" %in% names(summary) && !"yy1" %in% names(main)) {
      main$yy1 <- summary$yy1[match(main$y1, summary$y1)]
    }

    summary <- summary[, setdiff(names(summary), "yy1")]
    data <- merge(main, summary, by = "y1")
    if (!"yy1" %in% names(data)) stop("`yy1` missing after merge")
    data$imp <- as.numeric(substr(data$y1, nchar(data$y1), nchar(data$y1)))
    implicates <- lapply(1:5, function(i) subset(data, imp == i))
    if (any(sapply(implicates, nrow) == 0)) stop("Empty implicate detected for year ", year)

    rw[is.na(rw)] <- 0
    rw$y1 <- NULL
    for (i in 1:999) {
      w <- paste0("wt1b", i)
      m <- paste0("mm", i)
      if (all(c(w, m) %in% names(rw))) {
        rw[[w]] <- rw[[w]] * rw[[m]]
      }
    }
    rw <- rw[, c("yy1", paste0("wt1b", 1:999))]
    merged <- lapply(implicates, function(df) merge(df, rw, by = "yy1"))
    saveRDS(merged, out_file)
    output_files <- c(output_files, out_file)

    unlink(c(main_file, summary_file, rw_file), force = TRUE)
    if (verbose) message("Saved: ", out_file)
  }

  return(output_files)
}

#' T-Test of Means using SCF Microdata
#'
#' @description
#' Tests whether the mean of a continuous variable differs from a specified
#' value (one-sample), or whether group means differ across a binary factor
#' (two-sample). Estimates and standard errors are computed using `svymean()`
#' within each implicate, then pooled using Rubin’s Rules.  Use this function
#' to test hypotheses about means in the SCF microdata. 
#'
#' @param design A `scf_mi_survey` object created by [scf_load()].
#' @param var A one-sided formula specifying a numeric variable (e.g., `~income`).
#' @param group Optional one-sided formula specifying a binary grouping variable (e.g., `~female`).
#' @param mu Numeric. Null hypothesis value. Default is `0`.
#' @param alternative Character. One of `"two.sided"` (default), `"less"`, or `"greater"`.
#' @param conf.level Confidence level for the confidence interval. Default is `0.95`.
#'
#' @return An object of class `scf_ttest` with:
#' \describe{
#'   \item{results}{A data frame with pooled estimate, standard error, t-statistic, degrees of freedom, p-value, and confidence interval.}
#'   \item{means}{Group-specific means (for two-sample tests only).}
#'   \item{fit}{List describing the test type, null hypothesis, confidence level, and alternative.}
#' }
#'
#' @examples
#' # Do not implement these lines in real analysis:
#' # Use functions `scf_download()` and `scf_load()`
#' td <- tempfile("ttest_")
#' dir.create(td)
#' 
#' src <- system.file("extdata", "scf2022_mock_raw.rds", package = "scf")
#' file.copy(src, file.path(td, "scf2022.rds"), overwrite = TRUE)
#' scf2022 <- scf_load(2022, data_directory = td)
#'
#' # Wrangle data for example: Derive analysis vars
#' scf2022 <- scf_update(scf2022,
#'   female = factor(hhsex, levels = 1:2, labels = c("Male","Female")),
#'   over50 = age > 50
#' )
#'
#' # Example for real analysis:  One-sample t-test
#' scf_ttest(scf2022, ~income, mu = 75000)
#'
#' # Example for real analysis:  Two-sample t-test
#' scf_ttest(scf2022, ~income, group = ~female)
#' 
#' # Do not implement these lines in real analysis: Cleanup for package check
#' unlink(td, recursive = TRUE, force = TRUE)
#'
#' @seealso [scf_prop_test()], [scf_mean()], [scf_MIcombine()]
#' @export
scf_ttest <- function(design, var, group = NULL, mu = 0,
                      alternative = c("two.sided", "less", "greater"),
                      conf.level = 0.95) {

  stopifnot(inherits(design, "scf_mi_survey"))
  stopifnot(inherits(var, "formula"))

  if (isTRUE(attr(design, "mock"))) {
    warning("Mock data detected. Do not interpret results as valid SCF estimates.", call. = FALSE)
  }

  alternative <- match.arg(alternative)
  varname <- all.vars(var)[1]

  # Check variable type
  if (!is.numeric(design$mi_design[[1]]$variables[[varname]])) {
    stop(sprintf("Variable '%s' must be numeric.", varname))
  }

  if (is.null(group)) {
    # One-sample t-test
    stats <- lapply(design$mi_design, function(d) {
      est <- survey::svymean(as.formula(paste0("~", varname)), d)
      data.frame(est = coef(est), se = survey::SE(est))
    })
    stats_df <- do.call(rbind, stats)
    m <- length(stats_df$est)
    qbar <- mean(stats_df$est)
    ubar <- mean(stats_df$se^2)
    b <- var(stats_df$est)
    tvar <- ubar + (1 + 1/m) * b
    se <- sqrt(tvar)
    df <- (m - 1) * (1 + ubar / ((1 + 1/m) * b))^2
    tval <- (qbar - mu) / se
    pval <- switch(alternative,
                   "two.sided" = 2 * pt(-abs(tval), df),
                   "less" = pt(tval, df),
                   "greater" = 1 - pt(tval, df))
    ci <- switch(alternative,
                 "two.sided" = qbar + c(-1, 1) * qt(1 - (1 - conf.level)/2, df) * se,
                 "less" = c(-Inf, qbar + qt(conf.level, df) * se),
                 "greater" = c(qbar - qt(conf.level, df) * se, Inf))
    means_df <- NULL

  } else {
    # Two-sample t-test
    stopifnot(inherits(group, "formula"))
    groupname <- all.vars(group)[1]
    levels_detected <- levels(factor(design$mi_design[[1]]$variables[[groupname]]))
    if (length(levels_detected) != 2) {
      stop("Grouping variable must have exactly two levels.")
    }

    stats <- lapply(design$mi_design, function(d) {
      d$variables[[groupname]] <- factor(d$variables[[groupname]], levels = levels_detected)
      d1 <- subset(d, d$variables[[groupname]] == levels_detected[1])
      d2 <- subset(d, d$variables[[groupname]] == levels_detected[2])
      est1 <- survey::svymean(as.formula(paste0("~", varname)), d1)
      est2 <- survey::svymean(as.formula(paste0("~", varname)), d2)
      data.frame(
        diff = coef(est1) - coef(est2),
        se = sqrt(survey::SE(est1)^2 + survey::SE(est2)^2),
        m1 = coef(est1),
        m2 = coef(est2)
      )
    })
    stats_df <- do.call(rbind, stats)
    m <- nrow(stats_df)
    qbar <- mean(stats_df$diff)
    ubar <- mean(stats_df$se^2)
    b <- var(stats_df$diff)
    tvar <- ubar + (1 + 1/m) * b
    se <- sqrt(tvar)
    df <- (m - 1) * (1 + ubar / ((1 + 1/m) * b))^2
    tval <- (qbar - mu) / se
    pval <- switch(alternative,
                   "two.sided" = 2 * pt(-abs(tval), df),
                   "less" = pt(tval, df),
                   "greater" = 1 - pt(tval, df))
    ci <- switch(alternative,
                 "two.sided" = qbar + c(-1, 1) * qt(1 - (1 - conf.level)/2, df) * se,
                 "less" = c(-Inf, qbar + qt(conf.level, df) * se),
                 "greater" = c(qbar - qt(conf.level, df) * se, Inf))
    means_df <- data.frame(
      group = levels_detected,
      mean = c(mean(stats_df$m1), mean(stats_df$m2))
    )
  }

  results <- data.frame(
    estimate = qbar,
    std.error = se,
    t.value = tval,
    df = df,
    p.value = pval,
    conf.low = ci[1],
    conf.high = ci[2],
    stars = cut(pval,
                breaks = c(-Inf, 0.001, 0.01, 0.05, 0.10, Inf),
                labels = c("***", "**", "*", "^", ""), right = FALSE)
  )

  structure(list(
    results = results,
    means = means_df,
    fit = list(
      method = if (is.null(group)) "One-sample t-test" else "Two-sample t-test",
      null.value = mu,
      conf.level = conf.level,
      alternative = alternative
    ),
    aux = list(year = design$year)
  ), class = "scf_ttest")
}

#' @export
print.scf_ttest <- function(x, digits = 4, ...) {
  dollar_label <- if (isTRUE(attr(x, "deflated")))
    sprintf(" (%d$)", attr(x, "base_year")) else ""
  cat(sprintf("SCF %s%s\n", x$fit$method, dollar_label))
  cat("Alternative hypothesis:",
      switch(x$fit$alternative,
             "two.sided" = "mean is not equal to",
             "less" = "mean is less than",
             "greater" = "mean is greater than"),
      x$fit$null.value, "\n\n")

  if (!is.null(x$means)) {
    cat("Group means:\n")
    rounded_means <- x$means
    num_cols <- sapply(rounded_means, is.numeric)
    rounded_means[num_cols] <- round(rounded_means[num_cols], digits)
    print(rounded_means, row.names = FALSE)
    cat("\n")
  }

  cat(sprintf("Estimate: %.2f", x$results$estimate), "\n")
  cat(sprintf("Standard Error: %.2f", x$results$std.error), "\n")
  cat(sprintf("t = %.2f, df = %.1f, p = %.4f %s",
              x$results$t.value, x$results$df, x$results$p.value, x$results$stars), "\n")
  cat(sprintf("CI (%.0f%%): [%.2f, %.2f]",
              100 * x$fit$conf.level,
              x$results$conf.low,
              x$results$conf.high), "\n")
  invisible(x)
}


#' @export
summary.scf_ttest <- function(object, ...) {
  cat("SCF", object$fit$method, "\n")
  cat("Null hypothesis: mu =", object$fit$null.value, "\n")
  cat("Alternative hypothesis:", object$fit$alternative, "\n")
  cat(sprintf("Confidence level: %.0f%%\n\n", 100 * object$fit$conf.level))

  if (!is.null(object$means)) {
    cat("Group-specific means:\n")
    print(round(object$means, 2))
    cat("\n")
  }

  cat("Test results:\n")
  out <- object$results
  out$p.value <- format.pval(out$p.value, digits = 4, eps = .0001)
  out$estimate <- round(out$estimate, 2)
  out$std.error <- round(out$std.error, 2)
  out$t.value <- round(out$t.value, 2)
  out$conf.low <- round(out$conf.low, 2)
  out$conf.high <- round(out$conf.high, 2)
  print(out[, c("estimate", "std.error", "t.value", "df", "p.value", "conf.low", "conf.high", "stars")],
        row.names = FALSE)
  invisible(object)
}

# CPI-U-RS September values (x100, no decimal) for each SCF survey year.
# Source: Federal Reserve Board SCF Bulletin SAS macro (bulletin.macro.txt).
# See: https://www.bls.gov/cpi/research-series/r-cpi-u-rs-home.htm
.scf_cpi_urs <- c(
  `1989` = 1898, `1992` = 2112, `1995` = 2261, `1998` = 2400,
  `2001` = 2614, `2004` = 2785, `2007` = 3058, `2010` = 3204,
  `2013` = 3438, `2016` = 3548, `2019` = 3775, `2022` = 4376
)

.scf_deflation_factor <- function(survey_year, base_year) {
  sy <- as.character(survey_year)
  by <- as.character(base_year)
  valid <- names(.scf_cpi_urs)
  if (!sy %in% valid)
    stop(sprintf(
      "Survey year %s not in CPI table. Valid SCF years: %s.",
      sy, paste(valid, collapse = ", ")
    ), call. = FALSE)
  if (!by %in% valid)
    stop(sprintf(
      "base_year %s not in CPI table. Valid SCF years: %s.",
      by, paste(valid, collapse = ", ")
    ), call. = FALSE)
  unname(.scf_cpi_urs[by] / .scf_cpi_urs[sy])
}


#' Convert SCF Dollar Estimates to Real Terms
#'
#' Multiplies dollar-valued point estimates and their standard errors by the
#' CPI-U-RS deflation factor \code{CPI[base_year] / CPI[survey_year]},
#' converting nominal survey-year dollars to real dollars of the chosen base
#' year. The CPI-U-RS September values are taken directly from the Federal
#' Reserve Board's SCF Bulletin SAS macro.
#'
#' @details
#' Standard errors rescale correctly under linear multiplication, so both
#' estimates and SEs are multiplied by the same factor. Confidence intervals
#' and group means are also rescaled. The t-statistic, degrees of freedom,
#' and p-value in \code{scf_ttest} results are invariant to this rescaling
#' and are left unchanged.
#'
#' \strong{Supported functions:} \code{scf_mean}, \code{scf_median},
#' \code{scf_percentile}, and \code{scf_ttest} return dollar-valued estimates
#' that transform correctly under scalar multiplication.
#' \code{scf_freq}, \code{scf_xtab}, and \code{scf_prop_test} return
#' proportions and are not supported. \code{scf_corr} returns a dimensionless
#' coefficient and is not supported. The regression functions
#' (\code{scf_ols}, \code{scf_glm}, \code{scf_logit}, \code{scf_quantreg})
#' are not supported because deflating coefficients post hoc is ambiguous
#' when a model mixes dollar and non-dollar variables; for real-dollar
#' regression results, deflate variables upstream with \code{scf_update()}
#' before fitting.
#'
#' \strong{Income note:} SCF income is measured in the prior calendar year,
#' so a mean income estimate from the 2019 survey is in 2018 dollars, not
#' 2019 dollars. \code{scf_deflate()} applies \code{CPI[base] / CPI[2019]},
#' which is a close but not exact conversion for income. This is documented
#' in the Federal Reserve's SAS macro, which applies a separate lag
#' adjustment to income before the main deflation step.
#'
#' @section Conditions:
#' \describe{
#'   \item{Warning — double deflation}{If the input object has already been
#'     deflated (\code{attr(x, "deflated")} is \code{TRUE}), \code{scf_deflate()}
#'     issues a warning and proceeds. Applying the function twice compounds the
#'     CPI adjustment and produces incorrect results. Check
#'     \code{attr(result, "deflated")} and \code{attr(result, "base_year")}
#'     before calling.}
#'   \item{Error — stale result object}{If the result object does not carry a
#'     survey year (\code{x$aux$year} is \code{NULL}), the function stops with
#'     a message asking you to re-run the originating function (\code{scf_mean()},
#'     \code{scf_median()}, etc.) under the current package version. This field
#'     was introduced in version 1.0.6.}
#'   \item{Error — unsupported class}{Passing an object of an unsupported class
#'     (e.g., a regression result or a frequency table) stops with a message
#'     listing the supported classes: \code{scf_mean}, \code{scf_median},
#'     \code{scf_percentile}, and \code{scf_ttest}.}
#'   \item{Error — year not in CPI table}{If \code{base_year} is not one of
#'     the valid triennial SCF survey years (1989--2022), the function stops
#'     and lists the valid options.}
#' }
#'
#' @param x An object of class \code{scf_mean}, \code{scf_median},
#'   \code{scf_percentile}, or \code{scf_ttest}.
#' @param base_year Integer. Reference year for real dollars. Must be a valid
#'   SCF survey year (1989--2022, triennial). Default is \code{2022}.
#'
#' @return The input object with dollar estimates, standard errors, and
#'   confidence intervals rescaled to \code{base_year} dollars. Attributes
#'   \code{"deflated"} (logical) and \code{"base_year"} (integer) are set on
#'   the returned object.
#'
#' @examples
#' # Do not implement these lines in real analysis:
#' # Use functions `scf_download()` and `scf_load()`
#' td <- tempfile("deflate_")
#' dir.create(td)
#'
#' src <- system.file("extdata", "scf2022_mock_raw.rds", package = "scf")
#' file.copy(src, file.path(td, "scf2022.rds"), overwrite = TRUE)
#' scf2022 <- scf_load(2022, data_directory = td)
#'
#' # Deflate a mean estimate to real 2022 dollars
#' result <- scf_mean(scf2022, ~networth)
#' result_real <- scf_deflate(result, base_year = 2022)
#'
#' # Works with median and percentile results
#' med <- scf_median(scf2022, ~networth)
#' med_real <- scf_deflate(med, base_year = 2022)
#'
#' # Do not implement these lines in real analysis: Cleanup for package check
#' unlink(td, recursive = TRUE, force = TRUE)
#'
#' @seealso [scf_mean()], [scf_median()], [scf_percentile()], [scf_ttest()],
#'   [scf_update()]
#'
#' @export
scf_deflate <- function(x, base_year = 2022) {
  UseMethod("scf_deflate")
}

#' @export
scf_deflate.default <- function(x, base_year = 2022) {
  stop(sprintf(
    paste0(
      "scf_deflate() does not support objects of class '%s'.\n",
      "Supported classes: scf_mean, scf_median, scf_percentile, scf_ttest."
    ),
    paste(class(x), collapse = "', '")
  ), call. = FALSE)
}

#' @export
scf_deflate.scf_mean <- function(x, base_year = 2022) {
  if (isTRUE(attr(x, "deflated")))
    warning(
      "This object has already been deflated. ",
      "Applying scf_deflate() again will compound the adjustment.",
      call. = FALSE
    )
  survey_year <- x$aux$year
  if (is.null(survey_year))
    stop(
      "Survey year not found in result object. ",
      "Re-run scf_mean() with the current package version.",
      call. = FALSE
    )
  f <- .scf_deflation_factor(survey_year, base_year)

  for (col in c("estimate", "se", "min", "max")) {
    if (col %in% names(x$results))
      x$results[[col]] <- x$results[[col]] * f
  }
  x$imps <- lapply(x$imps, function(df) {
    for (col in c("estimate", "se"))
      if (col %in% names(df)) df[[col]] <- df[[col]] * f
    df
  })
  attr(x, "deflated")  <- TRUE
  attr(x, "base_year") <- as.integer(base_year)
  x
}

#' @export
scf_deflate.scf_percentile <- function(x, base_year = 2022) {
  if (isTRUE(attr(x, "deflated")))
    warning(
      "This object has already been deflated. ",
      "Applying scf_deflate() again will compound the adjustment.",
      call. = FALSE
    )
  survey_year <- x$aux$year
  if (is.null(survey_year))
    stop(
      "Survey year not found in result object. ",
      "Re-run scf_percentile() with the current package version.",
      call. = FALSE
    )
  f <- .scf_deflation_factor(survey_year, base_year)

  for (col in c("estimate", "se", "min", "max")) {
    if (col %in% names(x$results))
      x$results[[col]] <- x$results[[col]] * f
  }
  x$imps <- lapply(x$imps, function(df) {
    for (col in c("estimate", "se"))
      if (col %in% names(df)) df[[col]] <- df[[col]] * f
    df
  })
  attr(x, "deflated")  <- TRUE
  attr(x, "base_year") <- as.integer(base_year)
  x
}

#' @export
scf_deflate.scf_median <- function(x, base_year = 2022) {
  scf_deflate.scf_percentile(x, base_year = base_year)
}

#' @export
scf_deflate.scf_ttest <- function(x, base_year = 2022) {
  if (isTRUE(attr(x, "deflated")))
    warning(
      "This object has already been deflated. ",
      "Applying scf_deflate() again will compound the adjustment.",
      call. = FALSE
    )
  survey_year <- x$aux$year
  if (is.null(survey_year))
    stop(
      "Survey year not found in result object. ",
      "Re-run scf_ttest() with the current package version.",
      call. = FALSE
    )
  f <- .scf_deflation_factor(survey_year, base_year)

  # Scale estimate, SE, and confidence bounds; t, df, p are invariant
  for (col in c("estimate", "std.error", "conf.low", "conf.high")) {
    if (col %in% names(x$results))
      x$results[[col]] <- x$results[[col]] * f
  }
  # Scale group means (two-sample case)
  if (!is.null(x$means) && "mean" %in% names(x$means))
    x$means$mean <- x$means$mean * f
  # Scale the null hypothesis value
  x$fit$null.value <- x$fit$null.value * f

  attr(x, "deflated")  <- TRUE
  attr(x, "base_year") <- as.integer(base_year)
  x
}

#' Internal Import Declarations
#'
#' Declares functions from base packages used in nonstandard evaluation or
#' dynamic contexts across `scf` package functions. Ensures all used base
#' functions are properly registered in the NAMESPACE.
#'
#' @keywords internal
#' @importFrom stats var pnorm qnorm qt setNames binomial family xtabs aggregate IQR weighted.mean
#' @importFrom utils head tail unzip write.csv
#' @importFrom rlang .data
#' @importFrom survey SE cv
#' @import ggplot2
#' @name scf_imports
NULL

# .wts is the replicate-weight vector injected by survey::withReplicates()
# into user-supplied functions. Declared here to suppress R CMD check NOTE.
utils::globalVariables(".wts")

---
title: "Analyzing the Survey of Consumer Finances"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Analyzing the Survey of Consumer Finances}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}

knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  message = FALSE,
  warning = FALSE
)

library(dplyr)
library(scf)

```

# Introduction

The **Survey of Consumer Finances (SCF)** is a triennial survey of U.S. household finances conducted by the Federal Reserve Board. It is among the most detailed and methodologically sophisticated data sources on U.S. households' personal finances. 

To ensure valid estimation and inference, the SCF incorporates two key methodological features:

1. **Complex Survey Design**: The SCF uses a dual frame design with a geographic national sample and list sample of wealthy people selected from IRS records. Each implicate includes 999 replicate weights constructed via balanced repeated replication (BRR), which enable design-consistent estimation of variance.
2. **Multiple Imputation**: The SCF addresses item nonresponse through multiple imputation. Each release includes five implicates—plausible, complete versions of the dataset with different imputed values for missing items.

These design features demand appropriate statistical handling. Analysts unfamiliar with replicate weighting and imputation pooling may inadvertently produce biased or misleading results. In practice, these barriers have discouraged even quantitatively competent researchers from working directly with SCF microdata.

The `scf` package aims to reduce this friction. It provides a structured and reproducible R interface for downloading, transforming, and analyzing SCF data using methods appropriate to its design. The package handles replicate weights and Rubin’s Rules transparently and consistently across descriptive statistics, hypothesis testing, regression modeling, and visualization.

This vignette introduces the core analytic workflow supported by the package. For detailed methodological discussion, see Cohen (2025a).


# Workflow

## 1. Downloading and Loading the Data

Download raw SCF data and load it into a valid multiply-imputed survey object using `scf_download()` and `scf_load()`. The result is an `scf_mi_survey` object that contains replicate-weighted survey designs for each implicate.

```{r, include = F}
# Using Mock data with distribution
vtd <- file.path(tempdir(), "scf_vig")
dir.create(vtd, showWarnings = FALSE)

src <- system.file("extdata", "scf2022_mock_raw.rds", package = "scf")
file.copy(src, file.path(vtd, "scf2022.rds"), overwrite = TRUE)
scf2022 <- scf_load(2022, data_directory = vtd)
```

```{r, eval = F}
scf2022 <- scf_download(2022)
scf2022 <- scf_load(scf2022)

```

## 2. Creating and Transforming Variables

Before logging, you must bottom-code income and net worth at \$1 to avoid NA values due to log(0). The `scf_update()` function safely adds or modifies variables across all implicates.

```{r}

scf2022 <- scf_update(scf2022,
  senior = age >= 65,
  female = factor(hhsex, levels = 1:2, labels = c("Male", "Female")),
  rich = networth > 1e6,
  networth = ifelse(networth > 1, networth, 1),
  log_networth = log(networth),
  income = ifelse(income > 1, income, 1),
  log_income = log(income),
  npeople = x101
)

```

Use `names(scf2022$mi_design[[1]]$variables)` to inspect variables.

## 3. Univariate and Bivariate Distributions

Use `scf_mean()`, `scf_median()`, and `scf_percentile()` to calculate pooled estimates with Rubin’s Rules. Use `by =` for grouped statistics.

```{r}
scf_mean(scf2022, ~networth, by = ~senior)
scf_median(scf2022, ~income, by = ~female)
scf_percentile(scf2022, ~networth, q = 0.9)
scf_percentile(scf2022, ~networth, q = 0.75, by = ~female)
```

## 4. Hypothesis Tests

Conduct t-tests and proportion tests on pooled SCF data. These tests return interpretable outputs with correct degrees of freedom and pooled standard errors.

```{r}
scf_ttest(scf2022, ~networth, mu = 250000)
scf_ttest(scf2022, ~networth, group = ~senior)
scf_prop_test(scf2022, ~senior, p = 0.25)
scf_prop_test(scf2022, ~rich, ~female)
```

## 5. Regression Modeling

Fit linear or generalized linear models with Rubin-aware pooling. Logistic models can return odds ratios if requested.

```{r}
scf_ols(scf2022, networth ~ age + log_income)
scf_logit(scf2022, rich ~ age + log_income)
scf_logit(scf2022, rich ~ age + log_income, odds = TRUE)
scf_glm(scf2022, own ~ age , family = binomial())
```

> **Note on Warnings**  
> When running logistic regression with `scf_logit()` or other functions that use `family = binomial()`, you may see warnings like:  
>  
> ```
> `Warning: non-integer #successes in a binomial glm!`
> ```
>  
> This warning is harmless. It appears because `survey::svyglm()` uses replicate weights that can lead to fractional counts. The model still estimates correctly. For more background and discussion, see [Stack Overflow thread](https://stackoverflow.com/questions/12953045/warning-non-integer-successes-in-a-binomial-glm-survey-packages).


## 6. Visualization

Produce publication-quality plots using multiply-imputed data. All visuals account for weights and imputations.

```{r}
scf_plot_dbar(scf2022, ~senior)
scf_plot_bbar(scf2022, ~female, ~rich, scale = "percent")
scf_plot_cbar(scf2022, ~networth, ~edcl, stat = "median")
scf_plot_dist(scf2022, ~age, bins = 10) 
scf_plot_smooth(scf2022, ~age)
scf_plot_hex(scf2022, ~income, ~networth)
```

## 7. Inspecting Implicates and Pooled Objects

Use `scf_implicates()` to inspect individual implicate estimates for sensitivity analysis.

```{r}
freq_table <- scf_freq(scf2022, ~rich)
scf_implicates(freq_table, long = TRUE)
```


# Learn More
For more details on the SCF methodology and the `scf` package, see:

- Federal Reserve Board. (2023). *Survey of Consumer Finances*. [https://www.federalreserve.gov/econres/aboutscf.htm](https://www.federalreserve.gov/econres/aboutscf.htm)

```{r, include = F}
# Cleanup to avoid NOTE about leftover files
try(unlink(file.path(vtd, "scf2022.rds"), force=TRUE), silent=TRUE)
try(unlink(vtd, recursive=TRUE, force=TRUE), silent=TRUE)
rm(vtd)
```

---
title: "Analyzing the Survey of Consumer Finances"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Analyzing the Survey of Consumer Finances}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  message = FALSE,
  warning = FALSE
)
library(scf)
```

# Introduction

The **Survey of Consumer Finances (SCF)** is a triennial survey of U.S. household finances conducted by the Federal Reserve Board. It is among the most detailed data sources on U.S. household wealth, income, and financial behavior.

Valid estimation from the SCF requires handling two design features:

1. **Complex survey design**: The SCF uses a dual-frame design with 999 replicate weights per implicate, constructed via balanced repeated replication, which enable design-consistent variance estimation.
2. **Multiple imputation**: The SCF addresses item nonresponse through multiple imputation. Each release includes five implicates — complete, plausible versions of the dataset with different imputed values for missing items.

The `scf` package handles both. It wraps the five implicates in replicate-weighted `survey::svrepdesign()` objects, applies estimation routines across implicates, and pools results using Rubin's Rules or equivalent procedures.

This vignette demonstrates the core workflow. For methodological background, see the package paper.


# Workflow

## 1. Downloading and Loading the Data

Download SCF data and load it into a multiply-imputed survey object using `scf_download()` and `scf_load()`.

```{r, include = FALSE}
vtd <- file.path(tempdir(), "scf_vig")
dir.create(vtd, showWarnings = FALSE)
src <- system.file("extdata", "scf2022_mock_raw.rds", package = "scf")
file.copy(src, file.path(vtd, "scf2022.rds"), overwrite = TRUE)
scf2022 <- scf_load(2022, data_directory = vtd)
```

```{r, eval = FALSE}
scf_download(2022)
scf2022 <- scf_load(2022)
```

The result is a `scf_mi_survey` object containing five `svyrep.design` objects, one per implicate, and survey-year metadata.

## 2. Creating and Transforming Variables

`scf_update()` adds or modifies variables uniformly across all five implicates. Bottom-code skewed variables before logging to avoid `log(0)`.

```{r}
scf2022 <- scf_update(scf2022,
  senior       = age >= 65,
  female       = factor(hhsex, levels = 1:2, labels = c("Male", "Female")),
  rich         = networth > 1e6,
  networth     = ifelse(networth > 1, networth, 1),
  log_networth = log(networth),
  income       = ifelse(income > 1, income, 1),
  log_income   = log(income),
  npeople      = x101
)
```

Use `names(scf2022$mi_design[[1]]$variables)` to inspect available variables.

When a transformation depends on the distribution within each implicate — ranks, percentile flags, groupwise z-scores — use `scf_update_by_implicate()` instead:

```{r, eval = FALSE}
scf2022 <- scf_update_by_implicate(scf2022, function(df) {
  df$wealth_rank <- rank(df$networth) / nrow(df)
  df
})
```

## 3. Univariate and Bivariate Distributions

`scf_mean()`, `scf_median()`, and `scf_percentile()` return pooled population estimates with standard errors. The `by` argument produces group-level estimates.

```{r}
scf_mean(scf2022, ~networth, by = ~senior)
scf_median(scf2022, ~income, by = ~female)
scf_percentile(scf2022, ~networth, q = 0.9)
scf_percentile(scf2022, ~networth, q = 0.75, by = ~female)
```

Discrete distributions are summarized with `scf_freq()` and cross-tabulated with `scf_xtab()`.

```{r}
scf_freq(scf2022, ~senior)
scf_xtab(scf2022, ~senior, ~female, scale = "col")
```

## 4. Hypothesis Tests

`scf_ttest()` and `scf_prop_test()` produce pooled inferential tests with correct degrees of freedom.

```{r}
scf_ttest(scf2022, ~networth, mu = 250000)
scf_ttest(scf2022, ~networth, group = ~senior)
scf_prop_test(scf2022, ~senior, p = 0.25)
scf_prop_test(scf2022, ~rich, ~female)
```

## 5. Regression Modeling

`scf_ols()`, `scf_glm()`, and `scf_logit()` fit replicate-weighted models to each implicate and pool coefficients via Rubin's Rules.

```{r}
scf_ols(scf2022, log_networth ~ age + log_income)
scf_logit(scf2022, rich ~ age + log_income, odds = TRUE)
scf_glm(scf2022, own ~ age, family = binomial())
```

`scf_regtable()` formats results from one or more models for publication:

```{r}
m1 <- scf_ols(scf2022, log_networth ~ age)
m2 <- scf_ols(scf2022, log_networth ~ age + log_income)
scf_regtable(m1, m2, model.names = c("Model 1", "Model 2"), digits = 3)
```

## 6. Quantile Regression

OLS models the conditional mean, which is sensitive to outliers and skew. Wealth and income distributions in the SCF are highly right-skewed: a small number of high-wealth households can dominate mean estimates. Quantile regression estimates the conditional quantile of the outcome at a user-specified probability `tau`, giving a more complete picture of distributional associations.

`scf_quantreg()` fits `quantreg::rq()` with SCF final sampling weights to each implicate, then pools coefficients and variance-covariance matrices across implicates via `scf_MIcombine()`.

```{r}
# Median regression
m_med <- scf_quantreg(scf2022, log_networth ~ age + senior, tau = 0.50)
print(m_med)
```

```{r}
# 75th percentile
m_75 <- scf_quantreg(scf2022, log_networth ~ age + senior, tau = 0.75)
summary(m_75)
```

The `se` argument controls within-implicate variance estimation. All methods feed a covariance matrix into `scf_MIcombine()`, so between-implicate (imputation) variance is always incorporated.

- `"nid"` (default): non-iid sandwich estimator. Allows the conditional sparsity — the density of the error distribution at the quantile — to vary across observations. Appropriate when the error distribution is not constant across the covariate space, which is typical for wealth and income outcomes.
- `"iid"`: implements the Koenker-Bassett (1978) covariance formula `[θ(1-θ)/f(ξ(θ))²] Q⁻¹`, where `f(ξ(θ))` is the density of the error at the θ-quantile (the sparsity) and `Q = lim T⁻¹X'X`. This assumes the error distribution is identical across all observations — constant sparsity, not merely constant variance. Fastest option.
- `"ker"`: kernel density estimate of the conditional sparsity. More data-adaptive than `"nid"`.
- `"boot"`: pairs bootstrap. Distribution-free but slowest of the analytical options.
- `"replicate"`: re-fits the model using each of the SCF's 999 replicate weight vectors per implicate via `survey::withReplicates()`, accumulating variance as the weighted sum of squared deviations from the full-weight estimate. This matches the SCF's own published variance methodology and is recommended for final publication-quality estimates. It requires approximately 5,000 model fits across five implicates and is computationally intensive.

```{r, eval = FALSE}
# Replication-based variance (recommended for publication; slow)
m_rep <- scf_quantreg(scf2022, log_networth ~ age + senior,
                       tau = 0.50, se = "replicate")
summary(m_rep)
```

A note on design: the `survey` package provides `svyquantile()` for estimating marginal quantiles of a single variable, but it has no function for regression quantiles (conditional quantiles given covariates). `scf_quantreg()` therefore uses `quantreg::rq()` for point estimation, with `survey::withReplicates()` providing the design-based variance wrapper for the `"replicate"` path. This keeps the SCF's replication scheme encapsulated in the survey design object, rather than reimplementing it manually.

Koenker and Bassett (1978) established the asymptotic theory for the unweighted, i.i.d. case. Applying `rq()` with survey sampling weights extends the estimator to probability-weighted samples, which is standard empirical practice; the `"iid"` and `"nid"` standard error formulas extend correspondingly to the weighted design matrix `X'WX`. The `"replicate"` option sidesteps this by using the SCF's own replication scheme to estimate variance directly, without relying on distributional assumptions about the errors.

Results are compatible with `scf_regtable()`:

```{r}
scf_regtable(m_med, m_75,
             model.names = c("Median", "75th Pct"),
             digits = 3)
```

To study how associations vary across the outcome distribution, estimate the same model at multiple quantiles and compare:

```{r, eval = FALSE}
taus <- c(0.25, 0.50, 0.75, 0.90)
models <- lapply(taus, function(t) {
  scf_quantreg(scf2022, log_networth ~ age + senior, tau = t)
})
names(models) <- paste0("tau=", taus)
do.call(scf_regtable, c(models, list(digits = 3)))
```

## 7. Visualization

`scf` plotting functions account for survey weights and multiply-imputed data.

```{r}
scf_plot_dbar(scf2022, ~senior)
scf_plot_bbar(scf2022, ~female, ~rich, scale = "percent")
scf_plot_cbar(scf2022, ~networth, ~edcl, stat = "median")
scf_plot_dist(scf2022, ~age, bins = 10)
scf_plot_smooth(scf2022, ~age)
scf_plot_hex(scf2022, ~income, ~networth)
```

## 8. Inspecting Implicates

Use `scf_implicates()` to retrieve implicate-level estimates from any `scf_*` result for sensitivity analysis or custom pooling.

```{r}
freq_table <- scf_freq(scf2022, ~rich)
scf_implicates(freq_table, long = TRUE)
```

Implicate-level regression model objects are stored directly in the result:

```{r, eval = FALSE}
m <- scf_ols(scf2022, log_networth ~ age)
summary(m$imps[[1]])   # first implicate svyglm object
```


# Learn More

- Federal Reserve Board. *Survey of Consumer Finances*. <https://www.federalreserve.gov/econres/aboutscf.htm>
- Koenker R, Bassett G. Regression quantiles. *Econometrica*. 1978;46(1):33–50.
- Rubin DB. *Multiple Imputation for Nonresponse in Surveys*. Wiley; 1987.

```{r, include = FALSE}
try(unlink(file.path(vtd, "scf2022.rds"), force = TRUE), silent = TRUE)
try(unlink(vtd, recursive = TRUE, force = TRUE), silent = TRUE)
rm(vtd)
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scf_design.R
\name{scf_design}
\alias{scf_design}
\title{Construct an SCF Multiply-Imputed Survey Object}
\usage{
scf_design(design, year, n_households)
}
\arguments{
\item{design}{A list of five \code{\link[survey:svrepdesign]{survey::svrepdesign()}} objects (one per implicate).}

\item{year}{Numeric SCF survey year (e.g., 2022).}

\item{n_households}{Numeric total U.S. households represented in \code{year}.}
}
\value{
An object of class "scf_mi_survey" with:
\describe{
\item{mi_design}{List of replicate-weighted designs (one per implicate).}
\item{year}{SCF survey year.}
\item{n_households}{Estimated number of U.S. households.}
}
}
\description{
Wraps a list of replicate-weighted survey designs into an \code{scf_mi_survey}
object. This is called internally by \code{\link[=scf_load]{scf_load()}}, but is also available
directly for users who construct their own implicate-level designs outside
the standard download-and-load workflow — for example, when integrating
external or custom-prepared SCF data files.

Each element of \code{design} must be a \code{\link[survey:svrepdesign]{survey::svrepdesign()}} object representing
one SCF implicate with replicate weights.
}
\examples{
# Do not implement these lines in real analysis:
# Use functions `scf_download()` and `scf_load()`
td <- tempfile("design_")
dir.create(td)

src <- system.file("extdata", "scf2022_mock_raw.rds", package = "scf")
file.copy(src, file.path(td, "scf2022.rds"), overwrite = TRUE)
scf2022 <- scf_load(2022, data_directory = td)

# Example for real analysis: Construct scf_mi_survey object
obj <- scf_design(
  design = scf2022$mi_design,
  year = 2022,
  n_households = attr(scf2022, "n_households")
)
class(obj)
length(obj$mi_design)

# Do not implement these lines in real analysis: Cleanup for package check
unlink(td, recursive = TRUE, force = TRUE)

}
\seealso{
\code{\link[=scf_load]{scf_load()}}, \code{\link[=scf_update]{scf_update()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scf_download.R
\name{scf_download}
\alias{scf_download}
\title{Download and Prepare SCF Microdata for Local Analysis}
\usage{
scf_download(years = seq(1989, 2022, 3), overwrite = FALSE, verbose = TRUE)
}
\arguments{
\item{years}{Integer vector of SCF years to download (e.g., \code{c(2016, 2019)}). Must be triennial from 1989 to 2022.}

\item{overwrite}{Logical. If \code{TRUE}, re-download and overwrite existing \code{.rds} files. Default is \code{FALSE}.}

\item{verbose}{Logical. If \code{TRUE}, display progress messages. Default is \code{TRUE}.}
}
\value{
A character vector of paths to the \code{.rds} files written to disk (one per year).
Each file contains a list of five implicate data frames ready for use with \code{\link[=scf_load]{scf_load()}}.
}
\description{
Downloads SCF public-use microdata from official servers.  For each year,
this function retrieves five implicates, merges them with replicate weights
and official summary variables, and saves them as \code{.rds} files ready for use
with \code{\link[=scf_load]{scf_load()}}.
}
\section{Implementation}{

This function downloads from official servers three types of files for each
year:
\itemize{
\item five versions of the dataset (one per implicate), each stored as a separate data frame in a list
\item a table of replicate weights, and
\item a data table with official derivative variables
}

These tables are collected to a list and saved to an \code{.rds} format file in
the working directory.  By default, the function downloads all available
years.
}

\section{Details}{

The SCF employs multiply-imputed data sets to address unit-level missing
data. Each household appears in one of five implicates. This function ensures
all implicates are downloaded, merged, and prepared for downstream analysis
using \code{\link[=scf_load]{scf_load()}}, \code{\link[=scf_design]{scf_design()}}, and the \code{scf} workflow.
}

\examples{
if (FALSE) {
  # Download and prepare SCF data for 2022
  td <- tempfile("download_")
  dir.create(td)

  old <- getwd()
  setwd(td)
  scf_download(2022)

  # Load into a survey design object
  scf2022 <- scf_load(2022, data_directory = td)

  # Cleanup for package check
  unlink(td, recursive = TRUE, force = TRUE)
  setwd(old)
}

}
\references{
U.S. Federal Reserve. Codebook for 2022 Survey of Consumer Finances.
https://www.federalreserve.gov/econres/scfindex.htm
}
\seealso{
\code{\link[=scf_load]{scf_load()}}, \code{\link[=scf_design]{scf_design()}}, \code{\link[=scf_update]{scf_update()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scf_ttest.R
\name{scf_ttest}
\alias{scf_ttest}
\title{T-Test of Means using SCF Microdata}
\usage{
scf_ttest(
  design,
  var,
  group = NULL,
  mu = 0,
  alternative = c("two.sided", "less", "greater"),
  conf.level = 0.95
)
}
\arguments{
\item{design}{A \code{scf_mi_survey} object created by \code{\link[=scf_load]{scf_load()}}.}

\item{var}{A one-sided formula specifying a numeric variable (e.g., \code{~income}).}

\item{group}{Optional one-sided formula specifying a binary grouping variable (e.g., \code{~female}).}

\item{mu}{Numeric. Null hypothesis value. Default is \code{0}.}

\item{alternative}{Character. One of \code{"two.sided"} (default), \code{"less"}, or \code{"greater"}.}

\item{conf.level}{Confidence level for the confidence interval. Default is \code{0.95}.}
}
\value{
An object of class \code{scf_ttest} with:
\describe{
\item{results}{A data frame with pooled estimate, standard error, t-statistic, degrees of freedom, p-value, and confidence interval.}
\item{means}{Group-specific means (for two-sample tests only).}
\item{fit}{List describing the test type, null hypothesis, confidence level, and alternative.}
}
}
\description{
Tests whether the mean of a continuous variable differs from a specified
value (one-sample), or whether group means differ across a binary factor
(two-sample). Estimates and standard errors are computed using \code{svymean()}
within each implicate, then pooled using Rubin’s Rules.  Use this function
to test hypotheses about means in the SCF microdata.
}
\examples{
# Do not implement these lines in real analysis:
# Use functions `scf_download()` and `scf_load()`
td <- tempfile("ttest_")
dir.create(td)

src <- system.file("extdata", "scf2022_mock_raw.rds", package = "scf")
file.copy(src, file.path(td, "scf2022.rds"), overwrite = TRUE)
scf2022 <- scf_load(2022, data_directory = td)

# Wrangle data for example: Derive analysis vars
scf2022 <- scf_update(scf2022,
  female = factor(hhsex, levels = 1:2, labels = c("Male","Female")),
  over50 = age > 50
)

# Example for real analysis:  One-sample t-test
scf_ttest(scf2022, ~income, mu = 75000)

# Example for real analysis:  Two-sample t-test
scf_ttest(scf2022, ~income, group = ~female)

# Do not implement these lines in real analysis: Cleanup for package check
unlink(td, recursive = TRUE, force = TRUE)

}
\seealso{
\code{\link[=scf_prop_test]{scf_prop_test()}}, \code{\link[=scf_mean]{scf_mean()}}, \code{\link[=scf_MIcombine]{scf_MIcombine()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scf_activate_theme.R, R/scf_theme.R
\name{scf_activate_theme}
\alias{scf_activate_theme}
\alias{scf_theme}
\title{Default Plot Theme for SCF Visualizations}
\usage{
scf_activate_theme()

scf_theme(base_size = 13, base_family = "sans", grid = TRUE, axis = TRUE, ...)
}
\arguments{
\item{base_size}{Base font size. Defaults to 13.}

\item{base_family}{Font family. Defaults to "sans".}

\item{grid}{Logical. Show gridlines? Defaults to TRUE.}

\item{axis}{Logical. Include axis ticks and lines? Defaults to TRUE.}

\item{...}{Additional arguments passed to \code{\link[ggplot2:ggtheme]{ggplot2::theme_minimal()}}.}
}
\value{
\code{scf_theme()} returns a \code{ggplot2} theme object.
\code{scf_activate_theme()} returns \code{NULL} invisibly; called for
its side effect of setting the session-wide \code{ggplot2} theme.
}
\description{
The theme is designed to:
\itemize{
\item Render cleanly in \strong{print} (single-column or wrapped layout)
\item Scale well on \strong{HD desktop} monitors without visual clutter
\item Remain \strong{legible on mobile} with clear fonts and sufficient contrast
}

The default figure dimensions assumed for export are \strong{5.5 inches by 5.5 inches}
at \strong{300 dpi}, which balances compactness with accessibility across media.

All theme settings are exposed via comments to enable easy brand customization.
}
\details{
Defines the SCF package's default \code{ggplot2} theme, optimized for legibility,
clarity, and aesthetic coherence across print, desktop, and mobile platforms.
}
\section{Global activation}{

\code{scf_activate_theme()} sets \code{scf_theme()} as the default
\code{ggplot2} theme for all plots in the current R session — equivalent
to \code{ggplot2::theme_set(scf_theme())}. The effect lasts only for the
session and does not persist across sessions. All \code{scf_plot_*()} functions
apply \code{scf_theme()} internally, so this is most useful when producing
custom plots alongside the package's built-in visualizations.
}

\examples{
library(ggplot2)
ggplot(mtcars, aes(factor(cyl))) +
  geom_bar(fill = "#0072B2") +
  scf_theme()

# Activate globally for the session:
scf_activate_theme()

}
\seealso{
\code{\link[ggplot2:theme]{ggplot2::theme()}}, \code{\link[=scf_plot_dist]{scf_plot_dist()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scf_plot_hist.R
\name{scf_plot_hist}
\alias{scf_plot_hist}
\title{Histogram of a Continuous Variable in Multiply-Imputed SCF Data}
\usage{
scf_plot_hist(
  design,
  variable,
  bins = 30,
  xlim = NULL,
  title = NULL,
  xlab = NULL,
  ylab = "Weighted Count",
  fill = "#0072B2"
)
}
\arguments{
\item{design}{A \code{scf_mi_survey} object from \code{\link[=scf_load]{scf_load()}}.}

\item{variable}{A one-sided formula indicating the numeric variable to plot.}

\item{bins}{Number of bins (default: 30).}

\item{xlim}{Optional numeric range. Values outside will be included in edge bins.}

\item{title}{Optional plot title.}

\item{xlab}{Optional x-axis label. Defaults to the variable name.}

\item{ylab}{Optional y-axis label. Defaults to "Weighted Count".}

\item{fill}{Fill color for bars (default: \code{"#0072B2"}).}
}
\value{
A \code{ggplot2} object representing the Rubin-pooled histogram.
}
\description{
Produces a histogram of a continuous SCF variable by binning across implicates,
pooling weighted bin counts using \code{\link[=scf_freq]{scf_freq()}}, and plotting the result.
Values outside \code{xlim} are clamped into the nearest endpoint to ensure all
observations are included and replicate-weighted bins remain stable.
}
\section{Implementation}{

This function bins a continuous variable (after clamping to \code{xlim} if supplied),
applies the same \code{cut()} breaks across implicates using \code{\link[=scf_update_by_implicate]{scf_update_by_implicate()}},
and computes Rubin-pooled frequencies with \code{\link[=scf_freq]{scf_freq()}}. Results are filtered to
remove bins with undefined proportions and then plotted using \code{ggplot2::geom_col()}.

The logic here is specific to operations where the bin assignment must be computed
\strong{within} each implicate, not after pooling. This approach ensures consistent binning
and stable pooled estimation in the presence of multiply-imputed microdata.
}

\examples{
# Do not implement these lines in real analysis:
# Use functions `scf_download()` and `scf_load()`
td <- tempfile("plot_hist_")
dir.create(td)

src <- system.file("extdata", "scf2022_mock_raw.rds", package = "scf")
file.copy(src, file.path(td, "scf2022.rds"), overwrite = TRUE)
scf2022 <- scf_load(2022, data_directory = td)

# Example for real analysis: Plot histogram of age
scf_plot_hist(scf2022, ~age, bins = 10)

# Do not implement these lines in real analysis: Cleanup for package check
unlink(td, recursive = TRUE, force = TRUE)

}
\seealso{
\code{\link[=scf_freq]{scf_freq()}}, \code{\link[=scf_plot_dbar]{scf_plot_dbar()}}, \code{\link[=scf_plot_smooth]{scf_plot_smooth()}}, \code{\link[=scf_update_by_implicate]{scf_update_by_implicate()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scf_median.R
\name{scf_median}
\alias{scf_median}
\title{Estimate the Population Median of a Continuous SCF Variable}
\usage{
scf_median(scf, var, by = NULL, verbose = FALSE)
}
\arguments{
\item{scf}{A \code{scf_mi_survey} object created by \code{\link[=scf_load]{scf_load()}}. Must contain five implicates.}

\item{var}{A one-sided formula specifying the continuous variable of interest (e.g., \code{~networth}).}

\item{by}{Optional one-sided formula for a categorical grouping variable.}

\item{verbose}{Logical; if TRUE, show implicate-level results.}
}
\value{
A list of class \code{"scf_median"} with:
\describe{
\item{results}{A data frame with pooled medians, standard errors, and range across implicates.}
\item{imps}{A list of implicate-level results.}
\item{aux}{Variable and grouping metadata.}
}
}
\description{
Estimates the median (50th percentile) of a continuous SCF variable. Use this
operation to characterize a typical or average value.  In contrast to
\code{\link[=scf_mean]{scf_mean()}}, this function is both uninfluenced by, and insensitive to,
outliers.
}
\section{Implementation}{

This function wraps \code{\link[=scf_percentile]{scf_percentile()}} with \code{q = 0.5}. The user supplies a
\code{scf_mi_survey} object and a one-sided formula for the variable of interest,
with an optional grouping formula. Output includes pooled medians,
standard errors, min/max across implicates, and implicate-level values.
Point estimates are the mean of the five implicate medians. Standard errors
are computed using the Survey of Consumer Finances convention described
below, not Rubin’s Rules.
}

\section{Statistical Notes}{

Median estimates follow the Federal Reserve Board’s SCF variance convention.
For each implicate, the median is computed with replicate weights via
\code{\link[survey:svyquantile]{survey::svyquantile()}}. The pooled estimate is the average of the five
implicate medians. The pooled variance is
V_total = V1 + ((m + 1) / m) * B,
where V1 is the replicate-weight sampling variance from the first implicate
and B is the between-implicate variance of the five implicate medians, with
m = 5 implicates. The reported standard error is sqrt(V_total). This matches
the Federal Reserve Board's published SAS macro for SCF descriptive
statistics and is not Rubin’s Rules.
}

\examples{
# Do not implement these lines in real analysis:
# Use functions `scf_download()` and `scf_load()`
td <- tempfile("median_")
dir.create(td)

src <- system.file("extdata", "scf2022_mock_raw.rds", package = "scf")
file.copy(src, file.path(td, "scf2022.rds"), overwrite = TRUE)
scf2022 <- scf_load(2022, data_directory = td)

# Example for real analysis: Estimate medians
scf_median(scf2022, ~networth)
scf_median(scf2022, ~networth, by = ~edcl)

# Do not implement these lines in real analysis: Cleanup for package check
unlink(td, recursive = TRUE, force = TRUE)

}
\seealso{
\code{\link[=scf_percentile]{scf_percentile()}}, \code{\link[=scf_mean]{scf_mean()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scf_imports.R
\name{scf_imports}
\alias{scf_imports}
\title{Internal Import Declarations}
\description{
Declares functions from base packages used in nonstandard evaluation or
dynamic contexts across \code{scf} package functions. Ensures all used base
functions are properly registered in the NAMESPACE.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scf_percentile.R
\name{scf_percentile}
\alias{scf_percentile}
\title{Estimate Percentiles in SCF Microdata}
\usage{
scf_percentile(
  scf,
  var,
  q = 0.5,
  by = NULL,
  verbose = FALSE,
  method = c("implicate", "stack")
)
}
\arguments{
\item{scf}{A \code{scf_mi_survey} object created with \code{\link[=scf_load]{scf_load()}}. Must
contain the list of replicate-weighted designs for each implicate in
\code{scf$mi_design}.}

\item{var}{A one-sided formula naming the continuous variable to
summarize (for example \code{~networth}).}

\item{q}{Numeric percentile in between 0 and 1. Default 0.5 (median).}

\item{by}{Optional one-sided formula naming a categorical grouping
variable. If supplied, the percentile is estimated separately within
each group.}

\item{verbose}{Logical. If TRUE, include implicate-level estimates in
the returned object for inspection. Default FALSE.}

\item{method}{Character. One of \code{"implicate"} (default) or
\code{"stack"}. The implicate method averages percentile estimates
across the five implicates using the SCF Bulletin variance convention.
The stack method pools all five implicates into a single dataset with
weights divided by five and computes a single weighted quantile,
replicating the Federal Reserve's published figures. When
\code{method = "stack"} no standard error is returned, as the stacked
estimate is a deterministic weighted quantile rather than a
model-based estimate.}
}
\value{
An object of class \code{"scf_percentile"} containing:
\describe{
\item{results}{A data frame containing pooled percentile estimates, pooled
standard errors, and implicate min/max values. One row per group (if
\code{by} is supplied) or one row otherwise.}
\item{imps}{A list of implicate-level percentile estimates and standard errors.}
\item{aux}{A list containing the variable name, optional group variable name,
and the quantile requested.}
\item{verbose}{Logical flag indicating whether implicate-level estimates
should be printed by \code{print()} or \code{summary()}.}
}
}
\description{
This function estimates a weighted percentile of a continuous variable
in the Survey of Consumer Finances (SCF). Two methods are available.
The default \code{"implicate"} method estimates the percentile separately
within each implicate using \code{\link[survey:svyquantile]{survey::svyquantile()}} and pools the results
following the SCF Bulletin variance convention. The \code{"stack"} method
replicates the Federal Reserve's published convention by pooling all five
implicates with weights divided by five and computing a single weighted
quantile from the combined sample. The two methods will generally produce
similar but not identical results. Use \code{method = "stack"} when exact
replication of Federal Reserve published figures is required.
}
\details{
The implicate method computes estimates as follows:
\enumerate{
\item For each implicate, estimate the requested percentile using
\code{\link[survey:svyquantile]{survey::svyquantile()}} with \code{se = TRUE}.
\item The reported point estimate is the mean of the M implicate-specific
percentile estimates.
\item The standard error follows the SCF Bulletin SAS macro convention:

\if{html}{\out{<div class="sourceCode">}}\preformatted{V_total = V1 + ((M + 1) / M) * B
}\if{html}{\out{</div>}}

where:
\itemize{
\item V1 is the replicate-weight sampling variance of the percentile
from the first implicate only.
\item B  is the between-implicate variance of the percentile estimates.
}

The reported standard error is sqrt(V_total).
\item If a grouping variable is supplied, the same logic is applied
separately within each group.
}

The stack method returns no standard error, as the estimate is a
deterministic weighted quantile rather than a model-based estimate.
}
\examples{
# Do not implement these lines in real analysis:
# Use functions `scf_download()` and `scf_load()` for actual SCF data
td <- tempfile("percentile_")
dir.create(td)

src <- system.file("extdata", "scf2022_mock_raw.rds", package = "scf")
file.copy(src, file.path(td, "scf2022.rds"), overwrite = TRUE)
scf2022 <- scf_load(2022, data_directory = td)

# Estimate the 75th percentile of net worth
scf_percentile(scf2022, ~networth, q = 0.75)

# Estimate the median net worth by ownership group
scf_percentile(scf2022, ~networth, q = 0.5, by = ~own)

# Do not implement these lines in real analysis: Cleanup for package check
unlink(td, recursive = TRUE, force = TRUE)
rm(scf2022)

}
\references{
Federal Reserve Board. 2023c. "SAS Macro: Variable Definitions."
https://www.federalreserve.gov/econres/files/bulletin.macro.txt
}
\seealso{
\code{\link[=scf_median]{scf_median()}}, \code{\link[=scf_mean]{scf_mean()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scf_deflate.R
\name{scf_deflate}
\alias{scf_deflate}
\title{Convert SCF Dollar Estimates to Real Terms}
\usage{
scf_deflate(x, base_year = 2022)
}
\arguments{
\item{x}{An object of class \code{scf_mean}, \code{scf_median},
\code{scf_percentile}, or \code{scf_ttest}.}

\item{base_year}{Integer. Reference year for real dollars. Must be a valid
SCF survey year (1989--2022, triennial). Default is \code{2022}.}
}
\value{
The input object with dollar estimates, standard errors, and
confidence intervals rescaled to \code{base_year} dollars. Attributes
\code{"deflated"} (logical) and \code{"base_year"} (integer) are set on
the returned object.
}
\description{
Multiplies dollar-valued point estimates and their standard errors by the
CPI-U-RS deflation factor \code{CPI[base_year] / CPI[survey_year]},
converting nominal survey-year dollars to real dollars of the chosen base
year. The CPI-U-RS September values are taken directly from the Federal
Reserve Board's SCF Bulletin SAS macro.
}
\details{
Standard errors rescale correctly under linear multiplication, so both
estimates and SEs are multiplied by the same factor. Confidence intervals
and group means are also rescaled. The t-statistic, degrees of freedom,
and p-value in \code{scf_ttest} results are invariant to this rescaling
and are left unchanged.

\strong{Supported functions:} \code{scf_mean}, \code{scf_median},
\code{scf_percentile}, and \code{scf_ttest} return dollar-valued estimates
that transform correctly under scalar multiplication.
\code{scf_freq}, \code{scf_xtab}, and \code{scf_prop_test} return
proportions and are not supported. \code{scf_corr} returns a dimensionless
coefficient and is not supported. The regression functions
(\code{scf_ols}, \code{scf_glm}, \code{scf_logit}, \code{scf_quantreg})
are not supported because deflating coefficients post hoc is ambiguous
when a model mixes dollar and non-dollar variables; for real-dollar
regression results, deflate variables upstream with \code{scf_update()}
before fitting.

\strong{Income note:} SCF income is measured in the prior calendar year,
so a mean income estimate from the 2019 survey is in 2018 dollars, not
2019 dollars. \code{scf_deflate()} applies \code{CPI[base] / CPI[2019]},
which is a close but not exact conversion for income. This is documented
in the Federal Reserve's SAS macro, which applies a separate lag
adjustment to income before the main deflation step.
}
\section{Conditions}{

\describe{
\item{Warning — double deflation}{If the input object has already been
deflated (\code{attr(x, "deflated")} is \code{TRUE}), \code{scf_deflate()}
issues a warning and proceeds. Applying the function twice compounds the
CPI adjustment and produces incorrect results. Check
\code{attr(result, "deflated")} and \code{attr(result, "base_year")}
before calling.}
\item{Error — stale result object}{If the result object does not carry a
survey year (\code{x$aux$year} is \code{NULL}), the function stops with
a message asking you to re-run the originating function (\code{scf_mean()},
\code{scf_median()}, etc.) under the current package version. This field
was introduced in version 1.0.6.}
\item{Error — unsupported class}{Passing an object of an unsupported class
(e.g., a regression result or a frequency table) stops with a message
listing the supported classes: \code{scf_mean}, \code{scf_median},
\code{scf_percentile}, and \code{scf_ttest}.}
\item{Error — year not in CPI table}{If \code{base_year} is not one of
the valid triennial SCF survey years (1989--2022), the function stops
and lists the valid options.}
}
}

\examples{
# Do not implement these lines in real analysis:
# Use functions `scf_download()` and `scf_load()`
td <- tempfile("deflate_")
dir.create(td)

src <- system.file("extdata", "scf2022_mock_raw.rds", package = "scf")
file.copy(src, file.path(td, "scf2022.rds"), overwrite = TRUE)
scf2022 <- scf_load(2022, data_directory = td)

# Deflate a mean estimate to real 2022 dollars
result <- scf_mean(scf2022, ~networth)
result_real <- scf_deflate(result, base_year = 2022)

# Works with median and percentile results
med <- scf_median(scf2022, ~networth)
med_real <- scf_deflate(med, base_year = 2022)

# Do not implement these lines in real analysis: Cleanup for package check
unlink(td, recursive = TRUE, force = TRUE)

}
\seealso{
\code{\link[=scf_mean]{scf_mean()}}, \code{\link[=scf_median]{scf_median()}}, \code{\link[=scf_percentile]{scf_percentile()}}, \code{\link[=scf_ttest]{scf_ttest()}},
\code{\link[=scf_update]{scf_update()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scf_freq.R
\name{scf_freq}
\alias{scf_freq}
\title{Estimate the Frequencies of a Discrete Variable from SCF Microdata}
\usage{
scf_freq(scf, var, by = NULL, percent = TRUE)
}
\arguments{
\item{scf}{A \code{scf_mi_survey} object created by \code{\link[=scf_load]{scf_load()}}. Must contain five replicate-weighted implicates.}

\item{var}{A one-sided formula specifying a categorical variable (e.g., \code{~racecl}).}

\item{by}{Optional one-sided formula specifying a discrete grouping variable (e.g., \code{~own}).}

\item{percent}{Logical. If \code{TRUE} (default), scales results and standard errors to percentages.}
}
\value{
A list of class \code{"scf_freq"} with:
\describe{
\item{results}{Pooled category proportions and standard errors, by group if specified.}
\item{imps}{A named list of implicate-level proportion estimates.}
\item{aux}{Metadata about the variable and grouping structure.}
}
}
\description{
This function estimates the relative frequency (proportion) of each category
in a discrete variable from the SCF public-use microdata.  Use this function
to discern the univariate distribution of a discrete variable.
}
\details{
Computes weighted proportions and standard errors for a discrete variable
in multiply-imputed SCF data, optionally stratified by a grouping variable.
Proportions and standard errors are computed separately within each
implicate using \code{svymean()}, then averaged across
implicates using SCF-recommended pooling logic. Group-wise frequencies are
supported, but users may find the features of \code{\link[=scf_xtab]{scf_xtab()}} to be more useful.
}
\section{Details}{

Proportions are estimated within each implicate using \code{survey::svymean()},
then pooled using the standard MI formula for proportions. When a grouping
variable is provided via \code{by}, estimates are produced separately for each
group-category combination. Results may be scaled to percentages using the
\code{percent} argument.

Estimates are pooled using the standard formula:
\itemize{
\item The mean of implicate-level proportions is the point estimate
\item The standard error reflects both within-implicate variance and across-implicate variation
}

Unlike means or model parameters, category proportions do not use Rubin's full combination rules (e.g., degrees of freedom).
}

\examples{
# Do not implement these lines in real analysis:
# Use functions `scf_download()` and `scf_load()`
td <- tempfile("freq_")
dir.create(td)

src <- system.file("extdata", "scf2022_mock_raw.rds", package = "scf")
file.copy(src, file.path(td, "scf2022.rds"), overwrite = TRUE)
scf2022 <- scf_load(2022, data_directory = td)

# Example for real analysis: Proportions of homeownership
scf_freq(scf2022, ~own)

# Example for real analysis: Homeownership proportions by education
scf_freq(scf2022, ~own, by = ~edcl)

# Do not implement these lines in real analysis: Cleanup for package check
unlink(td, recursive = TRUE, force = TRUE)

}
\seealso{
\code{\link[=scf_xtab]{scf_xtab()}}, \code{\link[=scf_plot_dist]{scf_plot_dist()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scf_corr.R
\name{scf_corr}
\alias{scf_corr}
\alias{summary.scf_corr}
\title{Estimate Correlation Between Two Continuous Variables in SCF Microdata}
\usage{
scf_corr(scf, var1, var2)

\method{summary}{scf_corr}(object, ...)
}
\arguments{
\item{scf}{An \code{scf_mi_survey} object, created by \code{\link[=scf_load]{scf_load()}}}

\item{var1}{One-sided formula specifying the first variable}

\item{var2}{One-sided formula specifying the second variable}

\item{object}{A \code{scf_corr} object returned by \code{\link[=scf_corr]{scf_corr()}}.}

\item{...}{Currently unused; included for S3 generic compatibility.}
}
\value{
An object of class \code{scf_corr}, containing:
\describe{
\item{results}{Data frame with pooled correlation estimate, standard error,
t-statistic, degrees of freedom, p-value, and minimum/maximum values across implicates.}
\item{imps}{Named vector of implicate-level correlations.}
\item{aux}{Variable names used in the estimation.}
}
}
\description{
This function estimates the linear association between two continuous variables
using Pearson's correlation. Estimates are computed within each implicate and then
pooled across implicates to account for imputation uncertainty.
}
\details{
Computes the Pearson correlation coefficient between two continuous variables using
multiply-imputed, replicate-weighted SCF data. Returns pooled estimates and standard errors
using Rubin’s Rules.
}
\note{
Degrees of freedom are approximated using a simplified Barnard–Rubin
adjustment, since correlation is a scalar quantity. Interpret cautiously with
few implicates.
}
\section{Implementation}{

\itemize{
\item Inputs: an \code{scf_mi_survey} object and two one-sided formulas (e.g., \code{~income})
\item Correlation computed using \code{cor(..., use = "complete.obs")} within each implicate
\item Rubin’s Rules applied to pool results across implicates
}
}

\section{Interpretation}{

Pearson’s \verb{$r$} ranges from -1 to +1 and reflects the strength and
direction of a linear bivariate association between two continuous variables.
Values near 0 indicate weak linear association. Note that the operation is
sensitive to outliers and does not capture nonlinear relationships nor adjust
for covariates.
}

\section{Statistical Notes}{

Correlation is computed within each implicate using complete cases. Rubin’s
Rules are applied manually to pool estimates and calculate total variance.
This function does not use \code{\link[=scf_MIcombine]{scf_MIcombine()}}, which is intended
for vector-valued estimates; direct pooling is more appropriate for
scalar statistics like correlation coefficients.
}

\examples{
# Do not implement these lines in real analysis:
# Use functions `scf_download()` and `scf_load()`
td <- tempfile("corr_")
dir.create(td)

src <- system.file("extdata", "scf2022_mock_raw.rds", package = "scf")
file.copy(src, file.path(td, "scf2022.rds"), overwrite = TRUE)
scf2022 <- scf_load(2022, data_directory = td)

# Example for real analysis: Correlate income and net worth
corr <- scf_corr(scf2022, ~income, ~networth)
print(corr)
summary(corr)

# Do not implement these lines in real analysis: Cleanup for package check
unlink(td, recursive = TRUE, force = TRUE)

}
\seealso{
\code{\link[=scf_plot_hex]{scf_plot_hex()}}, \code{\link[=scf_ols]{scf_ols()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scf_mean.R
\name{scf_mean}
\alias{scf_mean}
\title{Estimate Mean in Multiply-Imputed SCF Data}
\usage{
scf_mean(scf, var, by = NULL, verbose = FALSE)
}
\arguments{
\item{scf}{A scf_mi_survey object created with \code{\link[=scf_load]{scf_load()}}. Must contain five replicate-weighted implicates.}

\item{var}{A one-sided formula identifying the continuous variable to summarize (e.g., ~networth).}

\item{by}{Optional one-sided formula specifying a discrete grouping variable for stratified means.}

\item{verbose}{Logical. If TRUE, include implicate-level results in print output. Default is FALSE.}
}
\value{
A list of class "scf_mean" with:
\describe{
\item{results}{Pooled estimates with standard errors and range across implicates. One row per group, or one row total.}
\item{imps}{A named list of implicate-level estimates.}
\item{aux}{Variable and group metadata.}
}
}
\description{
Returns the population-level estimate of a continuous variable's weighted
mean across the Survey's five implicates. Use this operation to derive an
estimate of a population's 'typical' or 'average' score on a continuous
variable.
}
\section{Details}{

The mean is a measure of central tendency that represents the arithmetic average of a distribution.
It is most appropriate when the distribution is symmetric and not heavily skewed.
Unlike the median, the mean is sensitive to extreme values, which may distort interpretation in the presence of outliers.
Use this function to describe the “typical” value of a continuous variable in the population or within subgroups.
}

\examples{
# Do not implement these lines in real analysis:
# Use functions `scf_download()` and `scf_load()`
td <- tempfile("mean_")
dir.create(td)

src <- system.file("extdata", "scf2022_mock_raw.rds", package = "scf")
file.copy(src, file.path(td, "scf2022.rds"), overwrite = TRUE)
scf2022 <- scf_load(2022, data_directory = td)

# Example for real analysis: Estimate means
scf_mean(scf2022, ~networth)
scf_mean(scf2022, ~networth, by = ~edcl)

# Do not implement these lines in real analysis: Cleanup for package check
unlink(td, recursive = TRUE, force = TRUE)


}
\seealso{
\code{\link[=scf_median]{scf_median()}}, \code{\link[=scf_percentile]{scf_percentile()}}, \code{\link[=scf_xtab]{scf_xtab()}}, \code{\link[=scf_plot_dist]{scf_plot_dist()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scf_update_by_implicate.R
\name{scf_update_by_implicate}
\alias{scf_update_by_implicate}
\title{Modify Each Implicate Individually in SCF Data}
\usage{
scf_update_by_implicate(object, f)
}
\arguments{
\item{object}{A \code{scf_mi_survey} object from \code{\link[=scf_load]{scf_load()}}.}

\item{f}{A function that takes one implicate's data frame as its sole argument
and returns a modified data frame with the same number of rows. The function
signature should be \code{function(df) \{ ... \}}. The returned data frame is
used to rebuild the replicate-weighted survey design.}
}
\value{
A modified \code{scf_mi_survey} object with updated implicate-level designs.
}
\description{
Each household in SCF data is represented by five \emph{implicates}, which reflect
uncertainty from the imputation process. Most transformations — such as computing
log income or assigning categorical bins — can be applied uniformly across implicates
using \code{\link[=scf_update]{scf_update()}}. However, some operations depend on the \emph{internal distribution}
of variables within each implicate. For those, you need to modify each one separately.

This function extracts each implicate from the replicate-weighted survey design,
applies your transformation, and rebuilds the survey design objects accordingly.
}
\details{
Applies a user-defined transformation to each implicate's data frame separately.
This is useful when you need to compute values that depend on the distribution
within each implicate — such as ranks, percentiles, or groupwise comparisons —
which cannot be computed reliably using \code{\link[=scf_update]{scf_update()}}.
}
\section{Use this When}{

\itemize{
\item You need implicate-specific quantiles (e.g., flag households in the top 10\% of wealth)
\item You want to assign percentile ranks (e.g., income percentile by implicate)
\item You are computing statistics within groups (e.g., groupwise z-scores)
\item You need to derive a variable based on implicate-specific thresholds or bins
}
}

\examples{
# Do not implement these lines in real analysis:
# Use functions `scf_download()` and `scf_load()`
td <- tempfile("update_by_implicate_")
dir.create(td)
src <- system.file("extdata", "scf2022_mock_raw.rds", package = "scf")
file.copy(src, file.path(td, "scf2022.rds"), overwrite = TRUE)
scf2022 <- scf_load(2022, data_directory = td)

# Example for real analysis: flag households in the top 10\% of net worth,
# using the unweighted implicate-specific 90th percentile as the threshold.
scf2022 <- scf_update_by_implicate(scf2022, function(df) {
  threshold <- stats::quantile(df$networth, probs = 0.90, na.rm = TRUE)
  df$top10nw <- df$networth >= threshold
  df
})

# Example for real analysis: compute implicate-specific z-scores of income
scf2022 <- scf_update_by_implicate(scf2022, function(df) {
  mu <- mean(df$income, na.rm = TRUE)
  sigma <- stats::sd(df$income, na.rm = TRUE)
  df$z_income <- (df$income - mu) / sigma
  df
})

# Verify new variable exists
head(scf2022$mi_design[[1]]$variables$z_income)

# Do not implement these lines in real analysis: Cleanup for package check
unlink(td, recursive = TRUE, force = TRUE)

}
\seealso{
\code{\link[=scf_update]{scf_update()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scf_MIcombine.R
\name{scf_MIcombine}
\alias{scf_MIcombine}
\alias{SE}
\alias{SE.scf_MIresult}
\title{Combine Estimates Across SCF Implicates Using Rubin's Rules}
\usage{
scf_MIcombine(results, variances, call = sys.call(), df.complete = Inf)

SE(object, ...)

\method{SE}{scf_MIresult}(object, ...)
}
\arguments{
\item{results}{A list of implicate-level model outputs. Each element must be a named numeric vector
or an object with methods for \code{coef()} and \code{vcov()}. Typically generated internally by modeling functions.}

\item{variances}{Optional list of variance-covariance matrices. If omitted, extracted using \code{vcov()}.}

\item{call}{Optional. The originating function call. Defaults to \code{sys.call()}.}

\item{df.complete}{Optional degrees of freedom for the complete-data model. Used for small-sample
corrections. Defaults to \code{Inf}, assuming large-sample asymptotics.}

\item{object}{A pooled result object of class \code{"scf_MIresult"} (for \code{SE()}).}

\item{...}{Not used.}
}
\value{
An object of class \code{"scf_MIresult"} with components:
\describe{
\item{coefficients}{Pooled point estimates across implicates.}
\item{variance}{Pooled variance-covariance matrix.}
\item{df}{Degrees of freedom for each parameter, adjusted using Barnard-Rubin formula.}
\item{missinfo}{Estimated fraction of missing information for each parameter.}
\item{nimp}{Number of implicates used in pooling.}
\item{call}{Function call recorded for reproducibility.}
}

Supports \code{coef()}, \code{\link[=SE]{SE()}}, \code{confint()}, and \code{summary()} methods.
}
\description{
This function implements \strong{Rubin’s Rules} for combining multiply-imputed
survey model results in the \code{scf} package. It pools point estimates,
variance-covariance matrices, and degrees of freedom across the SCF’s
five implicates.
}
\section{Scope}{

\code{scf_MIcombine()} is used for \strong{model-based analyses} such as
\code{\link[=scf_ols]{scf_ols()}}, \code{\link[=scf_glm]{scf_glm()}}, and \code{\link[=scf_logit]{scf_logit()}}, where each implicate’s model
output includes both parameter estimates and replicate-weighted sampling
variances.

Descriptive estimators—functions such as \code{\link[=scf_mean]{scf_mean()}}, \code{\link[=scf_percentile]{scf_percentile()}},
and \code{\link[=scf_median]{scf_median()}}—do \strong{not} apply Rubin’s Rules. They follow the Survey of
Consumer Finances convention used in the Federal Reserve Board’s SAS macro,
combining (i) the replicate-weight sampling variance from implicate 1 with
(ii) the between-implicate variance scaled by (m + 1)/m.

This separation is intentional: descriptive statistics in \code{scf} aim to
reproduce the Survey of Consumer Finances' published standard errors,
whereas model-based functions use Rubin's Rules.
}

\section{Implementation}{

\code{scf_MIcombine()} pools a set of implicate-level estimates and their
associated variance-covariance matrices using Rubin’s Rules.

This includes:
\itemize{
\item Calculation of pooled point estimates
\item Total variance from within- and between-imputation components
\item Degrees of freedom via Barnard-Rubin method
\item Fraction of missing information
}

Inputs are typically produced by modeling functions such as \code{\link[=scf_ols]{scf_ols()}},
\code{\link[=scf_glm]{scf_glm()}}, or \code{\link[=scf_logit]{scf_logit()}}, which return implicate-level coefficient
vectors and variance-covariance matrices.

This function is primarily used internally, but may be called directly by
advanced users constructing custom estimation routines from implicate-level
results.
}

\section{Details}{

The SCF provides five implicates per survey wave, each a plausible version
of the population under a specific missing-data model. Analysts conduct the
same statistical procedure on each implicate, producing a set of five
estimates \eqn{ Q_1, Q_2, ..., Q_5 }. These are then combined using Rubin’s
Rules, a procedure to combine results across these implicates with an
attempt to account for:
\itemize{
\item \strong{Within-imputation variance}: Uncertainty from complex sample design
\item \strong{Between-imputation variance}: Uncertainty due to missing data
}

For a scalar quantity \eqn{ Q }, the pooled estimate and
total variance are calculated as:

\deqn{ \bar{Q} = \frac{1}{M} \sum Q_m }
\deqn{ \bar{U} = \frac{1}{M} \sum U_m }
\deqn{ B = \frac{1}{M - 1} \sum (Q_m - \bar{Q})^2 }
\deqn{ T = \bar{U} + \left(1 + \frac{1}{M} \right) B }

Where:
\itemize{
\item \eqn{ M } is the number of implicates (typically 5 for SCF)
\item \eqn{ Q_m } is the estimate from implicate \eqn{ m }
\item \eqn{ U_m } is the sampling variance of \eqn{ Q_m }, accounting for replicate weights and design
}

The total variance \eqn{ T } reflects both within-imputation uncertainty (sampling error)
and between-imputation uncertainty (missing-data imputation).

The standard error of the pooled estimate is \eqn{ \sqrt{T} }. Degrees of freedom are
adjusted using the Barnard-Rubin method:

\deqn{ \nu = (M - 1) \left(1 + \frac{\bar{U}}{(1 + \frac{1}{M}) B} \right)^2 }

The fraction of missing information (FMI) is also reported:
it reflects the proportion of total variance attributable to imputation uncertainty.
}

\examples{
# Do not implement these lines in real analysis:
# Use functions `scf_download()` and `scf_load()`
td <- tempfile("MIcombine_")
dir.create(td)

src <- system.file("extdata", "scf2022_mock_raw.rds", package = "scf")
file.copy(src, file.path(td, "scf2022.rds"), overwrite = TRUE)
scf2022 <- scf_load(2022, data_directory = td)

# Example for real analysis: Pool simple survey mean for mock data
outlist <- lapply(scf2022$mi_design, function(d) survey::svymean(~I(age >= 65), d))
pooled  <- scf_MIcombine(outlist)     # vcov/coef extracted automatically
SE(pooled); coef(pooled)

unlink(td, recursive = TRUE, force = TRUE)

}
\references{
Barnard J, Rubin DB. Small-sample degrees of freedom with multiple imputation.
\doi{10.1093/biomet/86.4.948}.

Little RJA, Rubin DB. Statistical analysis with missing data.
ISBN: 9780470526798.

U.S. Federal Reserve. Codebook for 2022 Survey of Consumer Finances.
https://www.federalreserve.gov/econres/scfindex.htm
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scf_pctile_sum.R
\name{scf_pctile_sum}
\alias{scf_pctile_sum}
\alias{scf_pctile_cut}
\title{Summarize SCF Variables by Percentile Group}
\usage{
scf_pctile_sum(
  scf,
  var,
  probs = seq(0, 1, by = 0.1),
  labels = NULL,
  varname = NULL,
  method = c("implicate", "stack"),
  stat = c("mean", "median", "none"),
  stat_var = NULL
)

scf_pctile_cut(
  scf,
  var,
  probs = seq(0, 1, by = 0.1),
  labels = NULL,
  varname = NULL,
  method = c("implicate", "stack"),
  stat = c("mean", "median", "none"),
  stat_var = NULL
)
}
\arguments{
\item{scf}{A \code{scf_mi_survey} object created with \code{\link{scf_load}}.}

\item{var}{A one-sided formula naming the continuous variable to cut
(e.g., \code{~networth}).}

\item{probs}{Numeric vector with values in between 0 and 1 defining group boundaries,
including 0 and 1 as endpoints. Defaults to deciles
(\code{seq(0, 1, by = 0.1)}).}

\item{labels}{Optional character vector of group labels, length equal to
\code{length(probs) - 1}. If \code{NULL} (default), labels are generated
automatically in the form \code{"p0-p10"}, \code{"p10-p20"}, etc.}

\item{varname}{Optional name for the new grouping variable. Defaults to
\code{"{var}_pctile"} (e.g., \code{"networth_pctile"}).}

\item{method}{Character. One of \code{"implicate"} (default) or
\code{"stack"}. See Details.}

\item{stat}{Character. One of \code{"mean"} (default), \code{"median"},
or \code{"none"}. When \code{"none"}, returns the updated
\code{scf_mi_survey} object with the grouping variable added and no
summary statistic computed. When \code{"mean"} or \code{"median"},
computes the statistic within each percentile group and returns a data
frame of results.}

\item{stat_var}{Optional one-sided formula naming the variable to summarize
within each group. Defaults to \code{var} if not supplied.}
}
\value{
When \code{stat = "none"}, returns the input \code{scf_mi_survey}
object with a new factor variable added to each implicate's data frame.
The variable is named according to \code{varname} (default:
\code{"{var}_pctile"}), and can be passed directly to
\code{\link{scf_mean}}, \code{\link{scf_median}}, or other estimation
functions via the \code{by} argument. When \code{stat = "mean"} or
\code{"median"} with \code{method = "implicate"}, returns the output of
\code{\link{scf_mean}} or \code{\link{scf_median}} respectively. When
\code{stat != "none"} with \code{method = "stack"}, returns a data frame
with columns \code{group}, \code{variable}, and \code{estimate}. No
standard error is returned for the stack method.
}
\description{
Creates a percentile-based grouping variable for a continuous SCF variable
and optionally computes a summary statistic within each group. Two methods
are available. The \code{"implicate"} method (the default) computes
survey-weighted quantile thresholds separately within each implicate, the
statistically preferable approach under multiple imputation, as it correctly
accounts for between-implicate variation in imputed values. The
\code{"stack"} method replicates the Federal Reserve's published convention,
in which all five implicates are pooled with weights divided by five,
observations are sorted by the percentile variable and household identifier,
percentile groups are assigned from the cumulative weighted population share,
and a flat weighted statistic is computed directly on the stacked data.
}
\details{
The two methods will generally produce similar but not identical results.
Use \code{method = "stack"} when replication of the Federal Reserve's
published percentile-group point estimates is required. Note that
\code{method = "stack"} with \code{stat != "none"} computes a flat weighted
statistic on the pooled stacked data and does not use replicate-weight
survey machinery or Rubin's combining rules. No standard error is
returned in this case.

When \code{stat = "none"}, the function returns the input
\code{scf_mi_survey} object with the grouping variable added, ready to pass
to \code{\link{scf_mean}}, \code{\link{scf_median}}, or other estimation
functions via the \code{by} argument. For standard grouping variables
already published by the Fed, such as net worth percentile (\code{nwcat})
and income percentile (\code{inccat}), those variables are included directly
in the data returned by \code{\link{scf_load}} and can be passed to
\code{by} without calling \code{scf_pctile_sum}.

For \code{method = "stack"}, percentile groups are assigned using the
Federal Reserve SAS macro logic: after stacking implicates and dividing
weights by five, observations are sorted by the variable used to define the
percentile groups and by household identifier. The cumulative weighted
population share is then used to assign groups. This is not equivalent to
first estimating a percentile threshold and then applying a simple
\code{>} or \code{>=} comparison, especially when observations lie at or
near a percentile boundary.
}
\examples{
# Do not implement these lines in real analysis:
# Use functions `scf_download()` and `scf_load()`
td <- tempfile("pctile_sum_")
dir.create(td)
src <- system.file("extdata", "scf2022_mock_raw.rds", package = "scf")
file.copy(src, file.path(td, "scf2022.rds"), overwrite = TRUE)
scf2022 <- scf_load(2022, data_directory = td)

# Mean net worth, top vs bottom 90 percent, stack method (fast)
scf_pctile_sum(scf2022, ~networth,
               probs  = c(0, 0.9, 1),
               labels = c("bottom90", "top10"),
               method = "stack")

\dontrun{
# Implicate method (default): requires full SCF data; unreliable on mock data
scf_pctile_sum(scf2022, ~networth)

# Return grouping variable only, no summary statistic (implicate method)
scf2022 <- scf_pctile_sum(scf2022, ~networth,
                           probs  = c(0, 0.9, 1),
                           labels = c("bottom90", "top10"),
                           stat   = "none")
}

# Do not implement these lines in real analysis: Cleanup for package check
unlink(td, recursive = TRUE, force = TRUE)

}
\seealso{
\code{\link{scf_mean}}, \code{\link{scf_median}},
\code{\link{scf_percentile}}, \code{\link{scf_update_by_implicate}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scf_prop_test.R
\name{scf_prop_test}
\alias{scf_prop_test}
\title{Test a Proportion in SCF Data}
\usage{
scf_prop_test(
  design,
  var,
  group = NULL,
  p = 0.5,
  alternative = c("two.sided", "less", "greater"),
  conf.level = 0.95
)
}
\arguments{
\item{design}{A \code{scf_mi_survey} object created by \code{\link[=scf_load]{scf_load()}}. Must contain replicate-weighted implicates.}

\item{var}{A one-sided formula indicating a binary variable (e.g., \code{~rich}).}

\item{group}{Optional one-sided formula indicating a binary grouping variable (e.g., \code{~female}). If omitted, a one-sample test is performed.}

\item{p}{Null hypothesis value. Defaults to \code{0.5} for one-sample, \code{0} for two-sample tests.}

\item{alternative}{Character. One of \code{"two.sided"} (default), \code{"less"}, or \code{"greater"}.}

\item{conf.level}{Confidence level for the confidence interval. Default is \code{0.95}.}
}
\value{
An object of class \code{"scf_prop_test"} with:
\describe{
\item{results}{A data frame with the pooled estimate, standard error, z-statistic, p-value, confidence interval, and significance stars.}
\item{proportions}{(Only in two-sample tests) A data frame of pooled proportions by group.}
\item{fit}{A list describing the method, null value, alternative hypothesis, and confidence level.}
}
}
\description{
Tests a binary variable's proportion against a null hypothesis (one-sample),
or compares proportions across two groups (two-sample). Supports two-sided,
less-than, or greater-than alternatives.
}
\section{Statistical Notes}{

Proportions are computed in each implicate using weighted means, and variances are approximated under the binomial model.
Rubin’s Rules are applied to pool point estimates and standard errors. For pooling details, see \code{\link[=scf_MIcombine]{scf_MIcombine()}}.
}

\examples{
# Do not implement these lines in real analysis:
# Use functions `scf_download()` and `scf_load()`
td <- tempfile("proptest_")
dir.create(td)

src <- system.file("extdata", "scf2022_mock_raw.rds", package = "scf")
file.copy(src, file.path(td, "scf2022.rds"), overwrite = TRUE)
scf2022 <- scf_load(2022, data_directory = td)

# Wrangle data for example
scf2022 <- scf_update(scf2022,
  rich   = networth > 1e6,
  female = factor(hhsex, levels = 1:2, labels = c("Male","Female")),
  over50 = age > 50
)

# Example for real analysis: One-sample test
scf_prop_test(scf2022, ~rich, p = 0.10)

# Example for real analysis: Two-sample test
scf_prop_test(scf2022, ~rich, ~female, alternative = "less")

# Do not implement these lines in real analysis: Cleanup for package check
unlink(td, recursive = TRUE, force = TRUE)


}
\seealso{
\code{\link[=scf_ttest]{scf_ttest()}}, \code{\link[=scf_mean]{scf_mean()}}, \code{\link[=scf_MIcombine]{scf_MIcombine()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scf_plot_cbar.R
\name{scf_plot_cbar}
\alias{scf_plot_cbar}
\title{Bar Plot of Summary Statistics by Grouping Variable in SCF Data}
\usage{
scf_plot_cbar(
  design,
  yvar,
  xvar,
  stat = "mean",
  title = NULL,
  xlab = NULL,
  ylab = NULL,
  fill = "#0072B2",
  angle = 30,
  label_map = NULL
)
}
\arguments{
\item{design}{A \code{scf_mi_survey} object from \code{\link[=scf_load]{scf_load()}}.}

\item{yvar}{One-sided formula for the continuous variable (e.g., \code{~networth}).}

\item{xvar}{One-sided formula for the grouping variable (e.g., \code{~racecl}).}

\item{stat}{\code{"mean"} (default), \code{"median"}, or a quantile (numeric between 0 and 1).}

\item{title}{Plot title (optional).}

\item{xlab}{X-axis label (optional).}

\item{ylab}{Y-axis label (optional).}

\item{fill}{Bar fill color. Default is \code{"#0072B2"}.}

\item{angle}{Angle of x-axis labels. Default is 30.}

\item{label_map}{Optional named vector to relabel x-axis category labels.}
}
\value{
A \code{ggplot2} object.
}
\description{
Computes and plots a grouped summary statistic (either a mean, median, or
quantile) for a continuous variable across a discrete factor. Estimates are
pooled across implicates using \code{\link[=scf_mean]{scf_mean()}}, \code{\link[=scf_median]{scf_median()}}, or
\code{\link[=scf_percentile]{scf_percentile()}}. Use this function to visualize the bivariate relationship
between a discrete and a continuous variable.
}
\section{Implementation}{

The user specifies a continuous outcome (\code{yvar}) and a discrete grouping
variable (\code{xvar}) via one-sided formulas. Group means are plotted by default.
Medians or other percentiles can be specified via the \code{stat} argument.

Results are plotted using \code{ggplot2::geom_col()}, styled with \code{\link[=scf_theme]{scf_theme()}},
and optionally customized with additional arguments (e.g., axis labels,
color, angles).
}

\examples{
# Do not implement these lines in real analysis:
# Use functions `scf_download()` and `scf_load()`
td <- tempfile("plot_cbar_")
dir.create(td)

src <- system.file("extdata", "scf2022_mock_raw.rds", package = "scf")
file.copy(src, file.path(td, "scf2022.rds"), overwrite = TRUE)
scf2022 <- scf_load(2022, data_directory = td)

# Example for real analysis: Plot mean net worth by education level
scf_plot_cbar(scf2022, ~networth, ~edcl, stat = "mean")

# Example for real analysis: Visualize 90th percentile of income by education
scf_plot_cbar(scf2022, ~income, ~edcl, stat = 0.9, fill = "#D55E00")

# Do not implement these lines in real analysis: Cleanup for package check
unlink(td, recursive = TRUE, force = TRUE)

}
\seealso{
\code{\link[=scf_mean]{scf_mean()}}, \code{\link[=scf_median]{scf_median()}}, \code{\link[=scf_percentile]{scf_percentile()}}, \code{\link[=scf_theme]{scf_theme()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scf_plot_dist.R
\name{scf_plot_dist}
\alias{scf_plot_dist}
\title{Plot a Univariate Distribution of an SCF Variable}
\usage{
scf_plot_dist(
  design,
  variable,
  bins = 30,
  title = NULL,
  xlab = NULL,
  ylab = "Percent",
  angle = 30,
  fill = "#0072B2",
  labels = NULL
)
}
\arguments{
\item{design}{A \code{scf_mi_survey} object created by \code{\link[=scf_load]{scf_load()}}.}

\item{variable}{A one-sided formula specifying the variable to plot.}

\item{bins}{Number of bins for continuous variables. Default is 30.}

\item{title}{Optional plot title.}

\item{xlab}{Optional x-axis label.}

\item{ylab}{Optional y-axis label. Default is "Percent".}

\item{angle}{Angle for x-axis tick labels. Default is 30.}

\item{fill}{Fill color for bars. Default is \code{"#0072B2"}.}

\item{labels}{Optional named vector of custom axis labels (for discrete variables only).}
}
\value{
A \code{ggplot2} object.
}
\description{
This function provides a unified plotting interface for visualizing the
distribution of a single variable from multiply-imputed SCF data. Discrete
variables produce bar charts of pooled proportions; continuous variables
produce binned histograms.  Use this function to visualize the univariate
distribution of an SCF variable.
}
\section{Implementation}{

For discrete variables (factor or numeric with <= 25 unique values), the
function uses \code{\link[=scf_freq]{scf_freq()}} to calculate category proportions and produces a
bar chart.  For continuous variables, it bins values across implicates and
estimates Rubin-pooled frequencies for each bin.

Users may supply a named vector of custom axis labels using the \code{labels} argument.
}

\examples{
# Do not implement these lines in real analysis:
# Use functions `scf_download()` and `scf_load()`
td <- tempfile("plot_dist_")
dir.create(td)

src <- system.file("extdata", "scf2022_mock_raw.rds", package = "scf")
file.copy(src, file.path(td, "scf2022.rds"), overwrite = TRUE)
scf2022 <- scf_load(2022, data_directory = td)

# Example for real analysis: Distribution of homeownership
scf_plot_dist(scf2022, ~own)

# Example for real analysis: Distribution of age
scf_plot_dist(scf2022, ~age, bins = 10)

# Do not implement these lines in real analysis: Cleanup for package check
unlink(td, recursive = TRUE, force = TRUE)

}
\seealso{
\code{\link[=scf_theme]{scf_theme()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scf_regtable.R
\name{scf_regtable}
\alias{scf_regtable}
\title{Format and Display Regression Results from Multiply-Imputed SCF Models}
\usage{
scf_regtable(
  ...,
  model.names = NULL,
  digits = 0,
  auto_digits = FALSE,
  labels = NULL,
  output = c("console", "markdown", "latex", "csv"),
  file = NULL
)
}
\arguments{
\item{...}{One or more SCF regression model objects, or a single list of such models.}

\item{model.names}{Optional character vector naming the models. Defaults to
\code{"Model 1"}, \code{"Model 2"}, etc.}

\item{digits}{Integer specifying decimal places for numeric formatting when
\code{auto_digits = FALSE}. Default is 0.}

\item{auto_digits}{Logical; if \code{TRUE}, uses adaptive decimal places:
0 digits for large numbers (>= 1000), 2 digits for moderate (>= 1),
and 3 digits for smaller values.}

\item{labels}{Optional named character vector or labeling function to replace
term names with descriptive labels.}

\item{output}{Output format: one of \code{"console"} (print to console),
\code{"markdown"} (print Markdown table for R Markdown), \code{"latex"}
(print LaTeX table for PDF compilation), or \code{"csv"}
(write CSV file).}

\item{file}{File path for CSV output; required if \code{output = "csv"}.}
}
\value{
Invisibly returns a data frame with formatted regression results and fit statistics.
}
\description{
This function formats and aligns coefficient estimates, standard errors, and
significance stars from one or more SCF regression model objects
(e.g., from \code{scf_ols()}, \code{scf_logit()}, \code{scf_quantreg()}, or \code{scf_glm()}).
}
\details{
It compiles a side-by-side table with terms matched across models, appends
model fit statistics (sample size N, R-squared or pseudo-R-squared, quantile tau,
and AIC where applicable), and outputs the results as console text, Markdown for
R Markdown documents, LaTeX for PDF compilation, or a CSV file.

The function aligns all unique coefficient terms across provided models, formats
coefficients with significance stars and standard errors, appends model fit
statistics as additional rows, and renders output in the specified format.

Fit statistics rows are automatically selected based on model class:
\describe{
\item{All models}{Sample size (N)}
\item{OLS models}{R-squared and AIC}
\item{Logit/GLM models}{Pseudo-R-squared and AIC}
\item{Quantile regression}{Quantile tau, R1(tau), and adjusted R1(tau)}
}

It avoids external dependencies by using base R formatting and simple text,
Markdown, LaTeX, or CSV output.
}
\examples{
# Do not implement these lines in real analysis:
# Use functions `scf_download()` and `scf_load()`
td <- tempfile("regtable_")
dir.create(td)

src <- system.file("extdata", "scf2022_mock_raw.rds", package = "scf")
file.copy(src, file.path(td, "scf2022.rds"), overwrite = TRUE)
scf2022 <- scf_load(2022, data_directory = td)

# Wrangle data for example:  Perform OLS regression 
m1 <- scf_ols(scf2022, income ~ age)

# Example for real analysis: Print regression results as a console table
scf_regtable(m1, digits = 2)

# Do not implement these lines in real analysis: Cleanup for package check
unlink(td, recursive = TRUE, force = TRUE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scf_update.R
\name{scf_update}
\alias{scf_update}
\title{Create or Alter SCF Variables}
\usage{
scf_update(object, ...)
}
\arguments{
\item{object}{A \code{scf_mi_survey} object, typically created by \code{\link[=scf_load]{scf_load()}}.}

\item{...}{Named expressions assigning new or modified variables using \code{=} syntax.
Each expression must return a vector of the same length as the implicate data frame.}
}
\value{
The input \code{scf_mi_survey} object with \code{mi_design} updated to reflect
the new or modified variables. All other attributes (\code{year}, \code{n_households},
\code{mock}) are preserved unchanged.
}
\description{
Use this function to create or alter SCF variables once the raw data set has
been loaded into memory using the \code{scf_load()} function. This function
updates an \code{scf_mi_survey} object by evaluating transformations within each
implicate, and then returning a new object with the new or amended variables.

Most of the time, you can use \code{scf_update()} to define variables based on
simple logical conditions, arithmetic transformations, or categorical
binning. These rules are evaluated separately in each implicate, using the
same formula. However, if the transformation you want to apply depends on the
distribution of the data within each implicate, such as computing an
average percentile or ranking households across all implicates,
this function will not suffice. In those cases, use
\code{\link[=scf_update_by_implicate]{scf_update_by_implicate()}} to write a custom function that operates on each
implicate individually.
}
\section{Usage}{

Use \code{scf_update()} during data wrangling to clean, create, or alter variables before calculating
statistics or running models. The function is useful when the analyst wishes to:
\itemize{
\item Recode missing values that are coded as numeric data
\item Recast variables that are not in the desired format (e.g., converting a numeric variable to a factor)
\item Create new variables based on existing ones (e.g., calculating ratios, differences, or indicators)
}
}

\examples{
# Do not implement these lines in real analysis:
# Use functions `scf_download()` and `scf_load()`
td <- tempfile("update_")
dir.create(td)

src <- system.file("extdata", "scf2022_mock_raw.rds", package = "scf")
file.copy(src, file.path(td, "scf2022.rds"), overwrite = TRUE)
scf2022 <- scf_load(2022, data_directory = td)

# Example for real analysis: Create a binary indicator for being over age 50
scf2022 <- scf_update(scf2022,
  over50 = age > 50
)

# Example: Create a log-transformed income variable
scf2022 <- scf_update(scf2022,
  log_income = log(income + 1)
)

# Do not implement these lines in real analysis: Cleanup for package check
unlink(td, recursive = TRUE, force = TRUE)

}
\seealso{
\code{\link[=scf_load]{scf_load()}}, \code{\link[=scf_update_by_implicate]{scf_update_by_implicate()}}, \code{\link[survey:svrepdesign]{survey::svrepdesign()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scf_plot_smooth.R
\name{scf_plot_smooth}
\alias{scf_plot_smooth}
\title{Smoothed Distribution Plot of a Continuous Variable in SCF Data}
\usage{
scf_plot_smooth(
  design,
  variable,
  binwidth = NULL,
  xlim = NULL,
  method = "loess",
  span = 0.2,
  color = "blue",
  xlab = NULL,
  ylab = "Percent of Households",
  title = NULL
)
}
\arguments{
\item{design}{A \code{scf_mi_survey} object created by \code{\link[=scf_load]{scf_load()}}.}

\item{variable}{A one-sided formula specifying a continuous variable (e.g., \code{~networth}).}

\item{binwidth}{Optional bin width. Default uses Freedman–Diaconis rule.}

\item{xlim}{Optional numeric vector of length 2 to truncate axis.}

\item{method}{Character. Smoothing method: \code{"loess"} (default) or \code{"lm"}.}

\item{span}{Numeric LOESS span. Default is \code{0.2}. Ignored if \code{method = "lm"}.}

\item{color}{Line color. Default is \code{"blue"}.}

\item{xlab}{Optional label for x-axis. Defaults to the variable name.}

\item{ylab}{Optional label for y-axis. Defaults to \code{"Percent of Households"}.}

\item{title}{Optional plot title.}
}
\value{
A \code{ggplot2} object.
}
\description{
Draws a smoothed distribution plot of a continuous variable in the SCF. Use
this function to visualize a single continuous variable's distribution.
}
\section{Implementation}{

Visualizes the weighted distribution of a continuous SCF variable by stacking implicates,
binning observations, and smoothing pooled proportions. This function is useful for
examining distribution shape, skew, or modality in variables like income or wealth.

All implicates are stacked and weighted, binned across a data-driven or user-specified
bin width. Each bin's weight share is calculated, and a smoothing curve is fit to
the resulting pseudo-density.
}

\examples{
# Do not implement these lines in real analysis:
# Use functions `scf_download()` and `scf_load()`
td <- tempfile("plot_smooth_")
dir.create(td)

src <- system.file("extdata", "scf2022_mock_raw.rds", package = "scf")
file.copy(src, file.path(td, "scf2022.rds"), overwrite = TRUE)
scf2022 <- scf_load(2022, data_directory = td)

# Example for real analysis: Plot smoothed distribution
scf_plot_smooth(scf2022, ~networth, xlim = c(0, 2e6),
                method = "loess", span = 0.25)
     
# Do not implement these lines in real analysis: Cleanup for package check           
unlink(td, recursive = TRUE, force = TRUE)

}
\seealso{
\code{\link[=scf_plot_hist]{scf_plot_hist()}}, \code{\link[=scf_plot_dist]{scf_plot_dist()}}, \code{\link[=scf_theme]{scf_theme()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scf_plot_dbar.R
\name{scf_plot_dbar}
\alias{scf_plot_dbar}
\title{Plot Bar Chart of a Discrete Variable from SCF Data}
\usage{
scf_plot_dbar(
  design,
  variable,
  title = NULL,
  xlab = NULL,
  ylab = "Percent",
  angle = 30,
  fill = "#0072B2",
  label_map = NULL
)
}
\arguments{
\item{design}{A \code{scf_mi_survey} object created by \code{\link[=scf_load]{scf_load()}}. Must contain valid implicates.}

\item{variable}{A one-sided formula specifying a categorical variable (e.g., \code{~racecl}).}

\item{title}{Optional character string for the plot title. Default: \code{"Distribution of <variable>"}.}

\item{xlab}{Optional x-axis label. Default: variable name.}

\item{ylab}{Optional y-axis label. Default: \code{"Percent"}.}

\item{angle}{Integer. Rotation angle for x-axis labels. Default is \code{30}.}

\item{fill}{Fill color for bars. Default is \code{"#0072B2"}.}

\item{label_map}{Optional named vector to relabel x-axis category labels.}
}
\value{
A \code{ggplot2} object representing the pooled bar chart.
}
\description{
Creates a bar chart that visualizes the distribution of a discrete variable.
}
\section{Implementation}{

This function internally calls \code{\link[=scf_freq]{scf_freq()}} to compute population proportion
estimates, which are then plotted using \code{ggplot2::geom_col()}. The default
output is scaled to percent and can be customized via title, axis labels,
angle, and color.
}

\section{Details}{

Produces a bar chart of category proportions from a one-way tabulation,
pooled across SCF implicates using \code{\link[=scf_freq]{scf_freq()}}. This function summarizes
weighted sample composition and communicates categorical distributions
effectively in descriptive analysis.
}

\section{Dependencies}{

Requires the \code{ggplot2} package.
}

\examples{
# Do not implement these lines in real analysis:
# Use functions `scf_download()` and `scf_load()`
td <- tempfile("plot_dbar_")
dir.create(td)

src <- system.file("extdata", "scf2022_mock_raw.rds", package = "scf")
file.copy(src, file.path(td, "scf2022.rds"), overwrite = TRUE)
scf2022 <- scf_load(2022, data_directory = td)

# Example for real analysis: Bar chart of education categories
scf_plot_dbar(scf2022, ~edcl)

# Do not implement these lines in real analysis: Cleanup for package check
unlink(td, recursive = TRUE, force = TRUE)

}
\seealso{
\code{\link[=scf_freq]{scf_freq()}}, \code{\link[=scf_plot_bbar]{scf_plot_bbar()}}, \code{\link[=scf_xtab]{scf_xtab()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scf_subset.R
\name{scf_subset}
\alias{scf_subset}
\title{Subset an \code{scf_mi_survey} Object}
\usage{
scf_subset(scf, expr)
}
\arguments{
\item{scf}{A \code{scf_mi_survey} object, typically created by \code{\link[=scf_load]{scf_load()}}.}

\item{expr}{A logical expression used to filter rows, evaluated separately in each implicate's variable frame (e.g., \code{age < 65 & own == 1}).}
}
\value{
A new \code{scf_mi_survey} object (see \code{\link[=scf_design]{scf_design()}})
}
\description{
Subsetting refers to the process of retaining only those observations that
satisfy a logical (TRUE/FALSE) condition. This function applies such a
filter independently to each implicate in an \code{scf_mi_survey} object created
by \code{\link[=scf_design]{scf_design()}} via \code{\link[=scf_load]{scf_load()}}. The result is a new multiply-imputed,
replicate-weighted survey object with appropriately restricted designs.
}
\section{Implementation}{

Use \code{scf_subset()} to focus analysis on analytically meaningful
sub-populations. For example, to analyze only households headed by seniors:

\preformatted{
scf2022_seniors <- scf_subset(scf2022, age >= 65)
}

This is especially useful when analyzing populations such as renters, homeowners, specific age brackets,
or any group defined by logical expressions over SCF variables.
}

\section{Details}{

Filtering is conducted separately in each implicate. This preserves valid design structure but means
that the same household may fall into or out of the subset depending on imputed values.
For example, a household with five different age imputations—say, 64, 66, 63, 65, and 67—would be
classified as a senior in only three of five implicates if subsetting on \code{age >= 65}.

Empty subsets in any implicate can cause downstream analysis to fail. Always check subgroup sizes after subsetting.
}

\examples{
# Do not implement these lines in real analysis:
# Use functions `scf_download()` and `scf_load()`
td <- tempfile("subset_")
dir.create(td)

src <- system.file("extdata", "scf2022_mock_raw.rds", package = "scf")
file.copy(src, file.path(td, "scf2022.rds"), overwrite = TRUE)
scf2022 <- scf_load(2022, data_directory = td)

# Example for real analysis: Filter for working-age households with positive net worth
scf_sub <- scf_subset(scf2022, age < 65 & networth > 0)
scf_mean(scf_sub, ~income)

# Do not implement these lines in real analysis: Cleanup for package check
unlink(td, recursive = TRUE, force = TRUE)


}
\seealso{
\code{\link[=scf_load]{scf_load()}}, \code{\link[=scf_update]{scf_update()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scf_ols.R
\name{scf_ols}
\alias{scf_ols}
\title{Estimate an Ordinary Least Squares Regression on SCF Microdata}
\usage{
scf_ols(object, formula)
}
\arguments{
\item{object}{A \code{scf_mi_survey} object created with \code{\link[=scf_load]{scf_load()}} and \code{\link[=scf_design]{scf_design()}}. Must contain five implicates with replicate weights.}

\item{formula}{A model formula specifying a continuous outcome and predictor variables (e.g., \code{networth ~ income + age}).}
}
\value{
An object of class \code{"scf_ols"} and \code{"scf_model_result"} with:
\describe{
\item{results}{A data frame of pooled coefficients, standard errors, t-values, p-values, and significance stars.}
\item{fit}{A list of model diagnostics including mean AIC, standard deviation of AIC, mean R-squared, and its standard deviation.}
\item{imps}{A list of implicate-level \code{svyglm} model objects.}
\item{call}{The matched call used to produce the model.}
}
}
\description{
Computes an OLS regression on SCF data using \code{svyglm()} across the SCF's
five implicates.  Returns coefficient estimates, standard errors, test
statistics, and model diagnostics.
}
\details{
Fits a replicate-weighted linear regression model to each implicate of
multiply-imputed SCF data and pools coefficients and standard errors using
Rubin’s Rules.
}
\section{Implementation}{

Ordinary least squares (OLS) regression estimates the linear relationship
between a continuous outcome and one or more predictor variables. Each
coefficient represents the expected change in the outcome for a one-unit
increase in the corresponding predictor, holding all other predictors
constant.

Use this function to model associations between SCF variables while
accounting for complex survey design and multiple imputation.

This function takes a \code{scf_mi_survey} object and a model formula. Internally,
it fits a weighted linear regression to each implicate using
\code{survey::svyglm()}, extracts coefficients and variance-covariance matrices,
and pools them via \code{\link[=scf_MIcombine]{scf_MIcombine()}}.
}

\examples{
# Do not implement these lines in real analysis:
# Use functions `scf_download()` and `scf_load()`
td <- tempfile("ols_")
dir.create(td)

src <- system.file("extdata", "scf2022_mock_raw.rds", package = "scf")
file.copy(src, file.path(td, "scf2022.rds"), overwrite = TRUE)
scf2022 <- scf_load(2022, data_directory = td)

# Example for real analysis: Run OLS model
model <- scf_ols(scf2022, networth ~ income + age)
summary(model)

# Do not implement these lines in real analysis: Cleanup for package check
unlink(td, recursive = TRUE, force = TRUE)

}
\seealso{
\code{\link[=scf_glm]{scf_glm()}}, \code{\link[=scf_logit]{scf_logit()}}, \code{\link[=scf_MIcombine]{scf_MIcombine()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scf_model_methods.R
\name{scf_model_result_methods}
\alias{scf_model_result_methods}
\alias{coef.scf_model_result}
\alias{vcov.scf_model_result}
\alias{AIC.scf_model_result}
\alias{residuals.scf_model_result}
\alias{predict.scf_model_result}
\alias{formula.scf_model_result}
\title{S3 Methods for scf_model_result Objects}
\usage{
\method{formula}{scf_model_result}(x, ...)

\method{residuals}{scf_model_result}(object, ...)

\method{coef}{scf_model_result}(object, ...)

\method{vcov}{scf_model_result}(object, ...)

\method{AIC}{scf_model_result}(object, k = 2, ...)

\method{predict}{scf_model_result}(object, newdata, type = "link", ...)
}
\arguments{
\item{x}{An object of class \code{"scf_model_result"} (for \code{formula}).}

\item{...}{Additional arguments (not used by most methods).}

\item{object}{An object of class \code{"scf_model_result"}.}

\item{k}{Penalty term passed to \code{AIC()} (default 2 for AIC, \code{log(n)} for BIC).}

\item{newdata}{Optional data frame of new observations for \code{predict()}.
If missing, predictions are made on the original training data.}

\item{type}{Prediction scale for \code{predict()}: \code{"link"} (default)
or \code{"response"}.}
}
\description{
Generic S3 methods dispatched on objects of class \code{"scf_model_result"},
as returned by \code{\link{scf_ols}}, \code{\link{scf_glm}},
\code{\link{scf_logit}}, and \code{\link{scf_quantreg}}.

\describe{
\item{\code{coef()}}{Pooled coefficient estimates (Rubin's Rules).}
\item{\code{vcov()}}{Pooled variance-covariance matrix.}
\item{\code{AIC()}}{Mean AIC across implicates.}
\item{\code{residuals()}}{Residuals from the first implicate model (diagnostic use only).}
\item{\code{predict()}}{Mean predictions pooled across all five implicate models.}
\item{\code{formula()}}{The model formula.}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scf_plot_bbar.R
\name{scf_plot_bbar}
\alias{scf_plot_bbar}
\title{Stacked Bar Chart of Two Discrete Variables in SCF Data}
\usage{
scf_plot_bbar(
  design,
  rowvar,
  colvar,
  scale = c("percent", "count"),
  percent_by = c("total", "row", "col"),
  title = NULL,
  xlab = NULL,
  ylab = NULL,
  fill_colors = NULL,
  row_labels = NULL,
  col_labels = NULL
)
}
\arguments{
\item{design}{A \code{scf_mi_survey} object created by \code{\link[=scf_load]{scf_load()}}. Must contain five implicates with replicate weights.}

\item{rowvar}{A one-sided formula for the x-axis grouping variable (e.g., \code{~edcl}).}

\item{colvar}{A one-sided formula for the stacked fill variable (e.g., \code{~racecl}).}

\item{scale}{Character. One of \code{"percent"} (default) or \code{"count"}.}

\item{percent_by}{Character. One of \code{"total"} (default), \code{"row"}, or \code{"col"} — determines normalization base when \code{scale = "percent"}.}

\item{title}{Optional character string for the plot title.}

\item{xlab}{Optional character string for the x-axis label.}

\item{ylab}{Optional character string for the y-axis label.}

\item{fill_colors}{Optional vector of fill colors to pass to \code{ggplot2::scale_fill_manual()}.}

\item{row_labels}{Optional named vector to relabel \code{row} categories (x-axis).}

\item{col_labels}{Optional named vector to relabel \code{col} categories (legend).}
}
\value{
A \code{ggplot2} object.
}
\description{
Visualizes a discrete-discrete bivariate distribution using stacked bars
based on pooled cross-tabulations from \code{\link[=scf_xtab]{scf_xtab()}}. Use this function to
visualize the relationship between two discrete variables.
}
\section{Implementation}{

This function calls \code{\link[=scf_xtab]{scf_xtab()}} to estimate the joint distribution of two
categorical variables across multiply-imputed SCF data. The result is translated
into a \code{ggplot2} stacked bar chart using pooled counts or normalized percentages.
}

\examples{
# Do not implement these lines in real analysis:
# Use functions `scf_download()` and `scf_load()`
td <- tempfile("plot_bbar_")
dir.create(td)

src <- system.file("extdata", "scf2022_mock_raw.rds", package = "scf")
file.copy(src, file.path(td, "scf2022.rds"), overwrite = TRUE)
scf2022 <- scf_load(2022, data_directory = td)

# Example for real analysis: Stacked bar chart: education by ownership
scf_plot_bbar(scf2022, ~own, ~edcl)

# Example for real analysis: Column percentages instead of total percent
scf_plot_bbar(scf2022, ~own, ~edcl, percent_by = "col")

# Example for real analysis: Raw counts (estimated number of households)
scf_plot_bbar(scf2022, ~own, ~edcl, scale = "count")

# Do not implement these lines in real analysis: Cleanup for package check
unlink(td, recursive = TRUE, force = TRUE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scf_implicates.R
\name{scf_implicates}
\alias{scf_implicates}
\title{Extract Implicate-Level Estimates from SCF Results}
\usage{
scf_implicates(x, long = FALSE)
}
\arguments{
\item{x}{A result object containing implicate-level estimates. Three types are supported:
\describe{
\item{Data frame results}{Objects from \code{\link[=scf_freq]{scf_freq()}}, \code{\link[=scf_mean]{scf_mean()}}, \code{\link[=scf_median]{scf_median()}},
\code{\link[=scf_percentile]{scf_percentile()}}, \code{\link[=scf_corr]{scf_corr()}}, and \code{\link[=scf_xtab]{scf_xtab()}}, whose \verb{$imps} slot contains
a list of data frames.}
\item{Survey statistic results}{Objects whose \verb{$imps} slot contains \code{svystat}
objects (e.g., raw outputs from \code{survey::svymean()}).}
\item{Model results}{Objects from \code{\link[=scf_ols]{scf_ols()}}, \code{\link[=scf_glm]{scf_glm()}}, \code{\link[=scf_logit]{scf_logit()}}, and
\code{\link[=scf_quantreg]{scf_quantreg()}}, whose \verb{$imps} or \verb{$models} slot contains \code{svyglm} fits.}
}}

\item{long}{Logical. If TRUE, returns stacked data frame. If FALSE, returns list.}
}
\value{
A list of implicate-level data frames, or a single stacked data frame if \code{long = TRUE}.
}
\description{
Returns implicate-level outputs from SCF result objects produced by functions
in the \code{scf} suite. Supports result objects containing implicate-level data
frames, \code{svystat} summaries, or \code{svyglm} model fits.
}
\section{Usage}{

This function allows users to inspect how estimates vary across the SCF’s five implicates,
which is important for diagnostics, robustness checks, and transparent reporting.

For example:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{scf_implicates(scf_mean(scf2022, ~income))
scf_implicates(scf_ols(scf2022, networth ~ age + income), long = TRUE)
}\if{html}{\out{</div>}}
}

\examples{
# Do not implement these lines in real analysis:
# Use functions `scf_download()` and `scf_load()`
td <- tempfile("implicates_")
dir.create(td)

src <- system.file("extdata", "scf2022_mock_raw.rds", package = "scf")
file.copy(src, file.path(td, "scf2022.rds"), overwrite = TRUE)
scf2022 <- scf_load(2022, data_directory = td)

# Example for real analysis: Extract implicate-level results
out <- scf_freq(scf2022, ~own)
scf_implicates(out, long = TRUE)

# Do not implement these lines in real analysis: Cleanup for package check
unlink(td, recursive = TRUE, force = TRUE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scf-package.R
\docType{package}
\name{scf}
\alias{scf-package}
\alias{scf}
\title{Analyzing Survey of Consumer Finances Public-Use Microdata}
\description{
This package provides functions to analyze the U.S. Federal Reserve's
Survey of Consumer Finances (SCF) public-use microdata. It encapsulates
the SCF’s multiply-imputed, replicate-weighted structure in a custom
object class (\code{scf_mi_survey}) and supports estimation of population-level
statistics, including univariate and bivariate distributions,
hypothesis tests, data visualizations, and regression models.

Designed for generalist analysts, the package assumes familiarity with
standard statistical methods but not with the complexities of
multiply-imputed or survey-weighted data. All functions prioritize
transparency, reproducibility, and pedagogical clarity.
}
\section{Methodological Background}{


The SCF is one of the most detailed and methodologically rigorous sources of
data on U.S. household finances. It is nationally representative, includes an
oversample of high-wealth households and households in predominantly Black
communities, and provides multiply-imputed estimates for item nonresponse.
These features increase the analytical value of the data set but also
introduce methodological complexity. Valid inference requires attention to:
\itemize{
\item \strong{Survey Weights:} The SCF employs a dual-frame, stratified, and clustered
probability sample. Analysts must apply the provided sampling weights to
produce population-representative estimates.
\item \strong{Replicate Weights:} Each observation is associated with 999 replicate weights,
generated using a custom replication method developed by the Federal
Reserve. These are used to estimate sampling variance.
\item \strong{Multiple Imputation:} The SCF uses multiple imputation to address item nonresponse,
providing five implicates per household. Estimates must be pooled across
implicates to obtain valid point estimates and standard errors.
}

The \code{scf} package provides a structured, user-friendly interface for handling
these design complexities, enabling applied researchers and generalist
analysts to conduct principled and reproducible analysis of SCF microdata
using familiar statistical workflows.
}

\section{Package Architecture and Workflow}{


This section recommends a sequence of operations enacted through the package's
functions.  For an in-depth discussion of the methodological considerations
involved in these functions' formulation, see Cohen (2026).
\enumerate{
\item \strong{Data Acquisition}:  Download the data from Federal Reserve servers to your working directory using \code{\link[=scf_download]{scf_download()}}.
\item \strong{Data Loading}: Load the data into R using \code{\link[=scf_load]{scf_load()}}. This function returns an \code{scf_mi_survey} object (described below).
\item \strong{Data Wrangling}: Use \code{\link[=scf_update]{scf_update()}} to modify the data, or \code{\link[=scf_subset]{scf_subset()}} to filter it. These functions return new \code{scf_mi_survey} objects.
\item \strong{Descriptive Statistics}: Compute univariate and bivariate statistics using functions like \code{\link[=scf_mean]{scf_mean()}}, \code{\link[=scf_median]{scf_median()}}, \code{\link[=scf_percentile]{scf_percentile()}}, \code{\link[=scf_freq]{scf_freq()}}, \code{\link[=scf_xtab]{scf_xtab()}}, and \code{\link[=scf_corr]{scf_corr()}}.
\item \strong{Basic Inferential Tests}: Conduct hypothesis tests using \code{\link[=scf_ttest]{scf_ttest()}} for means and \code{\link[=scf_prop_test]{scf_prop_test()}} for proportions.
\item \strong{Regression Modeling}: Fit regression models using \code{\link[=scf_ols]{scf_ols()}} for linear regression, \code{\link[=scf_logit]{scf_logit()}} for logistic regression, and \code{\link[=scf_glm]{scf_glm()}} for generalized linear models.
\item \strong{Data Visualization}: Create informative visualizations using \code{\link[=scf_plot_dist]{scf_plot_dist()}} for distributions, \code{\link[=scf_plot_cbar]{scf_plot_cbar()}} and \code{\link[=scf_plot_bbar]{scf_plot_bbar()}} for categorical data, \code{\link[=scf_plot_smooth]{scf_plot_smooth()}} for smoothers, and \code{\link[=scf_plot_hex]{scf_plot_hex()}} for hexbin plots.
\item \strong{Diagnostics and Infrastructure}: Use \code{\link[=scf_MIcombine]{scf_MIcombine()}} to pool results across implicates.
}
}

\section{Core Data Object and Its Structure}{


This suite of functions operate from a custom object class, \code{scf_mi_survey},
which is created by \code{\link[=scf_design]{scf_design()}} via \code{\link[=scf_load]{scf_load()}}.  Specifically, the
object is a structured list containing the elements:
\itemize{
\item \code{mi_design}: A list of five \code{survey::svrepdesign()} objects (one per implicate)
\item \code{year}: Year of survey
\item \code{n_households}: Estimated number of U.S. households in that year, per data from the Federal Reserve Economic Data (FRED) series TTLHH, accessed 6/17/2025.
}
}

\section{Imputed Missing Data}{


The SCF addresses item nonresponse using multiple imputation
(see Kennickell 1998). This procedure generates five completed data sets,
each containing distinct but plausible values for the missing entries. The
method applies a predictive model to the observed data, simulates variation
in both model parameters and residuals, and generates five independent
estimates for each missing value. These completed data sets—called
\emph{implicates}—reflect both observed relationships and the uncertainty in
estimating them. See \code{\link[=scf_MIcombine]{scf_MIcombine()}} for details.
}

\section{Mock Data for Testing}{


A mock SCF dataset (\code{scf2022_mock_raw.rds}) is bundled in \verb{inst/extdata/} for
internal testing purposes. It is a structurally valid \code{scf_mi_survey} object
created by retaining only the first ~200 rows per implicate and only variables
used in examples and tests.

This object is intended solely for package development and documentation rendering.
It is \strong{not suitable for analytical use or valid statistical inference.}
}

\section{Theming and Visual Style}{

All built-in graphics follow a common aesthetic set by \code{\link[=scf_theme]{scf_theme()}}. Users
may modify the default theme by calling \code{scf_theme()} explicitly within their
scripts. See \code{\link[=scf_theme]{scf_theme()}} documentation for customization options.
}

\section{Pedagogical Design}{


The package is designed to support instruction in advanced methods courses
on complex survey analysis and missing data. It promotes pedagogical
transparency through several features:
\itemize{
\item Each implicate’s design object is accessible via \code{scf_mi_survey$mi_design[[i]]}
\item Raw implicate-level data can be viewed directly through \code{scf_mi_survey$mi_design[[i]]$variables}
\item Users can execute analyses on individual implicates or combine them using Rubin’s Rules
\item Key functions implement design-based estimation strategies explicitly, such as replicate-weight variance estimation
\item Minimal abstraction is used, so each step remains visible and tractable
}

These features allow instructors to demonstrate how survey weights, replicate
designs, and multiple imputation contribute to final results. Students can
follow the full analytic path from raw inputs to pooled estimates using
transparent, inspectable code and data structures.
}

\references{
Cohen JN. Analyzing the Survey of Consumer Finances with \pkg{scf}. 2026.
\url{https://osf.io/azrsn}

Barnard J, Rubin DB. Small-sample degrees of freedom with multiple imputation.
\doi{10.1093/biomet/86.4.948}.

Bricker J, Henriques AM, Moore KB. Updates to the sampling of wealthy families in the Survey of Consumer Finances.
Finance and Economics Discussion Series 2017-114.
https://www.federalreserve.gov/econres/scfindex.htm

Kennickell AB, McManus DA, Woodburn RL. Weighting design for the 1992 Survey of Consumer Finances.
U.S. Federal Reserve.
\url{https://www.federalreserve.gov/Pubs/OSS/oss2/papers/weight92.pdf}

Kennickell AB. Multiple imputation in the Survey of Consumer Finances.
Statistical Journal of the IAOS 33(1):143-151.
\doi{10.3233/SJI-160278}.

Little RJA, Rubin DB. Statistical analysis with missing data.
ISBN: 9780470526798.

Lumley T. survey: Analysis of complex survey samples. R package version 4.1-1.
\url{https://CRAN.R-project.org/package=survey}

Lumley T. Analysis of complex survey samples.
\doi{10.18637/jss.v009.i08}.

Lumley T. Complex surveys: A guide to analysis using R. ISBN: 9781118210932.

U.S. Federal Reserve. Codebook for 2022 Survey of Consumer Finances.
https://www.federalreserve.gov/econres/scfindex.htm
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/jncohen/scf}
  \item Report bugs at \url{https://github.com/jncohen/scf/issues}
}

}
\author{
Joseph N. Cohen, CUNY Queens College
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scf_glm.R
\name{scf_glm}
\alias{scf_glm}
\title{Estimate Generalized Linear Model from SCF Microdata}
\usage{
scf_glm(object, formula, family = binomial())
}
\arguments{
\item{object}{A \code{scf_mi_survey} object, typically created using \code{\link[=scf_load]{scf_load()}} and \code{\link[=scf_design]{scf_design()}}.}

\item{formula}{A valid model formula, e.g., \code{rich ~ age + factor(edcl)}.}

\item{family}{A GLM family object such as \code{\link[=binomial]{binomial()}}, \code{\link[=poisson]{poisson()}}, or \code{\link[=gaussian]{gaussian()}}. Defaults to \code{binomial()}.}
}
\value{
An object of class \code{"scf_glm"} and \code{"scf_model_result"} with:
\describe{
\item{results}{A data frame of pooled coefficients, standard errors, z-values, p-values, and significance stars.}
\item{fit}{A list of fit diagnostics including mean and SD of AIC; for binomial models, pseudo-R2 and its SD.}
\item{models}{A list of implicate-level \code{svyglm} model objects.}
\item{call}{The matched function call.}
}
}
\description{
Estimates generalized linear models (GLMs) with SCF public-use microdata.
Use this function when modeling outcomes that follow non-Gaussian
distributions (e.g., binary or count data). Rubin's Rules are used to combine
implicate-level coefficient and variance estimates.

GLMs are performed across SCF implicates using \code{svyglm()} and returns
pooled coefficients, standard errors, z-values, p-values, and fit diagnostics
including AIC and pseudo-R-Squared when applicable.
}
\section{Implementation}{

This function fits a GLM to each implicate in a \code{scf_mi_survey} object
using \code{survey::svyglm()}. The user specifies a model formula and a valid GLM
family (e.g., \code{binomial()}, \code{poisson()}, \code{gaussian()}). Coefficients and
variance-covariance matrices are extracted from each implicate and pooled
using Rubin's Rules.
}

\section{Details}{

Generalized linear models (GLMs) extend linear regression to accommodate
non-Gaussian outcome distributions. The choice of \code{family} determines the
link function and error distribution. For example:
\itemize{
\item \code{binomial()} fits logistic regression for binary outcomes
\item \code{poisson()} models count data
\item \code{gaussian()} recovers standard OLS behavior
}

Model estimation is performed independently on each implicate using
\code{svyglm()} with replicate weights. Rubin's Rules are used to pool coefficient
estimates and variance matrices. For the pooling procedure, see
\code{\link[=scf_MIcombine]{scf_MIcombine()}}.
}

\section{Internal Suppression}{


For CRAN compliance and to prevent diagnostic overload during package checks,
this function internally wraps each implicate-level model call in \code{suppressWarnings()}.
This suppresses the known benign warning:

\code{"non-integer #successes in a binomial glm!"}

which arises from using replicate weights with \code{family = binomial()}. This suppression
does not affect model validity or inference. Users wishing to inspect warnings can
run \code{survey::svyglm()} directly on individual implicates via \code{model$models[[i]]}.

For further background, see:
https://stackoverflow.com/questions/12953045/warning-non-integer-successes-in-a-binomial-glm-survey-packages
}

\examples{
\donttest{
# Do not implement these lines in real analysis:
# Use functions `scf_download()` and `scf_load()`
td <- tempfile("glm_")
dir.create(td)

src <- system.file("extdata", "scf2022_mock_raw.rds", package = "scf")
file.copy(src, file.path(td, "scf2022.rds"), overwrite = TRUE)
scf2022 <- scf_load(2022, data_directory = td)

# Example for real analysis: Run logistic regression
model <- suppressWarnings(scf_glm(scf2022, own ~ hhsex, family = binomial()))
summary(model)

# Do not implement these lines in real analysis: Cleanup for package check
unlink(td, recursive = TRUE, force = TRUE)
}

}
\seealso{
\code{\link[=scf_ols]{scf_ols()}}, \code{\link[=scf_logit]{scf_logit()}}, \code{\link[=scf_regtable]{scf_regtable()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scf_load.R
\name{scf_load}
\alias{scf_load}
\title{Load SCF Data as Multiply-Imputed Survey Designs}
\usage{
scf_load(min_year, max_year = min_year, data_directory = ".")
}
\arguments{
\item{min_year}{Integer. First SCF year to load (1989–2022, divisible by 3).}

\item{max_year}{Integer. Last SCF year to load. Defaults to \code{min_year}.}

\item{data_directory}{Character. Directory containing \code{.rds} files or a
full path to a single \code{.rds} file. Defaults to the current working directory \code{"."}.
For examples and tests, use \code{tempdir()} to avoid leaving files behind.}
}
\value{
Invisibly returns a \code{scf_mi_survey} (or named list if multiple years).
Attributes: \code{mock} (logical), \code{year}, \code{n_households}.
}
\description{
Converts SCF \code{.rds} files prepared by \code{scf_download()} into \code{scf_mi_survey}
objects. Each object wraps five implicates per year in replicate-weighted,
multiply-imputed survey designs suitable for use with \code{scf_} functions.
}
\section{Implementation}{

Provide a year or range and either (1) a directory containing \verb{scf<year>.rds}
files, or (2) a full path to a single \code{.rds} file. Files must contain five
implicate data frames with columns \code{wgt} and \code{wt1b1..wt1bK} (typically K=999).
}

\examples{
# Using with CRAN-compliant mock data:
# Use functions `scf_download()` and `scf_load()`
td <- tempfile("load_")
dir.create(td)

src <- system.file("extdata", "scf2022_mock_raw.rds", package = "scf")
file.copy(src, file.path(td, "scf2022.rds"), overwrite = TRUE)
scf2022 <- scf_load(2022, data_directory = td)

# Do not implement these lines in real analysis: Cleanup for package check
unlink(td, recursive = TRUE, force = TRUE)

}
\seealso{
\code{\link[=scf_download]{scf_download()}}, \code{\link[=scf_design]{scf_design()}}, \code{\link[=scf_update]{scf_update()}}, \code{\link[survey:svrepdesign]{survey::svrepdesign()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scf_logit.R
\name{scf_logit}
\alias{scf_logit}
\title{Estimate Logistic Regression Model using SCF Microdata}
\usage{
scf_logit(object, formula, odds = TRUE, ...)
}
\arguments{
\item{object}{A \code{scf_mi_survey} object created with \code{\link[=scf_load]{scf_load()}} and \code{\link[=scf_design]{scf_design()}}.}

\item{formula}{A model formula specifying a binary outcome and predictors, e.g., \code{rich ~ age + factor(edcl)}.}

\item{odds}{Logical. If \code{TRUE} (default), exponentiates coefficient estimates to produce odds ratios for interpretability.}

\item{...}{Additional arguments passed to \code{\link[=scf_glm]{scf_glm()}}.}
}
\value{
An object of class \code{"scf_logit"} and \code{"scf_model_result"} with:
\describe{
\item{results}{A data frame of pooled estimates (log-odds or odds ratios), standard errors, and test statistics.}
\item{fit}{Model diagnostics including AIC and pseudo-R-Squared (for binomial family).}
\item{models}{List of implicate-level \code{svyglm} model objects.}
\item{call}{The matched function call.}
}
}
\description{
Fits a replicate-weighted logistic regression model to multiply-imputed SCF data,
returning pooled coefficients or odds ratios with model diagnostics. Use this
function to model a binary variable as a function of predictors.
}
\section{Details}{

This function internally calls \code{scf_glm()} with \code{family = binomial()} and optionally
exponentiates pooled log-odds to odds ratios.

Logistic regression models the probability of a binary outcome using the
logit link.

Coefficients reflect the change in log-odds associated with a one-unit change
in the predictor.

When \code{odds = TRUE}, the coefficient estimates and standard errors are
transformed from log-odds to odds ratios and approximate SEs.
}

\section{Warning}{

When modeling binary outcomes using survey-weighted logistic regression,
users may encounter the warning:

\code{"non-integer #successes in a binomial glm!"}

This message is benign. It results from replicate-weighted survey designs
where the implied number of "successes" is non-integer. The model is
estimated correctly. Coefficients are valid and consistent with
maximum likelihood.

For background, see:
https://stackoverflow.com/questions/12953045/warning-non-integer-successes-in-a-binomial-glm-survey-packages
}

\examples{
\donttest{
# Do not implement these lines in real analysis:
# Use functions `scf_download()` and `scf_load()`
td <- tempfile("logit_")
dir.create(td)

src <- system.file("extdata", "scf2022_mock_raw.rds", package = "scf")
file.copy(src, file.path(td, "scf2022.rds"), overwrite = TRUE)
scf2022 <- scf_load(2022, data_directory = td)

# Example for real analysis: Run logistic regression
model <- suppressWarnings(scf_logit(scf2022, own ~ hhsex))
summary(model)

# Do not implement these lines in real analysis: Cleanup for package check
unlink(td, recursive = TRUE, force = TRUE)
}

}
\seealso{
\code{\link[=scf_glm]{scf_glm()}}, \code{\link[=scf_ols]{scf_ols()}}, \code{\link[=scf_MIcombine]{scf_MIcombine()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scf_quantreg.R
\name{scf_quantreg}
\alias{scf_quantreg}
\title{Estimate a Quantile Regression Model on SCF Microdata}
\usage{
scf_quantreg(
  object,
  formula,
  tau = 0.5,
  se = c("nid", "iid", "ker", "boot", "replicate"),
  ...
)
}
\arguments{
\item{object}{A \code{scf_mi_survey} object created with \code{\link[=scf_load]{scf_load()}} or
\code{\link[=scf_design]{scf_design()}}.}

\item{formula}{A model formula specifying the outcome and predictors,
e.g., \code{networth ~ age + factor(edcl)}. The outcome should be numeric.}

\item{tau}{Numeric scalar in (0, 1) specifying the quantile to estimate.
Defaults to \code{0.5} (median regression). To estimate multiple quantiles,
call \code{scf_quantreg()} separately for each. Analytical SE methods (\code{"nid"},
\code{"iid"}, \code{"ker"}, \code{"boot"}) may be unreliable at quantiles with high mass
points; prefer \code{se = "replicate"} in such cases.}

\item{se}{Character string specifying the standard error estimation method
for within-implicate variance. One of \code{"nid"} (default), \code{"iid"},
\code{"ker"}, \code{"boot"}, or \code{"replicate"}. See the \strong{Standard Error Methods}
section for details. \code{"replicate"} is theoretically preferred for quantile
regression but is computationally intensive.}

\item{...}{Additional arguments passed to \code{\link[quantreg:rq]{quantreg::rq()}}.}
}
\value{
An object of class \code{"scf_quantreg"} and \code{"scf_model_result"} with:
\describe{
\item{\code{results}}{A data frame of pooled coefficients, standard errors,
t-values, p-values, and significance stars.}
\item{\code{tau}}{The quantile estimated.}
\item{\code{se_method}}{The SE method used.}
\item{\code{fit}}{A list of goodness-of-fit statistics. See the
\strong{Goodness of Fit} section for details. Components: \code{rho},
\code{rho_null}, \code{r1}, \code{r1_adj}, \code{nobs}, \code{nobs_mean}.}
\item{\code{models}}{A list of implicate-level \code{rq} model objects for
direct inspection.}
\item{\code{fit_errors}}{Character vector of any implicate-level errors.}
\item{\code{call}}{The matched call.}
\item{\code{formula}}{The model formula.}
}
}
\description{
\code{scf_quantreg()} estimates a linear quantile regression at a user-specified
quantile (\code{tau}) across the five SCF implicates. Each implicate is fit
independently via \code{\link[quantreg:rq]{quantreg::rq()}} using the SCF final sampling weights.
Coefficient estimates and variance-covariance matrices are then pooled across
implicates using \code{\link[=scf_MIcombine]{scf_MIcombine()}}.

Unlike \code{\link[=scf_ols]{scf_ols()}}, which models the conditional mean, quantile regression
models the conditional quantile of the outcome distribution. This makes it
especially useful for analyzing wealth and income data, which are highly
right-skewed: the conditional median (\code{tau = 0.5}) and upper quantiles
(\code{tau = 0.75}, \code{tau = 0.90}) describe the distribution more completely than
the mean alone.
}
\details{
Fits a survey-weighted quantile regression to each implicate of
multiply-imputed SCF data and pools coefficients and standard errors using
Rubin's Rules.
}
\section{Standard Error Methods}{


The \code{se} argument controls how within-implicate variance is estimated.
All methods produce a variance-covariance matrix passed to \code{\link[=scf_MIcombine]{scf_MIcombine()}}.

\describe{
\item{\code{"replicate"} (recommended)}{Replication-based variance estimation.
For each implicate, the model is re-fit using each of the SCF's 999
replicate weight vectors. Variance is accumulated as the weighted sum of
squared deviations from the full-weight estimate, matching the SCF's own
published variance methodology via \code{\link[survey:withReplicates]{survey::withReplicates()}}.
This method is theoretically preferred for quantile regression because
replication-based variance estimators are consistent for sample quantiles,
whereas analytical (sandwich) estimators are not guaranteed consistent for
nonsmooth statistics (Rust and Rao, 1996).
Computationally intensive (~5,000 model fits for five implicates).}
\item{\code{"nid"} (default)}{Non-iid sandwich estimator. Allows the
conditional sparsity (density of the error distribution at the quantile)
to vary across observations. Appropriate when the shape of the error
distribution differs across the covariate space, which is typical for
skewed outcomes such as wealth and income. Unreliable near quantiles
with high mass points (e.g., \code{tau <= 0.25} when net worth has substantial
mass at zero); use \code{se = "replicate"} in such cases.}
\item{\code{"iid"}}{Assumes identically distributed errors, i.e., constant
sparsity across all observations. Implements the covariance formula from
Koenker and Bassett (1978, Theorem 4.2): \eqn{[\theta(1-\theta)/f(\xi(\theta))^2] Q^{-1}},
where \eqn{f(\xi(\theta))} is the density of the error distribution at its
\eqn{\theta}-quantile and \eqn{Q = \lim T^{-1}X'X}. Fastest analytical
option; subject to the same mass-point caveat as \code{"nid"}.}
\item{\code{"ker"}}{Kernel smoothing estimate of the conditional sparsity.
More data-adaptive than \code{"nid"} but slower. Suitable for large samples.}
\item{\code{"boot"}}{Pairs bootstrap over observations. Provides
distribution-free variance estimates but is the slowest analytical
option. Useful for robustness checks.}
}
}

\section{Goodness of Fit}{


The \code{fit} component of the returned object contains per-quantile goodness-of-fit
measures following Koenker and Machado (1999). These are computed by comparing
the full model to an intercept-only null model at the same \code{tau}.

\describe{
\item{\code{rho}}{Mean minimized weighted sum of absolute residuals
(\eqn{\sum \rho_\tau(y_i - x_i'\hat\beta)}) from the full model, averaged
across implicates. This is the quantile regression analog of the residual
sum of squares.}
\item{\code{rho_null}}{Same quantity from the intercept-only null model. Larger
values relative to \code{rho} indicate greater explanatory power.}
\item{\code{r1}}{The Koenker-Machado R\eqn{^1(\tau)} statistic:
\eqn{1 - \bar{V}(\tau) / \tilde{V}(\tau)}, where \eqn{\bar{V}} is the
mean full-model objective and \eqn{\tilde{V}} is the mean null-model
objective, each averaged across implicates. Ranges from 0 to 1. Measures
the proportional reduction in the weighted sum of absolute residuals due
to the covariates at quantile \code{tau}. This is a \emph{local} measure for the
specific quantile estimated; it is not a global summary of fit across
the distribution.}
\item{\code{r1_adj}}{Degrees-of-freedom-adjusted R\eqn{^1(\tau)}:
\eqn{1 - (1 - R^1) \cdot n / (n - p)}, where \eqn{n} is the mean
number of observations and \eqn{p} is the number of estimated parameters.
Penalizes model complexity analogously to adjusted R\eqn{^2} in OLS.
Note: this adjustment is not derived from the asymptotic theory in
Koenker and Machado (1999) and should be interpreted descriptively.}
\item{\code{nobs}}{Integer vector of per-implicate sample sizes.}
\item{\code{nobs_mean}}{Mean sample size across successful implicates.}
}
}

\section{Implementation}{

For each implicate:
\enumerate{
\item Extracts the survey data and final sampling weights from the
\code{svyrep.design} object.
\item Fits \code{\link[quantreg:rq]{quantreg::rq()}} at quantile \code{tau} with the final weights.
\item Estimates the within-implicate variance-covariance matrix using
the method specified by \code{se}.
\item Fits an intercept-only \code{\link[quantreg:rq]{quantreg::rq()}} at the same \code{tau} and
weights to obtain the null-model objective value for goodness-of-fit.
}
SCF public-use microdata contain no missing values; each implicate is a
complete dataset. Pooled estimates follow Rubin's Rules (see
\code{\link[=scf_MIcombine]{scf_MIcombine()}}): the total variance combines within-implicate sampling
uncertainty and between-implicate imputation uncertainty.
}

\examples{
# Do not implement these lines in real analysis:
# Use functions `scf_download()` and `scf_load()`
td <- tempfile("qreg_")
dir.create(td)

src <- system.file("extdata", "scf2022_mock_raw.rds", package = "scf")
file.copy(src, file.path(td, "scf2022.rds"), overwrite = TRUE)
scf2022 <- scf_load(2022, data_directory = td)

# Example for real analysis: median regression of net worth on age and education
m_med <- scf_quantreg(scf2022, networth ~ age + factor(edcl), tau = 0.5)
summary(m_med)

# Access goodness-of-fit statistics
m_med$fit$r1
m_med$fit$r1_adj

# Example for real analysis: 75th-percentile regression
m_75 <- scf_quantreg(scf2022, networth ~ age + factor(edcl), tau = 0.75)
summary(m_75)

# Do not implement these lines in real analysis: Cleanup for package check
unlink(td, recursive = TRUE, force = TRUE)

}
\references{
Koenker R, Bassett G. Regression quantiles. \emph{Econometrica}.
1978;46(1):33--50. \doi{10.2307/1913643}

Koenker R, Machado JAF. Goodness of fit and related inference processes
for quantile regression. \emph{Journal of the American Statistical
Association}. 1999;94(448):1296--1310. \doi{10.2307/2669943}

Rust KF, Rao JNK. Variance estimation for complex surveys using replication
techniques. \emph{Statistical Methods in Medical Research}.
1996;5(3):283--310. \doi{10.1177/096228029600500305}
}
\seealso{
\code{\link[=scf_ols]{scf_ols()}}, \code{\link[=scf_glm]{scf_glm()}}, \code{\link[=scf_MIcombine]{scf_MIcombine()}}, \code{\link[quantreg:rq]{quantreg::rq()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scf_plot_hex.R
\name{scf_plot_hex}
\alias{scf_plot_hex}
\title{Hexbin Plot of Two Continuous SCF Variables}
\usage{
scf_plot_hex(design, x, y, bins = 50, title = NULL, xlab = NULL, ylab = NULL)
}
\arguments{
\item{design}{A \code{scf_mi_survey} object created by \code{\link[=scf_load]{scf_load()}}.}

\item{x}{A one-sided formula for the x-axis variable (e.g., \code{~income}).}

\item{y}{A one-sided formula for the y-axis variable (e.g., \code{~networth}).}

\item{bins}{Integer. Number of hexagonal bins along the x-axis. Default is \code{50}.}

\item{title}{Optional character string for the plot title.}

\item{xlab}{Optional x-axis label. Defaults to the variable name.}

\item{ylab}{Optional y-axis label. Defaults to the variable name.}
}
\value{
A \code{ggplot2} object displaying a Rubin-pooled hexbin plot.
}
\description{
Visualizes the bivariate relationship between two continuous SCF variables
using hexagonal bins.
}
\section{Implementation}{

The function stacks all implicates into one data frame, retains replicate weights,
and uses \code{ggplot2::geom_hex()} to produce a density-style scatterplot. The color
intensity of each hexagon reflects the Rubin-pooled weighted count of households
in that cell. Missing values are excluded.

This plot is especially useful for visualizing joint distributions with large
samples and skewed marginals, such as net worth vs. income.
}

\section{Aesthetic Guidance}{

This plot uses a log-scale fill and \code{viridis} palette to highlight variation
in density. To adjust the visual style globally, use \code{\link[=scf_theme]{scf_theme()}} or set it
explicitly with \code{ggplot2::theme_set(scf_theme())}. For mobile-friendly or
publication-ready appearance, export the plot at 5.5 x 5.5 inches, 300 dpi.
}

\section{Dependencies}{

Requires the \code{ggplot2} package. The fill scale uses \code{scale_fill_viridis_c()} from \code{ggplot2}.
Requires the \pkg{hexbin} package. The function will stop with an error if it is not installed.
}

\examples{
# Do not implement these lines in real analysis:
# Use functions `scf_download()` and `scf_load()`
td <- tempfile("plot_hex_")
dir.create(td)

src <- system.file("extdata", "scf2022_mock_raw.rds", package = "scf")
file.copy(src, file.path(td, "scf2022.rds"), overwrite = TRUE)
scf2022 <- scf_load(2022, data_directory = td)

# Example for real analysis: Plot hexbin of income vs. net worth
# Note: mock data has ~75 rows per implicate; hexbin output will be sparse.
# Results on full SCF data will show a meaningful joint density.
scf_plot_hex(scf2022, ~income, ~networth)

# Do not implement these lines in real analysis: Cleanup for package check
unlink(td, recursive = TRUE, force = TRUE)

}
\seealso{
\code{\link[=scf_corr]{scf_corr()}}, \code{\link[=scf_plot_smooth]{scf_plot_smooth()}}, \code{\link[=scf_theme]{scf_theme()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scf_xtab.R
\name{scf_xtab}
\alias{scf_xtab}
\title{Cross-Tabulate Two Discrete Variables in Multiply-Imputed SCF Data}
\usage{
scf_xtab(scf, rowvar, colvar, scale = "cell")
}
\arguments{
\item{scf}{A \code{scf_mi_survey} object, typically created by \code{\link[=scf_load]{scf_load()}}. Must include five implicates with replicate weights.}

\item{rowvar}{A one-sided formula specifying the row variable (e.g., \code{~edcl}).}

\item{colvar}{A one-sided formula specifying the column variable (e.g., \code{~racecl}).}

\item{scale}{Character. Proportion basis: "cell" (default), "row", or "col".}
}
\value{
A list of class \code{"scf_xtab"} with:
\describe{
\item{results}{Data frame with one row per cell. Columns: \code{row}, \code{col}, \code{prop}, \code{se}, \code{row_share}, \code{col_share}, \code{rowvar}, and \code{colvar}.}
\item{matrices}{List of matrices: \code{cell} (default proportions), \code{row}, \code{col}, and \code{se}.}
\item{imps}{List of implicate-level cell count tables.}
\item{aux}{List with \code{rowvar} and \code{colvar} names.}
}
}
\description{
Computes replicate-weighted two-way cross-tabulations of two discrete variables
using multiply-imputed SCF data. Estimates cell proportions and standard errors,
with optional scaling of proportions by cell, row, or column. Results are pooled
across implicates using Rubin's Rules.
}
\section{Statistical Notes}{

Implicate-level tables are created using \code{svytable()} on replicate-weighted designs.
Proportions are calculated as shares of total population estimates. Variance across
implicates is used to estimate uncertainty. Rubin's Rules are applied in simplified form.

For technical details on pooling logic, see \code{\link[=scf_MIcombine]{scf_MIcombine()}} or the SCF package manual.
}

\examples{
\donttest{
# Do not implement these lines in real analysis:
# Use functions `scf_download()` and `scf_load()`
td <- tempfile("xtab_")
dir.create(td)

src <- system.file("extdata", "scf2022_mock_raw.rds", package = "scf")
file.copy(src, file.path(td, "scf2022.rds"), overwrite = TRUE)
scf2022 <- scf_load(2022, data_directory = td)

# Example for real analysis: Cross-tabulate ownership by sex
suppressWarnings(scf_xtab(scf2022, ~own, ~hhsex, scale = "row"))

# Do not implement these lines in real analysis: Cleanup for package check
unlink(td, recursive = TRUE, force = TRUE)
}

}
